@@ -1,5 +1,5 @@
 #!/bin/bash
-set -e
+set -Eeuo pipefail
+
 bin/update_crates.sh --check
 
-
