@@ -1,7 +1,7 @@
 #!/bin/bash
 #!/bin/bash
 set -Eeuo pipefail
-for hurl_file in integration/hurl/{tests_failed,tests_ok}/*.hurl; do
+for hurl_file in integration/hurl/{tests_failed,tests_ok/**}/*.hurl; do
     echo "hurlfmt $hurl_file"
     output_file=/tmp/$(basename "$hurl_file")
     hurlfmt "$hurl_file" >"$output_file"
