@@ -1,5 +1,7 @@
 #!/bin/bash
-set -e
+set -Eeuo pipefail
+
 cargo install grcov
 rustup component add llvm-tools-preview
 cargo clean
+
