@@ -21,5 +21,9 @@ apk add --quiet \
     cargo \
     squid \
     jq
+
+# libxml crate >= 0.3.4 uses bindgen
+apk add --quiet clang-dev
+
 sudo squid -k shutdown || true
 sudo rm -v /dev/shm/squid*.shm >/dev/null 2>&1 || true
