@@ -4,9 +4,13 @@ set -Eeuo pipefail
 echo "----- install generic macos package -----"
 
 # install
-generic_macos_package=$(ls target/upload/hurl-*-x86_64-macos.tar.gz)
+toolchain=$(bin/release/get_active_toolchain.sh)
+echo "toolchain=${toolchain}"
+generic_macos_package=$(ls target/upload/hurl-*-"${toolchain}".tar.gz)
+echo "generic_macos_package=${generic_macos_package}"
 
 install_dir="/tmp/hurl-generic-macos"
+echo "install_dir=${install_dir}"
 mkdir -p "${install_dir}"
 tar xvf "${generic_macos_package}" -C "${install_dir}" --strip-components=1
 
