@@ -1,5 +1,5 @@
-#!/bin/sh
-set -e
+#!/bin/bash
+set -Eeuo pipefail
 
 rm -rf target/man
 mkdir -p target/man
@@ -9,4 +9,3 @@ bin/release/gen_manpage.py docs/manual/hurlfmt.md > target/man/hurlfmt.1
 gzip -n -9 target/man/hurl.1
 gzip -n -9 target/man/hurlfmt.1
 
-
