@@ -6,14 +6,15 @@ tar pxf vendor.tar.xz
 
 echo "## install rustc and cargo"
 rust_version=$(grep '^rust-version' packages/hurl/Cargo.toml | cut -f2 -d'"')
-rust_arch=$(uname -m)
-package="rust-${rust_version}-${rust_arch}-unknown-linux-gnu"
+arch=$(uname -m)
+package="rust-${rust_version}-${arch}-unknown-linux-gnu"
+packagelight="${package}-light"
 echo "rust_version=${rust_version}"
-echo "rust_architecture=${rust_arch}"
-echo "package=${package}"
-tar xf "${package}.tar.gz"
+echo "architecture=${arch}"
+echo "packagelight=${packagelight}"
+tar xf "${packagelight}.tar.xz"
 mkdir -p /tmp/rust
-./"${package}"/install.sh --verbose --destdir=/tmp/rust --disable-ldconfig
+./"${packagelight}"/install.sh --verbose --destdir=/tmp/rust --disable-ldconfig
 export PATH="/tmp/rust/usr/local/bin:$PATH"
 which rustc
 which cargo
