@@ -1,7 +1,8 @@
-from app import app
-from flask import make_response, request
 from io import BytesIO
 
+from app import app
+from flask import make_response
+
 
 @app.route("/error-assert-bytearray")
 def error_assert_bytearray():
