@@ -1,5 +1,5 @@
 #!/bin/bash
 set -Eeuo pipefail
 
-hurl --toto test_ok/hello.hurl
+hurl --toto test_ok/hello/hello.hurl
 
