@@ -2,4 +2,4 @@
 set -Eeuo pipefail
 mkdir -p build/tmp
 
-hurl --output build/tmp tests_ok/hello.hurl
+hurl --output build/tmp tests_ok/hello/hello.hurl
