@@ -4,11 +4,13 @@
 #
 import codecs
 import os
-import sys
 import subprocess
+import sys
+from typing import Literal
 
 
-def decode_string(encoded):
+def decode_string(encoded: bytes) -> str:
+    """Decodes bytes by guessing the encoding and returns a string."""
     if encoded.startswith(codecs.BOM_UTF8):
         return encoded.decode("utf-8-sig")
     elif encoded.startswith(codecs.BOM_UTF16):
@@ -18,7 +20,13 @@ def decode_string(encoded):
         return encoded.decode()
 
 
-def test(format_type, hurl_file):
+def test(format_type: Literal["hurl", "html", "json"], hurl_file: str):
+    """
+    Exports a Hurl file to different format:
+    - format_type == "hurl": input file is linted and compared to `foo.lint.hurl`
+    - format_type == "html": input file is exported to HTML and compared to `foo.html`
+    - format_type == "json": input file is exported to JSON and compared to `foo.json`
+    """
     extension = ".lint.hurl" if format_type == "hurl" else ("." + format_type)
     output_file = hurl_file.replace(".hurl", extension)
     if not os.path.exists(output_file):
@@ -36,7 +44,7 @@ def test(format_type, hurl_file):
 
 def main():
     if len(sys.argv) < 2:
-        print("usage: test_format.py json|html HURL_FILE..")
+        print("usage: test_format.py hurl|json|html HURL_FILE..")
         sys.exit(1)
     format_type = sys.argv[1]
 
