@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,9 +27,9 @@ use hurl::runner::Value;
 use hurl_core::input::Input;
 use hurl_core::typing::{BytesPerSec, Count, DurationUnit};
 
-use crate::cli::options::variables::{parse as parse_variable, parse_value};
-use crate::cli::options::{duration, CliOptionsError};
-use crate::cli::options::{ErrorFormat, HttpVersion, IpResolve, Output};
+use crate::cli::options::{
+    duration, variables, CliOptionsError, ErrorFormat, HttpVersion, IpResolve, Output,
+};
 use crate::cli::OutputType;
 
 pub fn cacert_file(arg_matches: &ArgMatches) -> Result<Option<String>, CliOptionsError> {
@@ -82,19 +82,17 @@ pub fn client_key_file(arg_matches: &ArgMatches) -> Result<Option<String>, CliOp
 }
 
 /// Returns true if Hurl output uses ANSI code and false otherwise.
-pub fn color(arg_matches: &ArgMatches) -> bool {
+///
+/// `allow_color_from_env` is false when the environment disallow usage of ANSI color (NO_COLOR en variable,
+/// or not tty context).
+pub fn color(arg_matches: &ArgMatches, allow_color_from_env: bool) -> bool {
     if has_flag(arg_matches, "color") {
         return true;
     }
     if has_flag(arg_matches, "no_color") {
         return false;
     }
-    if let Ok(v) = env::var("NO_COLOR") {
-        if !v.is_empty() {
-            return false;
-        }
-    }
-    io::stdout().is_terminal()
+    allow_color_from_env
 }
 
 pub fn compressed(arg_matches: &ArgMatches) -> bool {
@@ -102,7 +100,7 @@ pub fn compressed(arg_matches: &ArgMatches) -> bool {
 }
 
 pub fn connect_timeout(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
-    let s = get::<String>(arg_matches, "connect_timeout").unwrap_or_default();
+    let s = get::<String>(arg_matches, "connect_timeout").unwrap_or("300".to_string());
     get_duration(&s, DurationUnit::Second)
 }
 
@@ -127,15 +125,15 @@ pub fn curl_file(arg_matches: &ArgMatches) -> Option<PathBuf> {
 }
 
 pub fn delay(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
-    let s = get::<String>(arg_matches, "delay").unwrap_or_default();
+    let s = get::<String>(arg_matches, "delay").unwrap_or("0".to_string());
     get_duration(&s, DurationUnit::MilliSecond)
 }
 
 pub fn error_format(arg_matches: &ArgMatches) -> ErrorFormat {
-    let error_format = get::<String>(arg_matches, "error_format");
-    match error_format.as_deref() {
-        Some("long") => ErrorFormat::Long,
-        Some("short") => ErrorFormat::Short,
+    let error_format = get::<String>(arg_matches, "error_format").unwrap_or("short".to_string());
+    match error_format.as_str() {
+        "long" => ErrorFormat::Long,
+        "short" => ErrorFormat::Short,
         _ => ErrorFormat::Short,
     }
 }
@@ -155,6 +153,10 @@ pub fn from_entry(arg_matches: &ArgMatches) -> Option<usize> {
     get::<u32>(arg_matches, "from_entry").map(|x| x as usize)
 }
 
+pub fn headers(arg_matches: &ArgMatches) -> Vec<String> {
+    get_strings(arg_matches, "header").unwrap_or_default()
+}
+
 pub fn html_dir(arg_matches: &ArgMatches) -> Result<Option<PathBuf>, CliOptionsError> {
     if let Some(dir) = get::<String>(arg_matches, "report_html") {
         let path = Path::new(&dir);
@@ -289,7 +291,7 @@ pub fn max_filesize(arg_matches: &ArgMatches) -> Option<u64> {
 }
 
 pub fn max_redirect(arg_matches: &ArgMatches) -> Count {
-    match get::<i32>(arg_matches, "max_redirects").unwrap() {
+    match get::<i32>(arg_matches, "max_redirects").unwrap_or(50) {
         -1 => Count::Infinite,
         m => Count::Finite(m as usize),
     }
@@ -323,6 +325,10 @@ pub fn json_report_dir(arg_matches: &ArgMatches) -> Result<Option<PathBuf>, CliO
     }
 }
 
+pub fn negotiate(arg_matches: &ArgMatches) -> bool {
+    has_flag(arg_matches, "negotiate")
+}
+
 pub fn netrc(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "netrc")
 }
@@ -349,6 +355,10 @@ pub fn no_proxy(arg_matches: &ArgMatches) -> Option<String> {
     get::<String>(arg_matches, "noproxy")
 }
 
+pub fn ntlm(arg_matches: &ArgMatches) -> bool {
+    has_flag(arg_matches, "ntlm")
+}
+
 pub fn output(arg_matches: &ArgMatches) -> Option<Output> {
     get::<String>(arg_matches, "output").map(|filename| Output::new(&filename))
 }
@@ -371,8 +381,20 @@ pub fn path_as_is(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "path_as_is")
 }
 
+pub fn pinned_pub_key(arg_matches: &ArgMatches) -> Option<String> {
+    get::<String>(arg_matches, "pinned_pub_key")
+}
+
 pub fn progress_bar(arg_matches: &ArgMatches) -> bool {
-    test(arg_matches) && !interactive(arg_matches) && !is_ci() && io::stderr().is_terminal()
+    // The test progress bar is displayed only for in test mode, for interactive TTYs.
+    // It can be forced by `--progress-bar` option.
+    if !test(arg_matches) {
+        return false;
+    }
+    if has_flag(arg_matches, "progress_bar") {
+        return true;
+    }
+    io::stderr().is_terminal() && !is_ci()
 }
 
 pub fn proxy(arg_matches: &ArgMatches) -> Option<String> {
@@ -400,10 +422,32 @@ pub fn retry(arg_matches: &ArgMatches) -> Option<Count> {
 }
 
 pub fn retry_interval(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
-    let s = get::<String>(arg_matches, "retry_interval").unwrap_or_default();
+    let s = get::<String>(arg_matches, "retry_interval").unwrap_or("1000".to_string());
     get_duration(&s, DurationUnit::MilliSecond)
 }
 
+pub fn secret(matches: &ArgMatches) -> Result<HashMap<String, String>, CliOptionsError> {
+    let mut secrets = HashMap::new();
+    if let Some(secret) = get_strings(matches, "secret") {
+        for s in secret {
+            let inferred = false;
+            let (name, value) = variables::parse(&s, inferred)?;
+            // We check that there is no existing secrets
+            if secrets.contains_key(&name) {
+                return Err(CliOptionsError::Error(format!(
+                    "secret '{}' can't be reassigned",
+                    &name
+                )));
+            }
+            // Secrets can only be string.
+            if let Value::String(value) = value {
+                secrets.insert(name, value);
+            }
+        }
+    }
+    Ok(secrets)
+}
+
 pub fn ssl_no_revoke(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "ssl_no_revoke")
 }
@@ -417,7 +461,7 @@ pub fn test(arg_matches: &ArgMatches) -> bool {
 }
 
 pub fn timeout(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
-    let s = get::<String>(arg_matches, "max_time").unwrap_or_default();
+    let s = get::<String>(arg_matches, "max_time").unwrap_or("300".to_string());
     get_duration(&s, DurationUnit::Second)
 }
 
@@ -444,7 +488,8 @@ pub fn variables(matches: &ArgMatches) -> Result<HashMap<String, Value>, CliOpti
     // Use environment variables prefix by HURL_
     for (env_name, env_value) in env::vars() {
         if let Some(name) = env_name.strip_prefix("HURL_") {
-            let value = parse_value(env_value.as_str())?;
+            let inferred = true;
+            let value = variables::parse_value(env_value.as_str(), inferred)?;
             variables.insert(name.to_string(), value);
         }
     }
@@ -476,7 +521,8 @@ pub fn variables(matches: &ArgMatches) -> Result<HashMap<String, Value>, CliOpti
                 if line.starts_with('#') || line.is_empty() {
                     continue;
                 }
-                let (name, value) = parse_variable(line)?;
+                let inferred = true;
+                let (name, value) = variables::parse(line, inferred)?;
                 variables.insert(name.to_string(), value);
             }
         }
@@ -484,7 +530,8 @@ pub fn variables(matches: &ArgMatches) -> Result<HashMap<String, Value>, CliOpti
 
     if let Some(input) = get_strings(matches, "variable") {
         for s in input {
-            let (name, value) = parse_variable(&s)?;
+            let inferred = true;
+            let (name, value) = variables::parse(&s, inferred)?;
             variables.insert(name.to_string(), value);
         }
     }
@@ -558,14 +605,14 @@ fn get_duration(s: &str, default_unit: DurationUnit) -> Result<Duration, CliOpti
     let duration = duration::parse(s).map_err(CliOptionsError::Error)?;
     let unit = duration.unit.unwrap_or(default_unit);
     let millis = match unit {
-        DurationUnit::MilliSecond => duration.value,
-        DurationUnit::Second => duration.value * 1000,
-        DurationUnit::Minute => duration.value * 1000 * 60,
+        DurationUnit::MilliSecond => duration.value.as_u64(),
+        DurationUnit::Second => duration.value.as_u64() * 1000,
+        DurationUnit::Minute => duration.value.as_u64() * 1000 * 60,
     };
     Ok(Duration::from_millis(millis))
 }
 
-/// Whether or not this running in a Continuous Integration environment.
+/// Whether this running in a Continuous Integration environment.
 /// Code borrowed from <https://github.com/rust-lang/cargo/blob/master/crates/cargo-util/src/lib.rs>
 fn is_ci() -> bool {
     env::var("CI").is_ok() || env::var("TF_BUILD").is_ok()
