@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,6 +26,8 @@ use std::env;
 use std::path::{Path, PathBuf};
 use std::time::Duration;
 
+use clap::builder::styling::{AnsiColor, Effects};
+use clap::builder::Styles;
 use clap::ArgMatches;
 pub use error::CliOptionsError;
 use hurl::http;
@@ -61,6 +63,7 @@ pub struct CliOptions {
     pub follow_location: bool,
     pub follow_location_trusted: bool,
     pub from_entry: Option<usize>,
+    pub headers: Vec<String>,
     pub html_dir: Option<PathBuf>,
     pub http_version: Option<HttpVersion>,
     pub ignore_asserts: bool,
@@ -75,20 +78,24 @@ pub struct CliOptions {
     pub limit_rate: Option<BytesPerSec>,
     pub max_filesize: Option<u64>,
     pub max_redirect: Count,
+    pub negotiate: bool,
     pub netrc: bool,
     pub netrc_file: Option<String>,
     pub netrc_optional: bool,
     pub no_proxy: Option<String>,
+    pub ntlm: bool,
     pub output: Option<Output>,
     pub output_type: OutputType,
     pub parallel: bool,
     pub path_as_is: bool,
+    pub pinned_pub_key: Option<String>,
     pub progress_bar: bool,
     pub proxy: Option<String>,
     pub repeat: Option<Count>,
     pub resolves: Vec<String>,
     pub retry: Option<Count>,
     pub retry_interval: Duration,
+    pub secrets: HashMap<String, String>,
     pub ssl_no_revoke: bool,
     pub tap_file: Option<PathBuf>,
     pub test: bool,
@@ -156,19 +163,31 @@ impl From<IpResolve> for http::IpResolve {
 
 fn get_version() -> String {
     let libcurl_version = http::libcurl_version_info();
+    let pkg_version = env!("CARGO_PKG_VERSION");
     format!(
         "{} ({}) {}\nFeatures (libcurl):  {}\nFeatures (built-in): brotli",
-        clap::crate_version!(),
+        pkg_version,
         libcurl_version.host,
         libcurl_version.libraries.join(" "),
         libcurl_version.features.join(" ")
     )
 }
 
-pub fn parse() -> Result<CliOptions, CliOptionsError> {
+/// Parse the Hurl CLI options and returns a [`CliOptions`] result.
+///
+/// When a [`CliOptionsError::DisplayHelp`] variant is returned, `allow_color` is used
+/// to print an ANSI color help or not.
+pub fn parse(allow_color: bool) -> Result<CliOptions, CliOptionsError> {
+    let styles = Styles::styled()
+        .header(AnsiColor::Green.on_default() | Effects::BOLD)
+        .usage(AnsiColor::Green.on_default() | Effects::BOLD)
+        .literal(AnsiColor::Cyan.on_default() | Effects::BOLD)
+        .placeholder(AnsiColor::Cyan.on_default());
+
     let mut command = clap::Command::new("hurl")
         .version(get_version())
         .disable_colored_help(true)
+        .styles(styles)
         .about("Hurl, run and test HTTP requests with plain text")
         // HTTP options
         .arg(commands::aws_sigv4())
@@ -177,6 +196,7 @@ pub fn parse() -> Result<CliOptions, CliOptionsError> {
         .arg(commands::compressed())
         .arg(commands::connect_timeout())
         .arg(commands::connect_to())
+        .arg(commands::header())
         .arg(commands::http10())
         .arg(commands::http11())
         .arg(commands::http2())
@@ -192,8 +212,11 @@ pub fn parse() -> Result<CliOptions, CliOptionsError> {
         .arg(commands::max_filesize())
         .arg(commands::max_redirects())
         .arg(commands::max_time())
+        .arg(commands::negotiate())
         .arg(commands::noproxy())
+        .arg(commands::ntlm())
         .arg(commands::path_as_is())
+        .arg(commands::pinned_pub_key())
         .arg(commands::proxy())
         .arg(commands::resolve())
         .arg(commands::ssl_no_revoke())
@@ -209,6 +232,7 @@ pub fn parse() -> Result<CliOptions, CliOptionsError> {
         .arg(commands::no_color())
         .arg(commands::no_output())
         .arg(commands::output())
+        .arg(commands::progress_bar())
         .arg(commands::verbose())
         .arg(commands::very_verbose())
         // Run options
@@ -222,6 +246,7 @@ pub fn parse() -> Result<CliOptions, CliOptionsError> {
         .arg(commands::repeat())
         .arg(commands::retry())
         .arg(commands::retry_interval())
+        .arg(commands::secret())
         .arg(commands::test())
         .arg(commands::to_entry())
         .arg(commands::variable())
@@ -240,36 +265,42 @@ pub fn parse() -> Result<CliOptions, CliOptionsError> {
         .arg(commands::netrc_file())
         .arg(commands::netrc_optional());
 
-    let arg_matches = command.try_get_matches_from_mut(env::args_os())?;
+    let arg_matches = command.try_get_matches_from_mut(env::args_os());
+    let arg_matches = match arg_matches {
+        Ok(args) => args,
+        Err(error) => return Err(CliOptionsError::from_clap(error, allow_color)),
+    };
 
     // If we've no file input (either from the standard input or from the command line arguments),
     // we just print help and exit.
     if !matches::has_input_files(&arg_matches) {
-        let help = command.render_help().to_string();
+        let help = if allow_color {
+            command.render_help().ansi().to_string()
+        } else {
+            command.render_help().to_string()
+        };
         return Err(CliOptionsError::NoInput(help));
     }
 
-    let opts = parse_matches(&arg_matches)?;
+    let opts = parse_matches(&arg_matches, allow_color)?;
     if opts.input_files.is_empty() {
         return Err(CliOptionsError::Error(
             "No input files provided".to_string(),
         ));
     }
 
-    if opts.cookie_output_file.is_some() && opts.input_files.len() > 1 {
-        return Err(CliOptionsError::Error(
-            "Only save cookies for a unique session".to_string(),
-        ));
-    }
     Ok(opts)
 }
 
-fn parse_matches(arg_matches: &ArgMatches) -> Result<CliOptions, CliOptionsError> {
+fn parse_matches(
+    arg_matches: &ArgMatches,
+    allow_color: bool,
+) -> Result<CliOptions, CliOptionsError> {
     let aws_sigv4 = matches::aws_sigv4(arg_matches);
     let cacert_file = matches::cacert_file(arg_matches)?;
     let client_cert_file = matches::client_cert_file(arg_matches)?;
     let client_key_file = matches::client_key_file(arg_matches)?;
-    let color = matches::color(arg_matches);
+    let color = matches::color(arg_matches, allow_color);
     let compressed = matches::compressed(arg_matches);
     let connect_timeout = matches::connect_timeout(arg_matches)?;
     let connects_to = matches::connects_to(arg_matches);
@@ -282,6 +313,7 @@ fn parse_matches(arg_matches: &ArgMatches) -> Result<CliOptions, CliOptionsError
     let file_root = matches::file_root(arg_matches);
     let (follow_location, follow_location_trusted) = matches::follow_location(arg_matches);
     let from_entry = matches::from_entry(arg_matches);
+    let headers = matches::headers(arg_matches);
     let html_dir = matches::html_dir(arg_matches)?;
     let http_version = matches::http_version(arg_matches);
     let ignore_asserts = matches::ignore_asserts(arg_matches);
@@ -290,17 +322,21 @@ fn parse_matches(arg_matches: &ArgMatches) -> Result<CliOptions, CliOptionsError
     let insecure = matches::insecure(arg_matches);
     let interactive = matches::interactive(arg_matches);
     let ip_resolve = matches::ip_resolve(arg_matches);
+    let jobs = matches::jobs(arg_matches);
     let json_report_dir = matches::json_report_dir(arg_matches)?;
     let junit_file = matches::junit_file(arg_matches);
     let limit_rate = matches::limit_rate(arg_matches);
     let max_filesize = matches::max_filesize(arg_matches);
     let max_redirect = matches::max_redirect(arg_matches);
+    let negotiate = matches::negotiate(arg_matches);
     let netrc = matches::netrc(arg_matches);
     let netrc_file = matches::netrc_file(arg_matches)?;
     let netrc_optional = matches::netrc_optional(arg_matches);
     let no_proxy = matches::no_proxy(arg_matches);
+    let ntlm = matches::ntlm(arg_matches);
     let parallel = matches::parallel(arg_matches);
     let path_as_is = matches::path_as_is(arg_matches);
+    let pinned_pub_key = matches::pinned_pub_key(arg_matches);
     let progress_bar = matches::progress_bar(arg_matches);
     let proxy = matches::proxy(arg_matches);
     let output = matches::output(arg_matches);
@@ -309,6 +345,7 @@ fn parse_matches(arg_matches: &ArgMatches) -> Result<CliOptions, CliOptionsError
     let resolves = matches::resolves(arg_matches);
     let retry = matches::retry(arg_matches);
     let retry_interval = matches::retry_interval(arg_matches)?;
+    let secrets = matches::secret(arg_matches)?;
     let ssl_no_revoke = matches::ssl_no_revoke(arg_matches);
     let tap_file = matches::tap_file(arg_matches);
     let test = matches::test(arg_matches);
@@ -320,7 +357,6 @@ fn parse_matches(arg_matches: &ArgMatches) -> Result<CliOptions, CliOptionsError
     let variables = matches::variables(arg_matches)?;
     let verbose = matches::verbose(arg_matches);
     let very_verbose = matches::very_verbose(arg_matches);
-    let jobs = matches::jobs(arg_matches);
     Ok(CliOptions {
         aws_sigv4,
         cacert_file,
@@ -340,6 +376,7 @@ fn parse_matches(arg_matches: &ArgMatches) -> Result<CliOptions, CliOptionsError
         follow_location,
         follow_location_trusted,
         from_entry,
+        headers,
         html_dir,
         http_version,
         ignore_asserts,
@@ -353,11 +390,14 @@ fn parse_matches(arg_matches: &ArgMatches) -> Result<CliOptions, CliOptionsError
         limit_rate,
         max_filesize,
         max_redirect,
+        negotiate,
         netrc,
         netrc_file,
         netrc_optional,
         no_proxy,
+        ntlm,
         path_as_is,
+        pinned_pub_key,
         parallel,
         progress_bar,
         proxy,
@@ -367,6 +407,7 @@ fn parse_matches(arg_matches: &ArgMatches) -> Result<CliOptions, CliOptionsError
         resolves,
         retry,
         retry_interval,
+        secrets,
         ssl_no_revoke,
         tap_file,
         test,
@@ -393,6 +434,7 @@ pub enum OutputType {
 }
 
 impl CliOptions {
+    /// Converts this instance of [`CliOptions`] to an instance of [`RunnerOptions`]
     pub fn to_runner_options(&self, filename: &Input, current_dir: &Path) -> RunnerOptions {
         let aws_sigv4 = self.aws_sigv4.clone();
         let cacert_file = self.cacert_file.clone();
@@ -415,6 +457,7 @@ impl CliOptions {
         let follow_location = self.follow_location;
         let follow_location_trusted = self.follow_location_trusted;
         let from_entry = self.from_entry;
+        let headers = &self.headers;
         let http_version = match self.http_version {
             Some(version) => version.into(),
             None => RequestedHttpVersion::default(),
@@ -437,6 +480,7 @@ impl CliOptions {
         let no_proxy = self.no_proxy.clone();
         let output = self.output.clone();
         let path_as_is = self.path_as_is;
+        let pinned_pub_key = self.pinned_pub_key.clone();
         let post_entry = if self.interactive {
             Some(cli::interactive::post_entry as fn() -> bool)
         } else {
@@ -452,6 +496,8 @@ impl CliOptions {
         let retry = self.retry;
         let retry_interval = self.retry_interval;
         let ssl_no_revoke = self.ssl_no_revoke;
+        let negotiate = self.negotiate;
+        let ntlm = self.ntlm;
         let timeout = self.timeout;
         let to_entry = self.to_entry;
         let unix_socket = self.unix_socket.clone();
@@ -473,6 +519,7 @@ impl CliOptions {
             .follow_location(follow_location)
             .follow_location_trusted(follow_location_trusted)
             .from_entry(from_entry)
+            .headers(headers)
             .http_version(http_version)
             .ignore_asserts(ignore_asserts)
             .insecure(insecure)
@@ -481,12 +528,15 @@ impl CliOptions {
             .max_recv_speed(max_recv_speed)
             .max_redirect(max_redirect)
             .max_send_speed(max_send_speed)
+            .negotiate(negotiate)
             .netrc(netrc)
             .netrc_file(netrc_file)
             .netrc_optional(netrc_optional)
             .no_proxy(no_proxy)
+            .ntlm(ntlm)
             .output(output)
             .path_as_is(path_as_is)
+            .pinned_pub_key(pinned_pub_key)
             .post_entry(post_entry)
             .pre_entry(pre_entry)
             .proxy(proxy)
@@ -502,6 +552,7 @@ impl CliOptions {
             .build()
     }
 
+    /// Converts this instance of [`ClipOptions`] to an instance of [`LoggerOptions`]
     pub fn to_logger_options(&self) -> LoggerOptions {
         let verbosity = Verbosity::from(self.verbose, self.very_verbose);
         LoggerOptionsBuilder::new()
