@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,13 +33,15 @@ use crate::http::certificate::Certificate;
 use crate::http::curl_cmd::CurlCmd;
 use crate::http::debug::log_body;
 use crate::http::header::{
-    HeaderVec, ACCEPT_ENCODING, AUTHORIZATION, CONTENT_TYPE, EXPECT, LOCATION, USER_AGENT,
+    HeaderVec, ACCEPT_ENCODING, AUTHORIZATION, CONTENT_TYPE, EXPECT, LOCATION, SET_COOKIE,
+    USER_AGENT,
 };
+use crate::http::ip::IpAddr;
 use crate::http::options::ClientOptions;
 use crate::http::timings::Timings;
 use crate::http::url::Url;
 use crate::http::{
-    easy_ext, Call, Cookie, FileParam, Header, HttpError, HttpVersion, IpResolve, Method,
+    easy_ext, Body, Call, Cookie, FileParam, Header, HttpError, HttpVersion, IpResolve, Method,
     MultipartParam, Param, Request, RequestCookie, RequestSpec, RequestedHttpVersion, Response,
     Verbosity,
 };
@@ -55,8 +57,6 @@ use crate::util::path::ContextDir;
 pub struct Client {
     /// The handle to libcurl binding
     handle: easy::Easy,
-    /// Current State
-    state: ClientState,
     /// HTTP version support
     http2: bool,
     http3: bool,
@@ -64,31 +64,6 @@ pub struct Client {
     certificates: HashMap<i64, Certificate>,
 }
 
-/// Represents the state of the HTTP client.
-/// We only keep the last requested HTTP version because it's the only property which change
-/// must trigger a connection reset on the libcurl handle. The state can be queried to check
-/// if there has been change from the previous HTTP request.
-#[derive(Copy, Clone, Eq, PartialEq, Default, Debug)]
-struct ClientState {
-    changed: bool,
-    requested_http_version: Option<RequestedHttpVersion>,
-}
-
-impl ClientState {
-    /// Set a new requested HTTP version.
-    pub fn set_requested_http_version(&mut self, version: RequestedHttpVersion) {
-        if let Some(prev_version) = self.requested_http_version {
-            self.changed = prev_version != version;
-        }
-        self.requested_http_version = Some(version);
-    }
-
-    /// Returns true if state has changed from the previous request.
-    pub fn has_changed(&self) -> bool {
-        self.changed
-    }
-}
-
 impl Client {
     /// Creates HTTP Hurl client.
     pub fn new() -> Client {
@@ -96,7 +71,6 @@ impl Client {
         let version = Version::get();
         Client {
             handle,
-            state: ClientState::default(),
             http2: version.feature_http2(),
             http3: version.feature_http3(),
             certificates: HashMap::new(),
@@ -144,20 +118,42 @@ impl Client {
                 }
             };
 
-            let redirect_method = redirect_method(status, request_spec.method);
+            let redirect_method = redirect_method(status, &request_spec.method);
             let mut headers = request_spec.headers;
 
-            // When following redirection, we filter `AUTHORIZATION` header unless explicitly told
-            // to trust the redirected host with `--location-trusted`.
+            // When following redirection, we filter `Authorization` and `Set-Cookie` headers if the
+            // hostname changes unless the user explicitly trusts the redirected host with `--location-trusted`.
+            // <https://curl.se/libcurl/c/CURLOPT_FOLLOWLOCATION.html>:
+            //
+            // > By default, libcurl only sends Authentication: or explicitly set Cookie: headers
+            // > to the initial host given in the original URL, to avoid leaking username + password
+            // > to other sites.
             let host_changed = request_url.host() != redirect_url.host();
             if host_changed && !options.follow_location_trusted {
                 headers.retain(|h| !h.name_eq(AUTHORIZATION));
+                headers.retain(|h| !h.name_eq(SET_COOKIE));
                 options.user = None;
             }
+
+            // If the request method has changed due to redirection, the body is dropped from the
+            // request, otherwise we keep it. We follow libcurl implementation <https://curl.se/libcurl/c/CURLOPT_FOLLOWLOCATION.html>:
+            //
+            // > When libcurl switches method to GET, it then uses that method without sending any
+            // > request body. If it does not change the method, it sends the subsequent request the
+            // > same way as the previous one; including the request body if one was provided.
+            let (form, multipart, body) = if redirect_method != request_spec.method {
+                (vec![], vec![], Body::Binary(vec![]))
+            } else {
+                (request_spec.form, request_spec.multipart, request_spec.body)
+            };
             request_spec = RequestSpec {
                 method: redirect_method,
                 url: redirect_url,
                 headers,
+                form,
+                multipart,
+                body,
+                cookies: request_spec.cookies,
                 ..Default::default()
             };
         }
@@ -183,7 +179,7 @@ impl Client {
         let verbose = options.verbosity.is_some();
         let very_verbose = options.verbosity == Some(Verbosity::VeryVerbose);
         let mut request_headers = HeaderVec::new();
-        let mut status_lines = vec![];
+        let mut status_lines = None;
         let mut response_headers = vec![];
         let has_body_data = !request_spec.body.bytes().is_empty()
             || !request_spec.form.is_empty()
@@ -254,7 +250,7 @@ impl Client {
             transfer.header_function(|h| {
                 if let Some(s) = decode_header(h) {
                     if s.starts_with("HTTP/") {
-                        status_lines.push(s);
+                        status_lines = Some(s);
                     } else {
                         response_headers.push(s);
                     }
@@ -291,8 +287,7 @@ impl Client {
         }
 
         let status = self.handle.response_code()?;
-        // TODO: explain why status_lines is Vec ?
-        let version = match status_lines.last() {
+        let version = match &status_lines {
             Some(status_line) => self.parse_response_version(status_line)?,
             None => return Err(HttpError::CouldNotParseResponse),
         };
@@ -305,6 +300,7 @@ impl Client {
         let timings = Timings::new(&mut self.handle, start_dt, stop_dt);
 
         let url = Url::from_str(&url)?;
+        let ip_addr = self.primary_ip()?;
         let request = Request::new(
             &method.to_string(),
             url.clone(),
@@ -319,6 +315,7 @@ impl Client {
             duration,
             url,
             certificate,
+            ip_addr,
         );
 
         if verbose {
@@ -391,21 +388,11 @@ impl Client {
             return Err(HttpError::UnsupportedHttpVersion(http_version));
         }
 
-        // libcurl tries to reuse connections as much as possible (see <https://curl.se/libcurl/c/CURLOPT_HTTP_VERSION.html>)
-        // That's why an `handle` initiated with a HTTP 2 version may keep using HTTP 2 protocol
-        // even if we ask to switch to HTTP 3 in the same session (using `[Options]` section for
-        // instance).
-        // > Note that the HTTP version is just a request. libcurl still prioritizes to reuse
-        // > existing connections so it might then reuse a connection using a HTTP version you
-        // > have not asked for.
-        //
-        // So, if we detect a change of requested HTTP version, we force libcurl to refresh its
-        // connections (see <https://curl.se/libcurl/c/CURLOPT_FRESH_CONNECT.html>)
-        self.state.set_requested_http_version(http_version);
-        if self.state.has_changed() {
+        if !options.allow_reuse {
             logger.debug("Force refreshing connections because requested HTTP version change");
-            self.handle.fresh_connect(true)?;
         }
+        self.handle.fresh_connect(!options.allow_reuse)?;
+        self.handle.forbid_reuse(!options.allow_reuse)?;
         self.handle.http_version(options.http_version.into())?;
 
         self.handle.ip_resolve(options.ip_resolve.into())?;
@@ -476,6 +463,19 @@ impl Client {
         if let Some(max_send_speed) = options.max_send_speed {
             self.handle.max_send_speed(max_send_speed.0)?;
         }
+        if let Some(pinned_pub_key) = &options.pinned_pub_key {
+            self.handle.pinned_public_key(pinned_pub_key)?;
+        }
+        if options.ntlm || options.negotiate {
+            let mut auth = easy::Auth::new();
+            if options.ntlm {
+                auth.ntlm(true);
+            }
+            if options.negotiate {
+                auth.gssnegotiate(true);
+            }
+            self.handle.http_auth(&auth)?;
+        }
 
         self.set_ssl_options(options.ssl_no_revoke)?;
 
@@ -488,7 +488,21 @@ impl Client {
         self.set_multipart(&request_spec.multipart)?;
         let request_spec_body = &request_spec.body.bytes();
         self.set_body(request_spec_body)?;
-        self.set_headers(request_spec, options)?;
+        // TODO: do we want to manage the headers with no content? There are two type of no-content
+        // headers: `foo:` and `foo;`. The first one can be used to remove libcurl headers (`Host:`)
+        // while the second one is used to send an empty header.
+        // See <https://github.com/Orange-OpenSource/hurl/issues/3536>
+        let options_headers = options
+            .headers
+            .iter()
+            .map(|h| h.as_str())
+            .collect::<Vec<&str>>();
+        let headers = &request_spec.headers.with_raw_headers(&options_headers);
+        self.set_headers(
+            headers,
+            request_spec.implicit_content_type.as_deref(),
+            options,
+        )?;
         if let Some(aws_sigv4) = &options.aws_sigv4 {
             if let Err(e) = self.handle.aws_sigv4(aws_sigv4.as_str()) {
                 return match e.code() {
@@ -533,52 +547,63 @@ impl Client {
     /// Sets HTTP headers.
     fn set_headers(
         &mut self,
-        request_spec: &RequestSpec,
+        headers: &HeaderVec,
+        implicit_content_type: Option<&str>,
         options: &ClientOptions,
     ) -> Result<(), HttpError> {
-        let mut list = List::new();
+        let mut list = headers.to_curl_headers()?;
 
-        for header in &request_spec.headers {
-            list.append(&format!("{}: {}", header.name, header.value))?;
-        }
-
-        // If request has no Content-Type header, we set it if the content type has been set
+        // If request has no `Content-Type` header, we set it if the content type has been set
         // implicitly on this request.
-        if !request_spec.headers.contains_key(CONTENT_TYPE) {
-            if let Some(s) = &request_spec.implicit_content_type {
-                list.append(&format!("{}: {s}", CONTENT_TYPE))?;
+        if !headers.contains_key(CONTENT_TYPE) {
+            if let Some(s) = implicit_content_type {
+                list.append(&format!("{CONTENT_TYPE}: {s}"))?;
             } else {
-                // We remove default Content-Type headers added by curl because we want
-                // to explicitly manage this header.
-                // For instance, with --data option, curl will send a 'Content-type: application/x-www-form-urlencoded'
-                // header.
-                list.append(&format!("{}:", CONTENT_TYPE))?;
+                // We remove default `Content-Type` headers added by curl because we want to
+                // explicitly manage this header.
+                // For instance, with --data option, curl will send a `Content-type: application/x-www-form-urlencoded`
+                // header. From <https://curl.se/libcurl/c/CURLOPT_HTTPHEADER.html>, we can delete
+                // the headers added by libcurl by adding a header with no content.
+                list.append(&format!("{CONTENT_TYPE}:"))?;
             }
         }
 
-        // Workaround for libcurl issue #11664: When Hurl explicitly sets `Expect:` to remove the header,
-        // libcurl will generate `SignedHeaders` that include `expect` even though the header is not
-        // present, causing some APIs to reject the request.
+        // Workaround for libcurl issue <https://github.com/curl/curl/issues/11664>:
+        // When Hurl explicitly sets `Expect:` to remove the header, libcurl will generate
+        // `SignedHeaders` that include `expect` even though the header is not present, causing
+        // some APIs to reject the request.
         // Therefore, we only remove this header when not in aws_sigv4 mode.
-        if !request_spec.headers.contains_key(EXPECT) && options.aws_sigv4.is_none() {
-            // We remove default Expect headers added by curl because we want
-            // to explicitly manage this header.
-            list.append(&format!("{}:", EXPECT))?;
+        if !headers.contains_key(EXPECT) && options.aws_sigv4.is_none() {
+            // We remove default Expect headers added by curl because we want to explicitly manage
+            // this header.
+            list.append(&format!("{EXPECT}:"))?;
         }
 
-        if !request_spec.headers.contains_key(USER_AGENT) {
+        if !headers.contains_key(USER_AGENT) {
             let user_agent = match options.user_agent {
                 Some(ref u) => u.clone(),
-                None => format!("hurl/{}", clap::crate_version!()),
+                None => {
+                    let pkg_version = env!("CARGO_PKG_VERSION");
+                    format!("hurl/{pkg_version}")
+                }
             };
-            list.append(&format!("{}: {user_agent}", USER_AGENT))?;
+            list.append(&format!("{USER_AGENT}: {user_agent}"))?;
         }
 
         if let Some(user) = &options.user {
-            if options.aws_sigv4.is_some() {
+            if options.aws_sigv4.is_some() || options.ntlm || options.negotiate {
                 // curl's aws_sigv4 support needs to know the username and password for the
                 // request, as it uses those values to calculate the Authorization header for the
                 // AWS V4 signature.
+                //
+                // --ntlm requires a username and password to be provided in order to complete the
+                // authentication process. With curl, this would be `-u username:password`
+                //
+                // --negotiate requires a username and password, though they are not used.
+                // From the curl man page:
+                // > When using this option, you must also provide a fake `-u, --user` option to
+                // > activate the authentication code properly. Sending a '-u :' is enough, as the
+                // > username and password from the `-u, --user` option are not actually used.
                 if let Some((username, password)) = user.split_once(':') {
                     self.handle.username(username)?;
                     self.handle.password(password)?;
@@ -586,13 +611,13 @@ impl Client {
             } else {
                 let user = user.as_bytes();
                 let authorization = general_purpose::STANDARD.encode(user);
-                if !request_spec.headers.contains_key(AUTHORIZATION) {
-                    list.append(&format!("{}: Basic {authorization}", AUTHORIZATION))?;
+                if !headers.contains_key(AUTHORIZATION) {
+                    list.append(&format!("{AUTHORIZATION}: Basic {authorization}"))?;
                 }
             }
         }
-        if options.compressed && !request_spec.headers.contains_key(ACCEPT_ENCODING) {
-            list.append(&format!("{}: gzip, deflate, br", ACCEPT_ENCODING))?;
+        if options.compressed && !headers.contains_key(ACCEPT_ENCODING) {
+            list.append(&format!("{ACCEPT_ENCODING}: gzip, deflate, br"))?;
         }
 
         self.handle.http_headers(list)?;
@@ -705,6 +730,14 @@ impl Client {
         headers
     }
 
+    /// Get the IP address of the last connection from libcurl
+    fn primary_ip(&mut self) -> Result<IpAddr, HttpError> {
+        match self.handle.primary_ip()? {
+            Some(ip) => Ok(IpAddr::new(ip.to_string())),
+            None => Err(HttpError::NoPrimaryIp),
+        }
+    }
+
     /// Retrieves an optional location to follow
     ///
     /// You need:
@@ -728,7 +761,7 @@ impl Client {
     }
 
     /// Returns cookie storage.
-    pub fn cookie_storage(&mut self) -> Vec<Cookie> {
+    pub fn cookie_storage(&mut self, logger: &mut Logger) -> Vec<Cookie> {
         let list = self.handle.cookies().unwrap();
         let mut cookies = vec![];
         for cookie in list.iter() {
@@ -736,27 +769,23 @@ impl Client {
             if let Ok(cookie) = Cookie::from_str(line) {
                 cookies.push(cookie);
             } else {
-                eprintln!("warning: line <{line}> can not be parsed as cookie");
+                logger.warning(&format!("Line <{line}> can not be parsed as cookie"));
             }
         }
         cookies
     }
 
     /// Adds a cookie to the cookie jar.
-    pub fn add_cookie(&mut self, cookie: &Cookie, options: &ClientOptions) {
-        if options.verbosity.is_some() {
-            eprintln!("* add to cookie store: {cookie}");
-        }
+    pub fn add_cookie(&mut self, cookie: &Cookie, logger: &mut Logger) {
+        logger.debug(&format!("Add to cookie store <{cookie}> (experimental)"));
         self.handle
             .cookie_list(cookie.to_string().as_str())
             .unwrap();
     }
 
     /// Clears cookie storage.
-    pub fn clear_cookie_storage(&mut self, options: &ClientOptions) {
-        if options.verbosity.is_some() {
-            eprintln!("* clear cookie storage");
-        }
+    pub fn clear_cookie_storage(&mut self, logger: &mut Logger) {
+        logger.debug("Clear cookie storage (experimental)");
         self.handle.cookie_list("ALL").unwrap();
     }
 
@@ -767,8 +796,9 @@ impl Client {
         context_dir: &ContextDir,
         output: Option<&Output>,
         options: &ClientOptions,
+        logger: &mut Logger,
     ) -> CurlCmd {
-        let cookies = self.cookie_storage();
+        let cookies = self.cookie_storage(logger);
         CurlCmd::new(request_spec, &cookies, context_dir, output, options)
     }
 
@@ -789,7 +819,7 @@ impl Client {
                     Ok(Some(value))
                 }
                 Err(message) => {
-                    logger.error(&format!("can not parse certificate - {message}"));
+                    logger.warning(&format!("Can not parse certificate - {message}"));
                     Ok(None)
                 }
             }
@@ -807,13 +837,13 @@ impl Client {
 }
 
 /// Returns the method used for redirecting a request/response with `response_status`.
-fn redirect_method(response_status: u32, original_method: Method) -> Method {
+fn redirect_method(response_status: u32, original_method: &Method) -> Method {
     // This replicates curl's behavior
     match response_status {
         301..=303 => Method("GET".to_string()),
         // Could be only 307 and 308, but curl does this for all 3xx
         // codes not converted to GET above.
-        _ => original_method,
+        _ => original_method.clone(),
     }
 }
 
@@ -835,7 +865,7 @@ pub fn all_cookies(cookie_storage: &[Cookie], request_spec: &RequestSpec) -> Vec
 }
 
 /// Matches cookie for a given URL.
-pub fn match_cookie(cookie: &Cookie, url: &Url) -> bool {
+fn match_cookie(cookie: &Cookie, url: &Url) -> bool {
     if let Some(domain) = url.domain() {
         if cookie.include_subdomain == "FALSE" {
             if cookie.domain != domain {
@@ -862,6 +892,21 @@ impl Header {
     }
 }
 
+impl HeaderVec {
+    /// Converts this list of [`Header`] to a lib curl header list.
+    fn to_curl_headers(&self) -> Result<List, HttpError> {
+        let mut curl_headers = List::new();
+        for header in self {
+            if header.value.is_empty() {
+                curl_headers.append(&format!("{};", header.name))?;
+            } else {
+                curl_headers.append(&format!("{}: {}", header.name, header.value))?;
+            }
+        }
+        Ok(curl_headers)
+    }
+}
+
 /// Splits an array of bytes into HTTP lines (\r\n separator).
 fn split_lines(data: &[u8]) -> Vec<String> {
     let mut lines = vec![];
@@ -882,16 +927,10 @@ fn split_lines(data: &[u8]) -> Vec<String> {
 }
 
 /// Decodes optionally header value as text with UTF-8 or ISO-8859-1 encoding.
-pub fn decode_header(data: &[u8]) -> Option<String> {
+fn decode_header(data: &[u8]) -> Option<String> {
     match str::from_utf8(data) {
         Ok(s) => Some(s.to_string()),
-        Err(_) => match ISO_8859_1.decode(data, DecoderTrap::Strict) {
-            Ok(s) => Some(s),
-            Err(_) => {
-                println!("Error decoding header both UTF-8 and ISO-8859-1 {data:?}");
-                None
-            }
-        },
+        Err(_) => ISO_8859_1.decode(data, DecoderTrap::Strict).ok(),
     }
 }
 
@@ -976,6 +1015,8 @@ mod tests {
     use std::path::PathBuf;
 
     use super::*;
+    use crate::util::logger::LoggerOptionsBuilder;
+    use crate::util::term::{Stderr, WriteMode};
 
     #[test]
     fn test_parse_header() {
@@ -1071,72 +1112,12 @@ mod tests {
         ];
         for (status, original, redirected) in data {
             assert_eq!(
-                redirect_method(status, Method(original.to_string())),
+                redirect_method(status, &Method(original.to_string())),
                 Method(redirected.to_string())
             );
         }
     }
 
-    #[test]
-    fn http_client_state_always_http2() {
-        let mut state = ClientState::default();
-        assert!(!state.has_changed());
-
-        // Client set HTTP 2 on all request, client state never changed
-
-        // - => HTTP/2: no change
-        state.set_requested_http_version(RequestedHttpVersion::Http2);
-        assert!(!state.has_changed());
-
-        // HTTP/2 => HTTP/2: no change
-        state.set_requested_http_version(RequestedHttpVersion::Http2);
-        assert!(!state.has_changed());
-    }
-
-    #[test]
-    fn http_client_state_always_default() {
-        let mut state = ClientState::default();
-        assert!(!state.has_changed());
-
-        // Client doesn't set HTTP version, client state never changed
-
-        // - => Default: no change
-        state.set_requested_http_version(RequestedHttpVersion::Default);
-        assert!(!state.has_changed());
-
-        // Default => Default: no change
-        state.set_requested_http_version(RequestedHttpVersion::Default);
-        assert!(!state.has_changed());
-    }
-
-    #[test]
-    fn http_client_state_changes() {
-        let mut state = ClientState::default();
-        assert!(!state.has_changed());
-
-        // Client set HTTP 2 on all request, client state never changed
-
-        // - => HTTP/2: no change
-        state.set_requested_http_version(RequestedHttpVersion::Http2);
-        assert!(!state.has_changed());
-
-        // HTTP/2 => HTTP/1.1: change
-        state.set_requested_http_version(RequestedHttpVersion::Http11);
-        assert!(state.has_changed());
-
-        // HTTP/1.1 => HTTP/1.1: no change
-        state.set_requested_http_version(RequestedHttpVersion::Http11);
-        assert!(!state.has_changed());
-
-        // HTTP/1.1 => Default: change
-        state.set_requested_http_version(RequestedHttpVersion::Default);
-        assert!(state.has_changed());
-
-        // Default => Default: no change
-        state.set_requested_http_version(RequestedHttpVersion::Default);
-        assert!(!state.has_changed());
-    }
-
     #[test]
     fn command_line_args() {
         let mut client = Client::new();
@@ -1165,7 +1146,11 @@ mod tests {
             ..Default::default()
         };
 
-        let cmd = client.curl_command_line(&request, &context_dir, output, &options);
+        let logger_options = LoggerOptionsBuilder::default().build();
+        let stderr = Stderr::new(WriteMode::Immediate);
+        let mut logger = Logger::new(&logger_options, stderr, &[]);
+
+        let cmd = client.curl_command_line(&request, &context_dir, output, &options, &mut logger);
         assert_eq!(
             cmd.to_string(),
             "curl \
@@ -1216,4 +1201,18 @@ mod tests {
             ("foo\\".to_string(), Some("toto\\:tata:tutu".to_string()))
         );
     }
+
+    #[test]
+    fn test_to_curl_headers() {
+        let mut headers = HeaderVec::new();
+        headers.push(Header::new("foo", "a"));
+        headers.push(Header::new("bar", "b"));
+        headers.push(Header::new("baz", ""));
+
+        let list = headers.to_curl_headers().unwrap();
+        assert_eq!(
+            list.iter().collect::<Vec<_>>(),
+            vec!["foo: a".as_bytes(), "bar: b".as_bytes(), "baz;".as_bytes()]
+        );
+    }
 }
