@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,17 +15,19 @@
  * limitations under the License.
  *
  */
+use core::fmt;
+use std::collections::HashMap;
+use std::path::Path;
+
+use hurl_core::typing::Count;
+
 use crate::http::client::all_cookies;
 use crate::http::{
-    Body, ClientOptions, Cookie, FileParam, Header, IpResolve, Method, MultipartParam, Param,
-    RequestSpec, RequestedHttpVersion, CONTENT_TYPE,
+    Body, ClientOptions, Cookie, FileParam, Header, HeaderVec, IpResolve, Method, MultipartParam,
+    Param, RequestSpec, RequestedHttpVersion, CONTENT_TYPE,
 };
 use crate::runner::Output;
 use crate::util::path::ContextDir;
-use core::fmt;
-use hurl_core::typing::Count;
-use std::collections::HashMap;
-use std::path::Path;
 
 /// Represents a curl command, with arguments.
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -60,10 +62,20 @@ impl CurlCmd {
     ) -> Self {
         let mut args = vec!["curl".to_string()];
 
-        let mut params = method_params(request_spec);
+        let mut params = method_params(request_spec, options.follow_location);
         args.append(&mut params);
 
-        let mut params = headers_params(request_spec);
+        let options_headers = options
+            .headers
+            .iter()
+            .map(|h| h.as_str())
+            .collect::<Vec<&str>>();
+        let headers = &request_spec.headers.with_raw_headers(&options_headers);
+        let mut params = headers_params(
+            headers,
+            request_spec.implicit_content_type.as_deref(),
+            &request_spec.body,
+        );
         args.append(&mut params);
 
         let mut params = body_params(request_spec, context_dir);
@@ -83,46 +95,51 @@ impl CurlCmd {
 }
 
 /// Returns the curl args corresponding to the HTTP method, from a request spec.
-fn method_params(request_spec: &RequestSpec) -> Vec<String> {
+fn method_params(request_spec: &RequestSpec, follow_location: bool) -> Vec<String> {
     let has_body = !request_spec.multipart.is_empty()
         || !request_spec.form.is_empty()
         || !request_spec.body.bytes().is_empty();
-    request_spec.method.curl_args(has_body)
+    request_spec.method.curl_args(has_body, follow_location)
 }
 
-/// Returns the curl args corresponding to the HTTP headers, from a request spec.
-fn headers_params(request_spec: &RequestSpec) -> Vec<String> {
+/// Returns the curl args corresponding to the HTTP headers, from a list of headers,
+/// an optional implicit content type, and the request body.
+fn headers_params(
+    headers: &HeaderVec,
+    implicit_content_type: Option<&str>,
+    body: &Body,
+) -> Vec<String> {
     let mut args = vec![];
 
-    for header in request_spec.headers.iter() {
+    for header in headers.iter() {
         args.append(&mut header.curl_args());
     }
 
-    let has_explicit_content_type = request_spec.headers.contains_key(CONTENT_TYPE);
+    let has_explicit_content_type = headers.contains_key(CONTENT_TYPE);
     if has_explicit_content_type {
         return args;
     }
 
-    if let Some(content_type) = &request_spec.implicit_content_type {
+    if let Some(content_type) = implicit_content_type {
         if content_type != "application/x-www-form-urlencoded"
             && content_type != "multipart/form-data"
         {
             args.push("--header".to_string());
-            args.push(format!("'{}: {content_type}'", CONTENT_TYPE));
+            args.push(format!("'{CONTENT_TYPE}: {content_type}'"));
         }
-    } else if !request_spec.body.bytes().is_empty() {
-        match request_spec.body {
+    } else if !body.bytes().is_empty() {
+        match body {
             Body::Text(_) => {
                 args.push("--header".to_string());
-                args.push(format!("'{}:'", CONTENT_TYPE));
+                args.push(format!("'{CONTENT_TYPE}:'"));
             }
             Body::Binary(_) => {
                 args.push("--header".to_string());
-                args.push(format!("'{}: application/octet-stream'", CONTENT_TYPE));
+                args.push(format!("'{CONTENT_TYPE}: application/octet-stream'"));
             }
             Body::File(_, _) => {
                 args.push("--header".to_string());
-                args.push(format!("'{}:'", CONTENT_TYPE));
+                args.push(format!("'{CONTENT_TYPE}:'"));
             }
         }
     }
@@ -263,7 +280,7 @@ fn encode_bytes(bytes: &[u8]) -> String {
 
 impl Method {
     /// Returns the curl args for HTTP method, given the request has a body or not.
-    fn curl_args(&self, has_body: bool) -> Vec<String> {
+    fn curl_args(&self, has_body: bool, follow_location: bool) -> Vec<String> {
         match self.0.as_str() {
             "GET" => {
                 if has_body {
@@ -274,8 +291,20 @@ impl Method {
             }
             "HEAD" => vec!["--head".to_string()],
             "POST" => {
+                // In <https://curl.se/docs/manpage.html#-X>, `--location` and `--request/-X` does not well play together:
+                //
+                // > The method string you set with -X, --request is used for all requests, which if you for example use -L,
+                // > --location may cause unintended side-effects when curl does not change request method according to the
+                // > HTTP 30x response codes - and similar.
+                //
+                // When we use `--request POST` with curl, we're telling curl to make POST requests for every request. This
+                // can interfere with `--location` option that can make GET requests following the initial POST. So, in the
+                // case of a POST request with `--location` option, we don't force the method to let curl decides the right
+                // method of the redirection steps.
                 if has_body {
                     vec![]
+                } else if follow_location {
+                    vec!["--data".to_string(), "''".to_string()]
                 } else {
                     vec!["--request".to_string(), "POST".to_string()]
                 }
@@ -291,7 +320,11 @@ impl Header {
         let value = &self.value;
         vec![
             "--header".to_string(),
-            encode_shell_string(&format!("{name}: {value}")),
+            if self.value.is_empty() {
+                encode_shell_string(&format!("{name};"))
+            } else {
+                encode_shell_string(&format!("{name}: {value}"))
+            },
         ]
     }
 }
@@ -415,6 +448,13 @@ impl ClientOptions {
             arguments.push("--max-redirs".to_string());
             arguments.push(max_redirect.to_string());
         }
+        if self.timeout != ClientOptions::default().timeout {
+            arguments.push("--max-time".to_string());
+            arguments.push(self.timeout.as_secs().to_string());
+        }
+        if self.negotiate {
+            arguments.push("--negotiate".to_string());
+        }
         if let Some(filename) = &self.netrc_file {
             arguments.push("--netrc-file".to_string());
             arguments.push(format!("'{filename}'"));
@@ -425,9 +465,16 @@ impl ClientOptions {
         if self.netrc {
             arguments.push("--netrc".to_string());
         }
+        if self.ntlm {
+            arguments.push("--ntlm".to_string());
+        }
         if self.path_as_is {
             arguments.push("--path-as-is".to_string());
         }
+        if let Some(ref pinned_pub_key) = self.pinned_pub_key {
+            arguments.push("--pinnedpubkey".to_string());
+            arguments.push(pinned_pub_key.clone());
+        }
         if let Some(ref proxy) = self.proxy {
             arguments.push("--proxy".to_string());
             arguments.push(format!("'{proxy}'"));
@@ -439,10 +486,6 @@ impl ClientOptions {
         if self.ssl_no_revoke {
             arguments.push("--ssl-no-revoke".to_string());
         }
-        if self.timeout != ClientOptions::default().timeout {
-            arguments.push("--timeout".to_string());
-            arguments.push(self.timeout.as_secs().to_string());
-        }
         if let Some(ref unix_socket) = self.unix_socket {
             arguments.push("--unix-socket".to_string());
             arguments.push(format!("'{unix_socket}'"));
@@ -503,13 +546,14 @@ fn escape_string(s: &str) -> String {
 
 #[cfg(test)]
 mod tests {
-    use crate::http::{HeaderVec, Url};
-    use hurl_core::typing::BytesPerSec;
     use std::path::Path;
     use std::str::FromStr;
     use std::time::Duration;
 
+    use hurl_core::typing::BytesPerSec;
+
     use super::*;
+    use crate::http::{HeaderVec, Url};
 
     #[test]
     fn hello_request_with_default_options() {
@@ -524,12 +568,12 @@ mod tests {
         let options = ClientOptions::default();
         let output = None;
 
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(cmd.to_string(), "curl 'http://localhost:8000/hello'");
 
         // Same requests with some output:
         let output = Some(Output::new("foo.out"));
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl \
@@ -542,7 +586,7 @@ mod tests {
         headers.push(Header::new("User-Agent", "iPhone"));
         headers.push(Header::new("Foo", "Bar"));
         request.headers = headers;
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl \
@@ -575,7 +619,7 @@ mod tests {
                 http_only: true,
             },
         ];
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl \
@@ -598,6 +642,7 @@ mod tests {
         let context_dir = &ContextDir::default();
         let cookies = vec![];
         let options = ClientOptions {
+            allow_reuse: true,
             aws_sigv4: None,
             cacert_file: None,
             client_cert_file: None,
@@ -608,6 +653,11 @@ mod tests {
             cookie_input_file: Some("cookie_file".to_string()),
             follow_location: true,
             follow_location_trusted: false,
+            headers: vec![
+                "Test-Header-1: content-1".to_string(),
+                "Test-Header-2: content-2".to_string(),
+                "Test-Header-Empty:".to_string(),
+            ],
             http_version: RequestedHttpVersion::Http10,
             insecure: true,
             ip_resolve: IpResolve::IpV6,
@@ -615,10 +665,13 @@ mod tests {
             max_recv_speed: Some(BytesPerSec(8000)),
             max_redirect: Count::Finite(10),
             max_send_speed: Some(BytesPerSec(8000)),
+            negotiate: true,
             netrc: false,
             netrc_file: Some("/var/run/netrc".to_string()),
             netrc_optional: true,
+            ntlm: true,
             path_as_is: true,
+            pinned_pub_key: None,
             proxy: Some("localhost:3128".to_string()),
             no_proxy: None,
             resolves: vec![
@@ -633,10 +686,13 @@ mod tests {
             verbosity: None,
         };
 
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, None, &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, None, &options);
         assert_eq!(
             cmd.to_string(),
             "curl \
+        --header 'Test-Header-1: content-1' \
+        --header 'Test-Header-2: content-2' \
+        --header 'Test-Header-Empty;' \
         --compressed \
         --connect-timeout 20 \
         --connect-to example.com:443:host-47.example.com:443 \
@@ -647,13 +703,15 @@ mod tests {
         --location \
         --limit-rate 8000 \
         --max-redirs 10 \
+        --max-time 10 \
+        --negotiate \
         --netrc-file '/var/run/netrc' \
         --netrc-optional \
+        --ntlm \
         --path-as-is \
         --proxy 'localhost:3128' \
         --resolve foo.com:80:192.168.0.1 \
         --resolve bar.com:443:127.0.0.1 \
-        --timeout 10 \
         --unix-socket '/var/run/example.sock' \
         --user 'user:password' \
         --user-agent 'my-useragent' \
@@ -674,7 +732,7 @@ mod tests {
         let options = ClientOptions::default();
         let output = None;
 
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl 'https://example.org/hello/../to/../your/../file'"
@@ -694,7 +752,7 @@ mod tests {
         let options = ClientOptions::default();
         let output = None;
 
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl \
@@ -726,7 +784,7 @@ mod tests {
         let options = ClientOptions::default();
         let output = None;
 
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl 'http://localhost:8000/querystring-params?param1=value1&param2=a%20b'",
@@ -736,7 +794,7 @@ mod tests {
         request.url =
             Url::from_str("http://localhost:8000/querystring-params?param3=foo&param4=bar")
                 .unwrap();
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl 'http://localhost:8000/querystring-params?param3=foo&param4=bar&param1=value1&param2=a%20b'",
@@ -765,7 +823,7 @@ mod tests {
         let options = ClientOptions::default();
         let output = None;
 
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl \
@@ -784,7 +842,7 @@ mod tests {
             method: Method("POST".to_string()),
             url: Url::from_str("http://localhost/json").unwrap(),
             headers,
-            body: Body::Text("".to_string()),
+            body: Body::Text(String::new()),
             implicit_content_type: Some("application/json".to_string()),
             ..Default::default()
         };
@@ -794,7 +852,7 @@ mod tests {
         let options = ClientOptions::default();
         let output = None;
 
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl \
@@ -805,7 +863,7 @@ mod tests {
 
         // Add a non-empty body
         request.body = Body::Text("{\"foo\":\"bar\"}".to_string());
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl \
@@ -816,7 +874,7 @@ mod tests {
 
         // Change method
         request.method = Method("PUT".to_string());
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl \
@@ -841,7 +899,7 @@ mod tests {
         let options = ClientOptions::default();
         let output = None;
 
-        let cmd = CurlCmd::new(&request, &cookies, &context_dir, output.as_ref(), &options);
+        let cmd = CurlCmd::new(&request, &cookies, context_dir, output.as_ref(), &options);
         assert_eq!(
             cmd.to_string(),
             "curl \
