@@ -1,6 +1,6 @@
 /*
 * Hurl (https://hurl.dev)
-* Copyright (C) 2024 Orange
+* Copyright (C) 2025 Orange
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -15,11 +15,11 @@
 * limitations under the License.
 *
 */
-
 use crate::http::RequestedHttpVersion;
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum HttpError {
+    CouldNotParseCookieExpires(String),
     CouldNotParseResponse,
     CouldNotUncompressResponse {
         description: String,
@@ -38,6 +38,7 @@ pub enum HttpError {
         option: String,
         minimum_version: String,
     },
+    NoPrimaryIp,
     TooManyRedirect,
     UnsupportedContentEncoding {
         description: String,
@@ -63,6 +64,7 @@ impl HttpError {
     pub fn description(&self) -> String {
         match self {
             HttpError::AllowedResponseSizeExceeded(_) => "HTTP connection".to_string(),
+            HttpError::CouldNotParseCookieExpires(_) => "HTTP connection".to_string(),
             HttpError::CouldNotParseResponse => "HTTP connection".to_string(),
             HttpError::CouldNotUncompressResponse { .. } => "Decompression error".to_string(),
             HttpError::InvalidCharset { .. } => "Invalid charset".to_string(),
@@ -70,6 +72,7 @@ impl HttpError {
             HttpError::InvalidUrl(..) => "Invalid URL".to_string(),
             HttpError::Libcurl { .. } => "HTTP connection".to_string(),
             HttpError::LibcurlUnknownOption { .. } => "HTTP connection".to_string(),
+            HttpError::NoPrimaryIp => "HTTP connection".to_string(),
             HttpError::TooManyRedirect => "HTTP connection".to_string(),
             HttpError::UnsupportedContentEncoding { .. } => "Decompression error".to_string(),
             HttpError::UnsupportedHttpVersion(_) => "Unsupported HTTP version".to_string(),
@@ -81,6 +84,9 @@ impl HttpError {
             HttpError::AllowedResponseSizeExceeded(max_size) => {
                 format!("exceeded the maximum allowed file size ({max_size} bytes)")
             }
+            HttpError::CouldNotParseCookieExpires(value) => {
+                format!("could not parse Cookie Expires attribute value <{value}>")
+            }
             HttpError::CouldNotParseResponse => "could not parse Response".to_string(),
             HttpError::CouldNotUncompressResponse { description } => {
                 format!("could not uncompress response with {description}")
@@ -99,6 +105,7 @@ impl HttpError {
                 option,
                 minimum_version,
             } => format!("Option {option} requires libcurl version {minimum_version} or higher"),
+            HttpError::NoPrimaryIp => "No primary IP found in response".to_string(),
             HttpError::TooManyRedirect => "too many redirect".to_string(),
             HttpError::UnsupportedHttpVersion(version) => {
                 format!("{version} is not supported, check --version").to_string()
