@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,7 +20,7 @@ use encoding::EncodingRef;
 
 use crate::http::header::CONTENT_ENCODING;
 use crate::http::response_decoding::ContentEncoding;
-use crate::http::{mimetype, HeaderVec, HttpError, CONTENT_TYPE};
+use crate::http::{mimetype, Header, HeaderVec, HttpError, CONTENT_TYPE};
 
 impl HeaderVec {
     /// Returns optional Content-type header value.
@@ -60,6 +60,18 @@ impl HeaderVec {
         }
         Ok(vec![])
     }
+
+    /// Returns a new list of headers with the headers from `self` and the raw headers `raw_headers`.
+    pub fn with_raw_headers(&self, raw_headers: &[&str]) -> HeaderVec {
+        let mut headers = self.clone();
+        // TODO: use another function that [`Header::parse`] because [`Header::parse`] is for
+        // parsing headers line coming from a server (and not from options header)
+        let raw_headers = raw_headers.iter().filter_map(|h| Header::parse(h));
+        for header in raw_headers {
+            headers.push(header);
+        }
+        headers
+    }
 }
 
 #[cfg(test)]
@@ -105,4 +117,37 @@ mod tests {
         headers.push(Header::new("content-type", "text/plain"));
         assert_eq!(headers.character_encoding().unwrap().name(), "utf-8");
     }
+
+    #[test]
+    fn test_with_raw_headers() {
+        let mut headers = HeaderVec::new();
+        headers.push(Header::new("Host", "localhost:8000"));
+        headers.push(Header::new("Repeated-Header", "original"));
+
+        let raw_headers = &[
+            "User-Agent: hurl/6.1.0",
+            "Invalid-Header",
+            "Repeated-Header: aggregated-1",
+            "Repeated-Header: aggregated-2",
+        ];
+        let headers = headers.with_raw_headers(raw_headers);
+
+        assert_eq!(
+            headers.get("Host"),
+            Some(&Header::new("Host", "localhost:8000"))
+        );
+        assert_eq!(
+            headers.get("User-Agent"),
+            Some(&Header::new("User-Agent", "hurl/6.1.0"))
+        );
+        assert_eq!(headers.get("Invalid-Header"), None);
+        assert_eq!(
+            headers.get_all("Repeated-Header"),
+            vec![
+                &Header::new("Repeated-Header", "original"),
+                &Header::new("Repeated-Header", "aggregated-1"),
+                &Header::new("Repeated-Header", "aggregated-2")
+            ]
+        );
+    }
 }
