@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -37,7 +37,7 @@ pub use self::response::{HttpVersion, Response};
 #[cfg(test)]
 pub use self::tests::*;
 pub use self::timings::Timings;
-pub use self::url::Url;
+pub use self::url::{Url, UrlError};
 pub use self::version::libcurl_version_info;
 
 mod call;
@@ -51,6 +51,7 @@ mod easy_ext;
 mod error;
 mod header;
 mod headers_helper;
+mod ip;
 mod mimetype;
 mod options;
 mod request;
