@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,14 +15,20 @@
  * limitations under the License.
  *
  */
-use hurl_core::typing::{BytesPerSec, Count};
 use std::time::Duration;
 
+use hurl_core::typing::{BytesPerSec, Count};
+
 use crate::http::request::RequestedHttpVersion;
 use crate::http::IpResolve;
 
 #[derive(Debug, Clone)]
 pub struct ClientOptions {
+    /// Allow reusing internal connections, `true` by default. Setting this to `false` forces the
+    /// HTTP client to use a new HTTP connection, and also marks this new connection as not reusable.
+    /// Under the hood, this activates libcurl [`CURLOPT_FRESH_CONNECT`](https://curl.se/libcurl/c/CURLOPT_FRESH_CONNECT.html)
+    /// and [`CURLOPT_FORBID_REUSE`](https://curl.se/libcurl/c/CURLOPT_FORBID_REUSE.html).
+    pub allow_reuse: bool,
     pub aws_sigv4: Option<String>,
     pub cacert_file: Option<String>,
     pub client_cert_file: Option<String>,
@@ -33,6 +39,7 @@ pub struct ClientOptions {
     pub cookie_input_file: Option<String>,
     pub follow_location: bool,
     pub follow_location_trusted: bool,
+    pub headers: Vec<String>,
     pub http_version: RequestedHttpVersion,
     pub insecure: bool,
     pub ip_resolve: IpResolve,
@@ -40,11 +47,14 @@ pub struct ClientOptions {
     pub max_recv_speed: Option<BytesPerSec>,
     pub max_redirect: Count,
     pub max_send_speed: Option<BytesPerSec>,
+    pub negotiate: bool,
     pub netrc: bool,
     pub netrc_file: Option<String>,
     pub netrc_optional: bool,
     pub no_proxy: Option<String>,
+    pub ntlm: bool,
     pub path_as_is: bool,
+    pub pinned_pub_key: Option<String>,
     pub proxy: Option<String>,
     pub resolves: Vec<String>,
     pub ssl_no_revoke: bool,
@@ -65,6 +75,7 @@ pub enum Verbosity {
 impl Default for ClientOptions {
     fn default() -> Self {
         ClientOptions {
+            allow_reuse: true,
             aws_sigv4: None,
             cacert_file: None,
             client_cert_file: None,
@@ -75,6 +86,7 @@ impl Default for ClientOptions {
             cookie_input_file: None,
             follow_location: false,
             follow_location_trusted: false,
+            headers: vec![],
             http_version: RequestedHttpVersion::default(),
             insecure: false,
             ip_resolve: IpResolve::default(),
@@ -82,11 +94,14 @@ impl Default for ClientOptions {
             max_recv_speed: None,
             max_redirect: Count::Finite(50),
             max_send_speed: None,
+            negotiate: false,
             netrc: false,
             netrc_file: None,
             netrc_optional: false,
             no_proxy: None,
+            ntlm: false,
             path_as_is: false,
+            pinned_pub_key: None,
             proxy: None,
             resolves: vec![],
             ssl_no_revoke: false,
