@@ -16,7 +16,7 @@
  *
  */
 
-use crate::jsonpath::ast::Query;
+use crate::jsonpath::ast::{Query, Selector};
 use crate::jsonpath::JsonpathResult;
 
 impl Query {
@@ -32,16 +32,24 @@ impl Query {
                 JsonpathResult::Collection(values) => {
                     let mut elements = vec![];
                     for value in values {
-                        match selector.eval(&value)? {
-                            JsonpathResult::SingleEntry(new_value) => {
-                                elements.push(new_value);
-                            }
-                            JsonpathResult::Collection(mut new_values) => {
-                                elements.append(&mut new_values);
+                        if let Some(value) = selector.eval(&value) {
+                            match value {
+                                JsonpathResult::SingleEntry(new_value) => {
+                                    elements.push(new_value);
+                                }
+                                JsonpathResult::Collection(mut new_values) => {
+                                    elements.append(&mut new_values);
+                                }
                             }
                         }
-                        result = JsonpathResult::Collection(elements.clone());
                     }
+
+                    // Retuns nothing (not exists) rather than an empty list with the array index selector
+                    // For example with an out of bound index or with an object input
+                    if (matches!(selector, Selector::ArrayIndex(_)) && elements.is_empty()) {
+                        return None;
+                    }
+                    result = JsonpathResult::Collection(elements.clone());
                 }
             }
         }
