@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,14 +18,17 @@
 mod cli;
 mod run;
 
+use std::collections::HashSet;
 use std::io::prelude::*;
+use std::io::IsTerminal;
 use std::path::Path;
 use std::time::Instant;
-use std::{env, process, thread};
+use std::{env, io, process, thread};
 
 use hurl::report::{curl, html, json, junit, tap};
 use hurl::runner;
 use hurl::runner::HurlResult;
+use hurl::util::redacted::Redact;
 use hurl_core::input::Input;
 use hurl_core::text;
 
@@ -53,10 +56,12 @@ struct HurlRun {
 fn main() {
     text::init_crate_colored();
 
-    let opts = match cli::options::parse() {
+    let allow_color = allow_color_from_env();
+
+    let opts = match cli::options::parse(allow_color) {
         Ok(v) => v,
         Err(e) => match e {
-            CliOptionsError::Info(_) => {
+            CliOptionsError::DisplayHelp(e) | CliOptionsError::DisplayVersion(e) => {
                 print!("{e}");
                 process::exit(EXIT_OK);
             }
@@ -90,20 +95,28 @@ fn main() {
         run::run_seq(&opts.input_files, current_dir, &opts)
     };
     let runs = match runs {
+        // Even in the presence of false assertions, `run::run_par` or `run::run_seq` return an `Ok`
+        // result. The false assertions "errors" are displayed in these functions and are not considered
+        // as program errors.
         Ok(r) => r,
-        Err(CliError::IO(msg)) => exit_with_error(&msg, EXIT_ERROR_PARSING, &base_logger),
+        // So, we're dealing here with I/O errors: input reading, parsing etc...
+        // We consider input read as "parsing" and don't have a specific exit code for the moment.
+        Err(CliError::InputRead(msg)) => exit_with_error(&msg, EXIT_ERROR_PARSING, &base_logger),
         // In case of parsing error, there is no error because the display of parsing error has been
         // done in the execution of the Hurl files, inside the crates (and not in the main).
         Err(CliError::Parsing) => exit_with_error("", EXIT_ERROR_PARSING, &base_logger),
-        Err(CliError::Runtime(msg)) => exit_with_error(&msg, EXIT_ERROR_RUNTIME, &base_logger),
+        Err(CliError::OutputWrite(msg)) => exit_with_error(&msg, EXIT_ERROR_RUNTIME, &base_logger),
+        Err(CliError::GenericIO(msg)) => exit_with_error(&msg, EXIT_ERROR_PARSING, &base_logger),
     };
 
     // Compute duration of the test here to not take reports writings into account.
     let duration = start.elapsed();
 
     // Write HTML, JUnit, TAP reports on disk.
-    let ret = export_results(&runs, &opts, &base_logger);
-    unwrap_or_exit(ret, EXIT_ERROR_UNDEFINED, &base_logger);
+    if has_report(&opts) {
+        let ret = export_results(&runs, &opts, &base_logger);
+        unwrap_or_exit(ret, EXIT_ERROR_UNDEFINED, &base_logger);
+    }
 
     if opts.test {
         let summary = cli::summary(&runs, duration);
@@ -132,52 +145,89 @@ fn exit_with_error(message: &str, code: i32, logger: &BaseLogger) -> ! {
     process::exit(code);
 }
 
+/// Returns `true` if any kind of report should be created, `false` otherwise.
+fn has_report(opts: &CliOptions) -> bool {
+    opts.curl_file.is_some()
+        || opts.junit_file.is_some()
+        || opts.tap_file.is_some()
+        || opts.html_dir.is_some()
+        || opts.json_report_dir.is_some()
+        || opts.cookie_output_file.is_some()
+}
+
+/// Returns `true` if we can use ANSI color, solely base on the environment.
+///
+/// This function doesn't take CLI options into account.
+fn allow_color_from_env() -> bool {
+    if let Ok(v) = env::var("NO_COLOR") {
+        if !v.is_empty() {
+            return false;
+        }
+    }
+    io::stdout().is_terminal()
+}
+
 /// Writes `runs` results on file, in HTML, TAP, JUnit or Cookie file format.
 fn export_results(
     runs: &[HurlRun],
     opts: &CliOptions,
     logger: &BaseLogger,
 ) -> Result<(), CliError> {
+    // Compute secrets from the result. As secrets can be redacted during execution, we can't
+    // consider only secrets introduced from cli, we have to get secrets produced during execution.
+    // We remove identical secrets as there may be a lot of identical secrets (those that come
+    // from the command line for instance)
+    let secrets = runs
+        .iter()
+        .flat_map(|r| r.hurl_result.variables.secrets())
+        .collect::<HashSet<_>>();
+    let secrets = secrets.iter().map(|s| s.as_ref()).collect::<Vec<_>>();
+
     if let Some(file) = &opts.curl_file {
-        create_curl_export(runs, file)?;
+        create_curl_export(runs, file, &secrets)?;
     }
     if let Some(file) = &opts.junit_file {
         logger.debug(&format!("Writing JUnit report to {}", file.display()));
-        create_junit_report(runs, file)?;
+        create_junit_report(runs, file, &secrets)?;
     }
     if let Some(file) = &opts.tap_file {
+        // TAP files doesn't need to be redacted, they don't expose any logs apart from files names.
         logger.debug(&format!("Writing TAP report to {}", file.display()));
         create_tap_report(runs, file)?;
     }
     if let Some(dir) = &opts.html_dir {
         logger.debug(&format!("Writing HTML report to {}", dir.display()));
-        create_html_report(runs, dir)?;
+        create_html_report(runs, dir, &secrets)?;
     }
     if let Some(dir) = &opts.json_report_dir {
         logger.debug(&format!("Writing JSON report to {}", dir.display()));
-        create_json_report(runs, dir)?;
+        create_json_report(runs, dir, &secrets)?;
     }
     if let Some(file) = &opts.cookie_output_file {
         logger.debug(&format!("Writing cookies to {}", file.display()));
-        create_cookies_file(runs, file)?;
+        create_cookies_file(runs, file, &secrets)?;
     }
     Ok(())
 }
 
 /// Creates an export of all curl commands for this run.
-fn create_curl_export(runs: &[HurlRun], filename: &Path) -> Result<(), CliError> {
+fn create_curl_export(runs: &[HurlRun], filename: &Path, secrets: &[&str]) -> Result<(), CliError> {
     let results = runs.iter().map(|r| &r.hurl_result).collect::<Vec<_>>();
-    curl::write_curl(&results, filename)?;
+    curl::write_curl(&results, filename, secrets)?;
     Ok(())
 }
 
 /// Creates a JUnit report for this run.
-fn create_junit_report(runs: &[HurlRun], filename: &Path) -> Result<(), CliError> {
+fn create_junit_report(
+    runs: &[HurlRun],
+    filename: &Path,
+    secrets: &[&str],
+) -> Result<(), CliError> {
     let testcases = runs
         .iter()
         .map(|r| junit::Testcase::from(&r.hurl_result, &r.content, &r.filename))
         .collect::<Vec<_>>();
-    junit::write_report(filename, &testcases)?;
+    junit::write_report(filename, &testcases, secrets)?;
     Ok(())
 }
 
@@ -192,7 +242,7 @@ fn create_tap_report(runs: &[HurlRun], filename: &Path) -> Result<(), CliError>
 }
 
 /// Creates an HTML report for this run.
-fn create_html_report(runs: &[HurlRun], dir_path: &Path) -> Result<(), CliError> {
+fn create_html_report(runs: &[HurlRun], dir_path: &Path, secrets: &[&str]) -> Result<(), CliError> {
     // We ensure that the containing folder exists.
     let store_path = dir_path.join("store");
     std::fs::create_dir_all(&store_path)?;
@@ -201,7 +251,7 @@ fn create_html_report(runs: &[HurlRun], dir_path: &Path) -> Result<(), CliError>
     for run in runs.iter() {
         let result = &run.hurl_result;
         let testcase = html::Testcase::from(result, &run.filename);
-        testcase.write_html(&run.content, &result.entries, &store_path)?;
+        testcase.write_html(&run.content, &result.entries, &store_path, secrets)?;
         testcases.push(testcase);
     }
     html::write_report(dir_path, &testcases)?;
@@ -209,7 +259,7 @@ fn create_html_report(runs: &[HurlRun], dir_path: &Path) -> Result<(), CliError>
 }
 
 /// Creates an JSON report for this run.
-fn create_json_report(runs: &[HurlRun], dir_path: &Path) -> Result<(), CliError> {
+fn create_json_report(runs: &[HurlRun], dir_path: &Path, secrets: &[&str]) -> Result<(), CliError> {
     // We ensure that the containing folder exists.
     let store_path = dir_path.join("store");
     std::fs::create_dir_all(&store_path)?;
@@ -220,7 +270,7 @@ fn create_json_report(runs: &[HurlRun], dir_path: &Path) -> Result<(), CliError>
         .collect::<Vec<_>>();
 
     let index_path = dir_path.join("report.json");
-    json::write_report(&index_path, &testcases, &store_path)?;
+    json::write_report(&index_path, &testcases, &store_path, secrets)?;
     Ok(())
 }
 
@@ -246,10 +296,24 @@ fn exit_code(runs: &[HurlRun]) -> i32 {
     }
 }
 
-fn create_cookies_file(runs: &[HurlRun], filename: &Path) -> Result<(), CliError> {
+/// Export cookies for this run to `filename` file.
+///
+/// The file format for the cookies is [Netscape cookie format](http://www.cookiecentral.com/faq/#3.5).
+fn create_cookies_file(
+    runs: &[HurlRun],
+    filename: &Path,
+    secrets: &[&str],
+) -> Result<(), CliError> {
+    if let Err(err) = hurl::util::path::create_dir_all(filename) {
+        return Err(CliError::GenericIO(format!(
+            "Issue creating parent directories for {}: {err:?}",
+            filename.display()
+        )));
+    }
+
     let mut file = match std::fs::File::create(filename) {
         Err(why) => {
-            return Err(CliError::IO(format!(
+            return Err(CliError::GenericIO(format!(
                 "Issue writing to {}: {why:?}",
                 filename.display()
             )));
@@ -261,20 +325,21 @@ fn create_cookies_file(runs: &[HurlRun], filename: &Path) -> Result<(), CliError
 
 "#
     .to_string();
-    match runs.first() {
-        None => {
-            return Err(CliError::IO("Issue fetching results".to_string()));
-        }
-        Some(run) => {
-            for cookie in run.hurl_result.cookies.iter() {
-                s.push_str(&cookie.to_string());
-                s.push('\n');
-            }
+
+    if runs.is_empty() {
+        return Err(CliError::GenericIO("Issue fetching results".to_string()));
+    }
+    for run in runs.iter() {
+        s.push_str(&format!("# Cookies for file <{}>", run.filename));
+        s.push('\n');
+        for cookie in run.hurl_result.cookies.iter() {
+            s.push_str(&cookie.redact(secrets));
+            s.push('\n');
         }
     }
 
     if let Err(why) = file.write_all(s.as_bytes()) {
-        return Err(CliError::IO(format!(
+        return Err(CliError::GenericIO(format!(
             "Issue writing to {}: {why:?}",
             filename.display()
         )));
