@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -91,7 +91,7 @@ pub fn write_last_body(
                 "stdout".to_string()
             };
             let source_info = last_entry.source_info;
-            let kind = OutputErrorKind::Io(format!("{filename} can not be written ({})", e));
+            let kind = OutputErrorKind::Io(format!("{filename} can not be written ({e})"));
             OutputError::new(source_info, kind)
         })?,
     }
@@ -123,7 +123,7 @@ mod tests {
     use hurl_core::ast::SourceInfo;
     use hurl_core::reader::Pos;
 
-    use crate::http::{Call, CurlCmd, Header, HeaderVec, HttpVersion, Request, Response, Url};
+    use crate::http::{Call, Header, HeaderVec, HttpVersion, Request, Response, Url};
     use crate::output::write_last_body;
     use crate::runner::{EntryResult, HurlResult, Output};
     use crate::util::term::{Stdout, WriteMode};
@@ -137,6 +137,7 @@ mod tests {
             duration: Default::default(),
             url: Url::from_str("http://localhost").unwrap(),
             certificate: None,
+            ip_addr: Default::default(),
         }
     }
 
@@ -163,12 +164,7 @@ mod tests {
                         response: default_response(),
                         timings: Default::default(),
                     }],
-                    captures: vec![],
-                    asserts: vec![],
-                    errors: vec![],
-                    transfer_duration: Duration::from_millis(0),
-                    compressed: false,
-                    curl_cmd: CurlCmd::default(),
+                    ..Default::default()
                 },
                 EntryResult {
                     entry_index: 2,
@@ -183,12 +179,7 @@ mod tests {
                         response: default_response(),
                         timings: Default::default(),
                     }],
-                    captures: vec![],
-                    asserts: vec![],
-                    errors: vec![],
-                    transfer_duration: Duration::from_millis(0),
-                    compressed: false,
-                    curl_cmd: CurlCmd::default(),
+                    ..Default::default()
                 },
                 EntryResult {
                     entry_index: 3,
@@ -208,21 +199,16 @@ mod tests {
                             duration: Default::default(),
                             url: Url::from_str("https://baz.com").unwrap(),
                             certificate: None,
+                            ip_addr: Default::default(),
                         },
                         timings: Default::default(),
                     }],
-                    captures: vec![],
-                    asserts: vec![],
-                    errors: vec![],
-                    transfer_duration: Duration::from_millis(0),
-                    compressed: false,
-                    curl_cmd: CurlCmd::default(),
+                    ..Default::default()
                 },
             ],
             duration: Duration::from_millis(100),
             success: true,
-            cookies: vec![],
-            timestamp: 0,
+            ..Default::default()
         }
     }
 
