@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,7 +18,9 @@
 
 /// Error triggered when running a [`crate::parallel::job::Job`].
 pub enum JobError {
-    IO(String),
+    /// An error has occurred while reading input.
+    InputRead(String),
     Parsing,
-    Runtime(String),
+    /// An error has occurred while writing to output.
+    OutputWrite(String),
 }
