@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -63,65 +63,49 @@ pub use testcase::Testcase;
 
 use crate::report::junit::xml::{Element, XmlDocument};
 use crate::report::ReportError;
+use crate::util::path::create_dir_all;
 
 /// Creates a JUnit from a list of `testcases`.
-pub fn write_report(filename: &Path, testcases: &[Testcase]) -> Result<(), ReportError> {
-    // We ensure that parent folder is created.
-    if let Some(parent) = filename.parent() {
-        match std::fs::create_dir_all(parent) {
-            Ok(_) => {}
-            Err(err) => {
-                return Err(ReportError::from_error(
-                    err,
-                    filename,
-                    "Issue writing Junit report",
-                ))
-            }
-        }
-    }
+///
+///  `secrets` strings are redacted from messages (mainly assert error messages).
+pub fn write_report(
+    filename: &Path,
+    testcases: &[Testcase],
+    secrets: &[&str],
+) -> Result<(), ReportError> {
+    create_dir_all(filename)
+        .map_err(|e| ReportError::from_io_error(&e, filename, "Issue writing JUnit report"))?;
 
     // If there is an existing JUnit report, we parse it to insert a new testsuite.
     let mut root = if filename.exists() {
-        let file = match File::open(filename) {
-            Ok(s) => s,
-            Err(e) => {
-                return Err(ReportError::from_error(
-                    e,
-                    filename,
-                    "Issue reading JUnit report",
-                ))
-            }
-        };
+        let file = File::open(filename)
+            .map_err(|e| ReportError::from_io_error(&e, filename, "Issue reading JUnit report"))?;
         let doc = XmlDocument::parse(file).unwrap();
         doc.root.unwrap()
     } else {
         Element::new("testsuites")
     };
 
-    let testsuite = create_testsuite(testcases);
+    let testsuite = create_testsuite(testcases, secrets);
     root = root.add_child(testsuite);
 
     let doc = XmlDocument::new(root);
-    let file = match File::create(filename) {
-        Ok(f) => f,
-        Err(e) => {
-            return Err(ReportError::from_error(
-                e,
-                filename,
-                "Issue writing JUnit report",
-            ))
-        }
-    };
-    match doc.write(file) {
-        Ok(_) => Ok(()),
-        Err(e) => Err(ReportError::from_string(&format!(
-            "Failed to produce Junit report: {e:?}"
-        ))),
-    }
+    let file = File::create(filename)
+        .map_err(|e| ReportError::from_io_error(&e, filename, "Issue writing JUnit report"))?;
+
+    doc.write(file).map_err(|e| {
+        ReportError::from_string(&format!(
+            "Issue writing JUnit report {} {e}",
+            filename.display()
+        ))
+    })?;
+    Ok(())
 }
 
 /// Returns a testsuite as a XML object, from a list of `testcases`.
-fn create_testsuite(testcases: &[Testcase]) -> Element {
+///
+/// `secrets` strings are redacted from the produced HTML.
+fn create_testsuite(testcases: &[Testcase], secrets: &[&str]) -> Element {
     let mut tests = 0;
     let mut errors = 0;
     let mut failures = 0;
@@ -138,7 +122,7 @@ fn create_testsuite(testcases: &[Testcase]) -> Element {
         .attr("failures", &failures.to_string());
 
     for testcase in testcases.iter() {
-        let child = testcase.to_xml();
+        let child = testcase.to_xml(secrets);
         element = element.add_child(child);
     }
     element
@@ -152,7 +136,7 @@ mod tests {
     use hurl_core::input::Input;
     use hurl_core::reader::Pos;
 
-    use crate::http::{CurlCmd, HttpError};
+    use crate::http::HttpError;
     use crate::report::junit::xml::XmlDocument;
     use crate::report::junit::{create_testsuite, Testcase};
     use crate::runner::{EntryResult, HurlResult, RunnerError, RunnerErrorKind};
@@ -162,13 +146,12 @@ mod tests {
         let content = "GET http://localhost:8000/not_found\n\
                        HTTP/1.0 200";
         let filename = Input::new("test.hurl");
+        let secrets = [];
         let mut testcases = vec![];
         let res = HurlResult {
-            entries: vec![],
-            duration: Duration::from_millis(230),
+            duration: Duration::from_millis(124),
             success: true,
-            cookies: vec![],
-            timestamp: 1,
+            ..Default::default()
         };
         let tc = Testcase::from(&res, content, &filename);
         testcases.push(tc);
@@ -177,9 +160,6 @@ mod tests {
             entries: vec![EntryResult {
                 entry_index: 1,
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 35)),
-                calls: vec![],
-                captures: vec![],
-                asserts: vec![],
                 errors: vec![RunnerError::new(
                     SourceInfo::new(Pos::new(2, 10), Pos::new(2, 13)),
                     RunnerErrorKind::AssertStatus {
@@ -187,14 +167,11 @@ mod tests {
                     },
                     true,
                 )],
-                transfer_duration: Duration::from_millis(0),
-                compressed: false,
-                curl_cmd: CurlCmd::default(),
+                ..Default::default()
             }],
-            duration: Duration::from_millis(230),
+            duration: Duration::from_millis(200),
             success: true,
-            cookies: vec![],
-            timestamp: 1,
+            ..Default::default()
         };
         let tc = Testcase::from(&res, content, &filename);
         testcases.push(tc);
@@ -203,9 +180,6 @@ mod tests {
             entries: vec![EntryResult {
                 entry_index: 1,
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 35)),
-                calls: vec![],
-                captures: vec![],
-                asserts: vec![],
                 errors: vec![RunnerError::new(
                     SourceInfo::new(Pos::new(1, 5), Pos::new(1, 19)),
                     RunnerErrorKind::Http(HttpError::Libcurl {
@@ -214,26 +188,23 @@ mod tests {
                     }),
                     false,
                 )],
-                transfer_duration: Duration::from_millis(0),
-                compressed: false,
-                curl_cmd: CurlCmd::default(),
+                ..Default::default()
             }],
             duration: Duration::from_millis(230),
             success: true,
-            cookies: vec![],
-            timestamp: 1,
+            ..Default::default()
         };
         let tc = Testcase::from(&res, content, &filename);
         testcases.push(tc);
 
-        let suite = create_testsuite(&testcases);
+        let suite = create_testsuite(&testcases, &secrets);
         let doc = XmlDocument::new(suite);
         assert_eq!(
-            doc.to_string().unwrap(),
+            doc.dump(),
             "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\
             <testsuite tests=\"3\" errors=\"1\" failures=\"1\">\
-                <testcase id=\"test.hurl\" name=\"test.hurl\" time=\"0.230\" />\
-                <testcase id=\"test.hurl\" name=\"test.hurl\" time=\"0.230\">\
+                <testcase id=\"test.hurl\" name=\"test.hurl\" time=\"0.124\" />\
+                <testcase id=\"test.hurl\" name=\"test.hurl\" time=\"0.200\">\
                     <failure>Assert status code\n  \
                     --&gt; test.hurl:2:10\n   \
                       |\n   \
