@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@ use hurl_core::input::Input;
 
 use crate::report::junit::xml::Element;
 use crate::runner::HurlResult;
+use crate::util::redacted::Redact;
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Testcase {
@@ -40,7 +41,7 @@ impl Testcase {
         let mut errors = vec![];
 
         for (error, entry_src_info) in hurl_result.errors() {
-            let message = error.to_string(
+            let message = error.render(
                 &name,
                 content,
                 Some(entry_src_info),
@@ -62,7 +63,7 @@ impl Testcase {
     }
 
     /// Serializes this testcase to XML.
-    pub fn to_xml(&self) -> Element {
+    pub fn to_xml(&self, secrets: &[&str]) -> Element {
         let time_in_seconds = format!("{:.3}", self.time_in_ms as f64 / 1000.0);
 
         let mut element = Element::new("testcase")
@@ -71,11 +72,13 @@ impl Testcase {
             .attr("time", &time_in_seconds);
 
         for failure in self.failures.iter() {
-            element = element.add_child(Element::new("failure").text(failure));
+            let failure = failure.redact(secrets);
+            element = element.add_child(Element::new("failure").text(&failure));
         }
 
         for error in self.errors.iter() {
-            element = element.add_child(Element::new("error").text(error));
+            let error = error.redact(secrets);
+            element = element.add_child(Element::new("error").text(&error));
         }
         element
     }
@@ -97,7 +100,7 @@ mod test {
     use hurl_core::input::Input;
     use hurl_core::reader::Pos;
 
-    use crate::http::{CurlCmd, HttpError};
+    use crate::http::HttpError;
     use crate::report::junit::testcase::Testcase;
     use crate::report::junit::xml::XmlDocument;
     use crate::runner::{EntryResult, HurlResult, RunnerError, RunnerErrorKind};
@@ -105,19 +108,18 @@ mod test {
     #[test]
     fn test_create_testcase_success() {
         let hurl_result = HurlResult {
-            entries: vec![],
             duration: Duration::from_millis(230),
             success: true,
-            cookies: vec![],
-            timestamp: 1,
+            ..Default::default()
         };
 
         let content = "";
+        let secrets = [];
         let filename = Input::new("test.hurl");
-        let element = Testcase::from(&hurl_result, content, &filename).to_xml();
+        let element = Testcase::from(&hurl_result, content, &filename).to_xml(&secrets);
         let doc = XmlDocument::new(element);
         assert_eq!(
-            doc.to_string().unwrap(),
+            doc.dump(),
             r#"<?xml version="1.0" encoding="UTF-8"?><testcase id="test.hurl" name="test.hurl" time="0.230" />"#
         );
     }
@@ -128,13 +130,11 @@ mod test {
 HTTP/1.0 200
 "#;
         let filename = Input::new("test.hurl");
+        let secrets = [];
         let hurl_result = HurlResult {
             entries: vec![EntryResult {
                 entry_index: 1,
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 35)),
-                calls: vec![],
-                captures: vec![],
-                asserts: vec![],
                 errors: vec![RunnerError::new(
                     SourceInfo::new(Pos::new(2, 10), Pos::new(2, 13)),
                     RunnerErrorKind::AssertStatus {
@@ -142,20 +142,17 @@ HTTP/1.0 200
                     },
                     true,
                 )],
-                transfer_duration: Duration::from_millis(0),
-                compressed: false,
-                curl_cmd: CurlCmd::default(),
+                ..Default::default()
             }],
             duration: Duration::from_millis(230),
-            success: true,
-            cookies: vec![],
-            timestamp: 1,
+            success: false,
+            ..Default::default()
         };
 
-        let element = Testcase::from(&hurl_result, content, &filename).to_xml();
+        let element = Testcase::from(&hurl_result, content, &filename).to_xml(&secrets);
         let doc = XmlDocument::new(element);
         assert_eq!(
-            doc.to_string().unwrap(),
+            doc.dump(),
             r#"<?xml version="1.0" encoding="UTF-8"?><testcase id="test.hurl" name="test.hurl" time="0.230"><failure>Assert status code
   --&gt; test.hurl:2:10
    |
@@ -170,13 +167,11 @@ HTTP/1.0 200
     fn test_create_testcase_error() {
         let content = "GET http://unknown";
         let filename = Input::new("test.hurl");
+        let secrets = ["unknown"];
         let hurl_result = HurlResult {
             entries: vec![EntryResult {
                 entry_index: 1,
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 18)),
-                calls: vec![],
-                captures: vec![],
-                asserts: vec![],
                 errors: vec![RunnerError::new(
                     SourceInfo::new(Pos::new(1, 5), Pos::new(1, 19)),
                     RunnerErrorKind::Http(HttpError::Libcurl {
@@ -185,24 +180,21 @@ HTTP/1.0 200
                     }),
                     false,
                 )],
-                transfer_duration: Duration::from_millis(0),
-                compressed: false,
-                curl_cmd: CurlCmd::default(),
+                ..Default::default()
             }],
             duration: Duration::from_millis(230),
-            success: true,
-            cookies: vec![],
-            timestamp: 1,
+            success: false,
+            ..Default::default()
         };
-        let element = Testcase::from(&hurl_result, content, &filename).to_xml();
+        let element = Testcase::from(&hurl_result, content, &filename).to_xml(&secrets);
         let doc = XmlDocument::new(element);
         assert_eq!(
-            doc.to_string().unwrap(),
+            doc.dump(),
             r#"<?xml version="1.0" encoding="UTF-8"?><testcase id="test.hurl" name="test.hurl" time="0.230"><error>HTTP connection
   --&gt; test.hurl:1:5
    |
- 1 | GET http://unknown
-   |     ^^^^^^^^^^^^^^ (6) Could not resolve host: unknown
+ 1 | GET http://***
+   |     ^^^^^^^^^^^^^^ (6) Could not resolve host: ***
    |</error></testcase>"#
         );
     }
