@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -31,17 +31,14 @@ use crate::util::path::ContextDir;
 
 impl AssertResult {
     /// Evaluates an assert and returns `None` if assert is succeeded or an `Error` if failed.
-    pub fn error(&self) -> Option<RunnerError> {
+    pub fn to_runner_error(&self) -> Option<RunnerError> {
         match self {
-            AssertResult::Version {
+            AssertResult::ImplicitVersion {
                 actual,
                 expected,
                 source_info,
             } => {
-                if expected.as_str() == "HTTP"
-                    || expected.as_str() == "HTTP/*"
-                    || actual == expected
-                {
+                if expected.as_str() == "HTTP" || actual == expected {
                     None
                 } else {
                     let kind = RunnerErrorKind::AssertVersion {
@@ -50,7 +47,7 @@ impl AssertResult {
                     Some(RunnerError::new(*source_info, kind, false))
                 }
             }
-            AssertResult::Status {
+            AssertResult::ImplicitStatus {
                 actual,
                 expected,
                 source_info,
@@ -64,7 +61,7 @@ impl AssertResult {
                     Some(RunnerError::new(*source_info, kind, false))
                 }
             }
-            AssertResult::Header {
+            AssertResult::ImplicitHeader {
                 actual,
                 expected,
                 source_info,
@@ -79,7 +76,7 @@ impl AssertResult {
                     }
                 }
             },
-            AssertResult::Body {
+            AssertResult::ImplicitBody {
                 actual,
                 expected,
                 source_info,
@@ -127,10 +124,10 @@ impl AssertResult {
     }
     pub fn line(&self) -> usize {
         match self {
-            AssertResult::Version { source_info, .. } => source_info.start.line,
-            AssertResult::Status { source_info, .. } => source_info.start.line,
-            AssertResult::Header { source_info, .. } => source_info.start.line,
-            AssertResult::Body { source_info, .. } => source_info.start.line,
+            AssertResult::ImplicitVersion { source_info, .. } => source_info.start.line,
+            AssertResult::ImplicitStatus { source_info, .. } => source_info.start.line,
+            AssertResult::ImplicitHeader { source_info, .. } => source_info.start.line,
+            AssertResult::ImplicitBody { source_info, .. } => source_info.start.line,
             AssertResult::Explicit { source_info, .. } => source_info.start.line,
         }
     }
@@ -144,7 +141,7 @@ fn use_diff(expected: &Value, actual: &Value) -> bool {
     }
 }
 
-/// Evaluates an explicit `assert`, given a set of `variables`, a HTTP response and a context
+/// Evaluates an explicit `assert`, given a set of `variables`, a list of HTTP `responses` and a context
 /// directory `context_dir`.
 ///
 /// The `cache` is used to store XML / JSON structured response data and avoid redundant parsing
@@ -152,11 +149,11 @@ fn use_diff(expected: &Value, actual: &Value) -> bool {
 pub fn eval_explicit_assert(
     assert: &Assert,
     variables: &VariableSet,
-    http_response: &http::Response,
+    http_responses: &[&http::Response],
     cache: &mut BodyCache,
     context_dir: &ContextDir,
 ) -> AssertResult {
-    let query_result = eval_query(&assert.query, variables, http_response, cache);
+    let query_result = eval_query(&assert.query, variables, http_responses, cache);
 
     let actual = if assert.filters.is_empty() {
         query_result
@@ -173,11 +170,7 @@ pub fn eval_explicit_assert(
                 assert: true,
             }),
             Some(value) => {
-                let filters = assert
-                    .filters
-                    .iter()
-                    .map(|(_, f)| f.clone())
-                    .collect::<Vec<_>>();
+                let filters = assert.filters.iter().map(|(_, f)| f).collect::<Vec<_>>();
                 match eval_filters(&filters, &value, variables, true) {
                     Ok(value) => Ok(value),
                     Err(e) => Err(e),
@@ -212,9 +205,10 @@ pub mod tests {
 
     use hurl_core::ast::{
         Filter, FilterValue, LineTerminator, Predicate, PredicateFunc, PredicateFuncValue,
-        PredicateValue, SourceInfo, Whitespace,
+        PredicateValue, SourceInfo, Whitespace, I64,
     };
     use hurl_core::reader::Pos;
+    use hurl_core::typing::ToSource;
 
     use super::super::query;
     use super::*;
@@ -234,8 +228,10 @@ pub mod tests {
                 source_info: SourceInfo::new(Pos::new(1, 22), Pos::new(1, 24)),
                 value: PredicateFuncValue::Equal {
                     space0: whitespace.clone(),
-                    value: PredicateValue::Number(hurl_core::ast::Number::Integer(3)),
-                    operator: true,
+                    value: PredicateValue::Number(hurl_core::ast::Number::Integer(I64::new(
+                        3,
+                        "3".to_source(),
+                    ))),
                 },
             },
         };
@@ -266,15 +262,15 @@ pub mod tests {
     #[test]
     fn test_eval() {
         let variables = VariableSet::new();
-        let current_dir = std::env::current_dir().unwrap();
+        let current_dir = Path::new("/home");
         let file_root = Path::new("file_root");
-        let context_dir = ContextDir::new(current_dir.as_path(), file_root);
+        let context_dir = ContextDir::new(current_dir, file_root);
         let mut cache = BodyCache::new();
         assert_eq!(
             eval_explicit_assert(
                 &assert_count_user(),
                 &variables,
-                &xml_three_users_http_response(),
+                &[&xml_three_users_http_response()],
                 &mut cache,
                 &context_dir
             ),
