@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -93,11 +93,12 @@ mod tests {
 
     use hurl_core::ast::{SourceInfo, TemplateElement, Whitespace};
     use hurl_core::reader::Pos;
+    use hurl_core::typing::ToSource;
 
     use super::*;
 
     #[test]
-    pub fn test_body_file() {
+    fn test_body_file() {
         // file, data.bin;
         let whitespace = Whitespace {
             value: String::from(" "),
@@ -106,21 +107,21 @@ mod tests {
 
         let bytes = Bytes::File(File {
             space0: whitespace.clone(),
-            filename: Template {
-                delimiter: None,
-                source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 15)),
-                elements: vec![TemplateElement::String {
+            filename: Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "tests/data.bin".to_string(),
-                    encoded: "tests/data.bin".to_string(),
+                    source: "tests/data.bin".to_source(),
                 }],
-            },
+                SourceInfo::new(Pos::new(1, 7), Pos::new(1, 15)),
+            ),
             space1: whitespace,
         });
 
         let variables = VariableSet::new();
-        let current_dir = std::env::current_dir().unwrap();
+        let current_dir = Path::new("/home");
         let file_root = Path::new("");
-        let context_dir = ContextDir::new(current_dir.as_path(), file_root);
+        let context_dir = ContextDir::new(current_dir, file_root);
         assert_eq!(
             eval_bytes(&bytes, &variables, &context_dir).unwrap(),
             http::Body::File(b"Hello World!".to_vec(), "tests/data.bin".to_string())
@@ -128,7 +129,7 @@ mod tests {
     }
 
     #[test]
-    pub fn test_body_file_error() {
+    fn test_body_file_error() {
         // file, data.bin;
         let whitespace = Whitespace {
             value: String::from(" "),
@@ -137,22 +138,22 @@ mod tests {
 
         let bytes = Bytes::File(File {
             space0: whitespace.clone(),
-            filename: Template {
-                delimiter: None,
-                source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 15)),
-                elements: vec![TemplateElement::String {
+            filename: Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "data.bin".to_string(),
-                    encoded: "data.bin".to_string(),
+                    source: "data.bin".to_source(),
                 }],
-            },
+                SourceInfo::new(Pos::new(1, 7), Pos::new(1, 15)),
+            ),
             space1: whitespace,
         });
 
         let variables = VariableSet::new();
 
-        let current_dir = std::env::current_dir().unwrap();
+        let current_dir = Path::new("/home");
         let file_root = Path::new("file_root");
-        let context_dir = ContextDir::new(current_dir.as_path(), file_root);
+        let context_dir = ContextDir::new(current_dir, file_root);
         let error = eval_bytes(&bytes, &variables, &context_dir).err().unwrap();
         assert_eq!(
             error.kind,
