@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
  * limitations under the License.
  *
  */
-use hurl_core::ast::Capture;
+use hurl_core::ast::{Capture, SourceInfo};
 
 use crate::http;
 use crate::runner::cache::BodyCache;
@@ -26,7 +26,7 @@ use crate::runner::result::CaptureResult;
 use crate::runner::template::eval_template;
 use crate::runner::VariableSet;
 
-/// Evaluates a `capture` with `variables` map and `http_response`, returns a
+/// Evaluates a `capture` with `variables` map and a list of `http_responses`, returns a
 /// [`CaptureResult`] on success or an [`RunnerError`].
 ///
 /// The `cache` is used to store XML / JSON structured response data and avoid redundant parsing
@@ -34,12 +34,31 @@ use crate::runner::VariableSet;
 pub fn eval_capture(
     capture: &Capture,
     variables: &VariableSet,
-    http_response: &http::Response,
+    http_responses: &[&http::Response],
     cache: &mut BodyCache,
 ) -> Result<CaptureResult, RunnerError> {
     let name = eval_template(&capture.name, variables)?;
-    let value = eval_query(&capture.query, variables, http_response, cache)?;
+    let value = eval_query(&capture.query, variables, http_responses, cache)?;
     let value = match value {
+        Some(value) => {
+            let filters = capture.filters.iter().map(|(_, f)| f).collect::<Vec<_>>();
+            match eval_filters(&filters, &value, variables, false)? {
+                None => {
+                    // If we have an error, we can be sure that there is at least one filter.
+                    // We don't know which filter in the filter chain firstly returns no value
+                    // so we diagnostic the whole filter chain as guilty.
+                    let start = filters.first().unwrap().source_info.start;
+                    let end = filters.last().unwrap().source_info.end;
+                    let pos = SourceInfo::new(start, end);
+                    return Err(RunnerError::new(
+                        pos,
+                        RunnerErrorKind::NoFilterResult,
+                        false,
+                    ));
+                }
+                Some(v) => v,
+            }
+        }
         None => {
             return Err(RunnerError::new(
                 capture.query.source_info,
@@ -47,23 +66,6 @@ pub fn eval_capture(
                 false,
             ));
         }
-        Some(value) => {
-            let filters = capture
-                .filters
-                .iter()
-                .map(|(_, f)| f.clone())
-                .collect::<Vec<_>>();
-            match eval_filters(&filters, &value, variables, false)? {
-                None => {
-                    return Err(RunnerError::new(
-                        capture.query.source_info,
-                        RunnerErrorKind::NoQueryResult,
-                        false,
-                    ));
-                }
-                Some(v) => v,
-            }
-        }
     };
 
     Ok(CaptureResult {
@@ -78,6 +80,7 @@ pub mod tests {
         LineTerminator, Query, QueryValue, SourceInfo, Template, TemplateElement, Whitespace,
     };
     use hurl_core::reader::Pos;
+    use hurl_core::typing::ToSource;
 
     use self::super::super::query;
     use super::*;
@@ -92,20 +95,22 @@ pub mod tests {
         Capture {
             line_terminators: vec![],
             space0: whitespace.clone(),
-            name: Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            name: Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "UserCount".to_string(),
-                    encoded: "UserCount".to_string(),
+                    source: "UserCount".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-            },
+                SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+            ),
             space1: whitespace.clone(),
             space2: whitespace.clone(),
 
             // xpath count(//user)
             query: query::tests::xpath_count_user_query(),
             filters: vec![],
+            space3: whitespace.clone(),
+            redacted: false,
             line_terminator0: LineTerminator {
                 space0: whitespace.clone(),
                 comment: None,
@@ -123,20 +128,22 @@ pub mod tests {
         Capture {
             line_terminators: vec![],
             space0: whitespace.clone(),
-            name: Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            name: Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "duration".to_string(),
-                    encoded: "duration".to_string(),
+                    source: "duration".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-            },
+                SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+            ),
             space1: whitespace.clone(),
             space2: whitespace.clone(),
 
             // xpath count(//user)
             query: query::tests::jsonpath_duration(),
             filters: vec![],
+            space3: whitespace.clone(),
+            redacted: false,
             line_terminator0: LineTerminator {
                 space0: whitespace.clone(),
                 comment: None,
@@ -156,19 +163,21 @@ pub mod tests {
         let capture = Capture {
             line_terminators: vec![],
             space0: whitespace.clone(),
-            name: Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            name: Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "count".to_string(),
-                    encoded: "count".to_string(),
+                    source: "count".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-            },
+                SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+            ),
             filters: vec![],
             space1: whitespace.clone(),
             space2: whitespace.clone(),
 
             query: query::tests::xpath_invalid_query(),
+            space3: whitespace.clone(),
+            redacted: false,
             line_terminator0: LineTerminator {
                 space0: whitespace.clone(),
                 comment: None,
@@ -179,13 +188,13 @@ pub mod tests {
         let error = eval_capture(
             &capture,
             &variables,
-            &http::xml_three_users_http_response(),
+            &[&http::xml_three_users_http_response()],
             &mut cache,
         )
         .err()
         .unwrap();
         assert_eq!(error.source_info.start, Pos { line: 1, column: 7 });
-        assert_eq!(error.kind, RunnerErrorKind::QueryInvalidXpathEval);
+        assert_eq!(error.kind, RunnerErrorKind::InvalidXPathEval);
     }
 
     #[test]
@@ -198,14 +207,14 @@ pub mod tests {
         let _capture = Capture {
             line_terminators: vec![],
             space0: whitespace.clone(),
-            name: Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            name: Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "???".to_string(),
-                    encoded: "???".to_string(),
+                    source: "???".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-            },
+                SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+            ),
             space1: whitespace.clone(),
             space2: whitespace.clone(),
 
@@ -214,17 +223,19 @@ pub mod tests {
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 13)),
                 value: QueryValue::Xpath {
                     space0: whitespace.clone(),
-                    expr: Template {
-                        delimiter: Some('"'),
-                        elements: vec![TemplateElement::String {
+                    expr: Template::new(
+                        Some('"'),
+                        vec![TemplateElement::String {
                             value: "//user".to_string(),
-                            encoded: "//user".to_string(),
+                            source: "//user".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 13)),
-                    },
+                        SourceInfo::new(Pos::new(1, 7), Pos::new(1, 13)),
+                    ),
                 },
             },
             filters: vec![],
+            space3: whitespace.clone(),
+            redacted: false,
             line_terminator0: LineTerminator {
                 space0: whitespace.clone(),
                 comment: None,
@@ -242,7 +253,7 @@ pub mod tests {
             eval_capture(
                 &user_count_capture(),
                 &variables,
-                &http::xml_three_users_http_response(),
+                &[&http::xml_three_users_http_response()],
                 &mut cache,
             )
             .unwrap(),
@@ -256,7 +267,7 @@ pub mod tests {
             eval_capture(
                 &duration_capture(),
                 &variables,
-                &http::json_http_response(),
+                &[&http::json_http_response()],
                 &mut cache
             )
             .unwrap(),
