@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
  * limitations under the License.
  *
  */
-use hurl_core::ast::{Entry, SourceInfo};
+use hurl_core::ast::{Entry, PredicateFuncValue, Response, SourceInfo};
 
 use crate::http;
 use crate::http::{ClientOptions, CurlCmd};
@@ -25,6 +25,7 @@ use crate::runner::result::{AssertResult, EntryResult};
 use crate::runner::runner_options::RunnerOptions;
 use crate::runner::{request, response, CaptureResult, RunnerErrorKind, VariableSet};
 use crate::util::logger::{Logger, Verbosity};
+use crate::util::term::WriteMode;
 
 /// Runs an `entry` with `http_client` and returns one [`EntryResult`].
 ///
@@ -44,6 +45,29 @@ pub fn run(
     let source_info = entry.source_info();
     let context_dir = &runner_options.context_dir;
 
+    // We don't allow creating secrets if the logger is immediate and verbose because, in this case,
+    // network logs have already been written and may have leaked secrets before captures evaluation.
+    // Note: in `--test` mode, the logger is buffered so there is no restriction on logger level.
+    if let Some(response_spec) = &entry.response {
+        let immediate_logs =
+            matches!(logger.stderr.mode(), WriteMode::Immediate) && logger.verbosity.is_some();
+        if immediate_logs {
+            let redacted = response_spec.captures().iter().find(|c| c.redacted);
+            if let Some(redacted) = redacted {
+                let source_info = redacted.name.source_info;
+                let error =
+                    RunnerError::new(source_info, RunnerErrorKind::PossibleLoggedSecret, false);
+                return EntryResult {
+                    entry_index,
+                    source_info,
+                    errors: vec![error],
+                    compressed,
+                    ..Default::default()
+                };
+            }
+        }
+    }
+
     // Evaluates our source requests given our set of variables
     let http_request = match request::eval_request(&entry.request, variables, context_dir) {
         Ok(r) => r,
@@ -57,19 +81,20 @@ pub fn run(
             };
         }
     };
+
     let client_options = ClientOptions::from(runner_options, logger.verbosity);
 
     // Experimental features with cookie storage
     use std::str::FromStr;
     if let Some(s) = request::cookie_storage_set(&entry.request) {
         if let Ok(cookie) = http::Cookie::from_str(s.as_str()) {
-            http_client.add_cookie(&cookie, &client_options);
+            http_client.add_cookie(&cookie, logger);
         } else {
             logger.warning(&format!("Cookie string can not be parsed: '{s}'"));
         }
     }
     if request::cookie_storage_clear(&entry.request) {
-        http_client.clear_cookie_storage(&client_options);
+        http_client.clear_cookie_storage(logger);
     }
 
     let curl_cmd = http_client.curl_command_line(
@@ -77,6 +102,7 @@ pub fn run(
         context_dir,
         runner_options.output.as_ref(),
         &client_options,
+        logger,
     );
 
     log_request(http_client, &curl_cmd, &http_request, logger);
@@ -102,8 +128,8 @@ pub fn run(
     };
 
     // Now, we can compute capture and asserts on the last HTTP request/response chains.
-    let call = calls.last().unwrap();
-    let http_response = &call.response;
+    let responses = calls.iter().map(|c| &c.response).collect::<Vec<_>>();
+    let http_response = responses.last().unwrap();
 
     // `transfer_duration` represent the network time of calls, not including assert processing.
     let transfer_duration = calls.iter().map(|call| call.timings.total).sum();
@@ -141,7 +167,7 @@ pub fn run(
     let captures = match &entry.response {
         None => vec![],
         Some(response_spec) => {
-            match response::eval_captures(response_spec, http_response, &mut cache, variables) {
+            match response::eval_captures(response_spec, &responses, &mut cache, variables) {
                 Ok(captures) => captures,
                 Err(e) => {
                     return EntryResult {
@@ -159,16 +185,22 @@ pub fn run(
             }
         }
     };
+
+    // After captures evaluation, we update the logger with secrets from the variable set. The variable
+    // set can have been updated with new secrets to redact.
+    logger.set_secrets(variables.secrets());
+
     log_captures(&captures, logger);
     logger.debug("");
 
     // Compute asserts
     if !runner_options.ignore_asserts {
         if let Some(response_spec) = &entry.response {
+            warn_deprecated(response_spec, logger);
             let mut other_asserts = response::eval_asserts(
                 response_spec,
                 variables,
-                http_response,
+                &responses,
                 &mut cache,
                 context_dir,
             );
@@ -195,7 +227,7 @@ pub fn run(
 fn asserts_to_errors(asserts: &[AssertResult]) -> Vec<RunnerError> {
     asserts
         .iter()
-        .filter_map(|assert| assert.error())
+        .filter_map(|assert| assert.to_runner_error())
         .map(
             |RunnerError {
                  source_info,
@@ -209,6 +241,7 @@ fn asserts_to_errors(asserts: &[AssertResult]) -> Vec<RunnerError> {
 impl ClientOptions {
     fn from(runner_options: &RunnerOptions, verbosity: Option<Verbosity>) -> Self {
         ClientOptions {
+            allow_reuse: runner_options.allow_reuse,
             aws_sigv4: runner_options.aws_sigv4.clone(),
             cacert_file: runner_options.cacert_file.clone(),
             client_cert_file: runner_options.client_cert_file.clone(),
@@ -219,16 +252,20 @@ impl ClientOptions {
             cookie_input_file: runner_options.cookie_input_file.clone(),
             follow_location: runner_options.follow_location,
             follow_location_trusted: runner_options.follow_location_trusted,
+            headers: runner_options.headers.clone(),
             http_version: runner_options.http_version,
             ip_resolve: runner_options.ip_resolve,
             max_filesize: runner_options.max_filesize,
             max_recv_speed: runner_options.max_recv_speed,
             max_redirect: runner_options.max_redirect,
             max_send_speed: runner_options.max_send_speed,
+            negotiate: runner_options.negotiate,
             netrc: runner_options.netrc,
             netrc_file: runner_options.netrc_file.clone(),
             netrc_optional: runner_options.netrc_optional,
+            ntlm: runner_options.ntlm,
             path_as_is: runner_options.path_as_is,
+            pinned_pub_key: runner_options.pinned_pub_key.clone(),
             proxy: runner_options.proxy.clone(),
             no_proxy: runner_options.no_proxy.clone(),
             insecure: runner_options.insecure,
@@ -256,7 +293,7 @@ fn log_request(
 ) {
     logger.debug("");
     logger.debug_important("Cookie store:");
-    for cookie in &http_client.cookie_storage() {
+    for cookie in &http_client.cookie_storage(logger) {
         logger.debug(&cookie.to_string());
     }
 
@@ -306,3 +343,15 @@ fn log_captures(captures: &[CaptureResult], logger: &mut Logger) {
         logger.capture(&c.name, &c.value);
     }
 }
+
+/// Warns some deprecation on this `response`.
+fn warn_deprecated(response_spec: &Response, logger: &mut Logger) {
+    if response_spec.asserts().iter().any(|a| {
+        matches!(
+            &a.predicate.predicate_func.value,
+            PredicateFuncValue::Include { .. }
+        )
+    }) {
+        logger.warning("<includes> predicate is now deprecated in favor of <contains> predicate");
+    }
+}
