@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -86,7 +86,9 @@ pub enum RunnerErrorKind {
     },
     FilterDecode(String),
     FilterInvalidEncoding(String),
+    /// Input of the filter is not valid, with a given reason.
     FilterInvalidInput(String),
+    FilterInvalidFormatSpecifier(String),
     FilterMissingInput,
     Http(HttpError),
     InvalidJson {
@@ -97,27 +99,31 @@ pub enum RunnerErrorKind {
         url: String,
         message: String,
     },
+    /// A XPath expression evaluation raised an error.
+    InvalidXPathEval,
+    /// One filter in the filter chains doesn't return value.
+    NoFilterResult,
+    /// A query on response doesn't return value.
     NoQueryResult,
+    PossibleLoggedSecret,
     QueryHeaderNotFound,
     QueryInvalidJsonpathExpression {
         value: String,
     },
-    QueryInvalidXpathEval,
     QueryInvalidXml,
     QueryInvalidJson,
-    ReadOnlySecret {
-        name: String,
-    },
     TemplateVariableNotDefined {
         name: String,
     },
-    UnrenderableExpression {
-        value: String,
-    },
     /// Unauthorized file access, check `--file-root` option.
     UnauthorizedFileAccess {
         path: PathBuf,
     },
+    /// Only string secrets are supported.
+    UnsupportedSecretType(String),
+    UnrenderableExpression {
+        value: String,
+    },
 }
 
 /// Textual Output for runner errors
@@ -140,25 +146,28 @@ impl DisplaySourceError for RunnerError {
             RunnerErrorKind::FilterDecode { .. } => "Filter error".to_string(),
             RunnerErrorKind::FilterInvalidEncoding { .. } => "Filter error".to_string(),
             RunnerErrorKind::FilterInvalidInput { .. } => "Filter error".to_string(),
+            RunnerErrorKind::FilterInvalidFormatSpecifier { .. } => "Filter error".to_string(),
             RunnerErrorKind::FilterMissingInput => "Filter error".to_string(),
             RunnerErrorKind::Http(http_error) => http_error.description(),
             RunnerErrorKind::InvalidJson { .. } => "Invalid JSON".to_string(),
-            RunnerErrorKind::InvalidUrl { .. } => "Invalid URL".to_string(),
             RunnerErrorKind::InvalidRegex => "Invalid regex".to_string(),
+            RunnerErrorKind::InvalidUrl { .. } => "Invalid URL".to_string(),
+            RunnerErrorKind::InvalidXPathEval => "Invalid XPath expression".to_string(),
+            RunnerErrorKind::NoFilterResult => "Filter error".to_string(),
             RunnerErrorKind::NoQueryResult => "No query result".to_string(),
+            RunnerErrorKind::PossibleLoggedSecret => "Invalid redacted secret".to_string(),
             RunnerErrorKind::QueryHeaderNotFound => "Header not found".to_string(),
             RunnerErrorKind::QueryInvalidJson => "Invalid JSON".to_string(),
             RunnerErrorKind::QueryInvalidJsonpathExpression { .. } => {
                 "Invalid JSONPath".to_string()
             }
             RunnerErrorKind::QueryInvalidXml => "Invalid XML".to_string(),
-            RunnerErrorKind::QueryInvalidXpathEval => "Invalid XPath expression".to_string(),
-            RunnerErrorKind::ReadOnlySecret { .. } => "Readonly secret".to_string(),
             RunnerErrorKind::TemplateVariableNotDefined { .. } => "Undefined variable".to_string(),
             RunnerErrorKind::UnauthorizedFileAccess { .. } => {
                 "Unauthorized file access".to_string()
             }
             RunnerErrorKind::UnrenderableExpression { .. } => "Unrenderable expression".to_string(),
+            RunnerErrorKind::UnsupportedSecretType(_) => "Invalid secret type".to_string(),
         }
     }
 
@@ -234,8 +243,13 @@ impl DisplaySourceError for RunnerError {
                 let message = error::add_carets(message, self.source_info, content);
                 color_red_multiline_string(&message)
             }
-            RunnerErrorKind::FilterInvalidInput(message) => {
-                let message = &format!("invalid filter input: {message}");
+            RunnerErrorKind::FilterInvalidInput(reason) => {
+                let message = &format!("invalid filter input: {reason}");
+                let message = error::add_carets(message, self.source_info, content);
+                color_red_multiline_string(&message)
+            }
+            RunnerErrorKind::FilterInvalidFormatSpecifier(format) => {
+                let message = &format!("date format <{format}> is not supported");
                 let message = error::add_carets(message, self.source_info, content);
                 color_red_multiline_string(&message)
             }
@@ -254,18 +268,33 @@ impl DisplaySourceError for RunnerError {
                 let message = error::add_carets(message, self.source_info, content);
                 color_red_multiline_string(&message)
             }
-            RunnerErrorKind::InvalidUrl { url, message } => {
-                let message = &format!("invalid URL <{url}> ({message})");
-                let message = error::add_carets(message, self.source_info, content);
-                color_red_multiline_string(&message)
-            }
             RunnerErrorKind::InvalidRegex => {
                 let message = "regex expression is not valid";
                 let message = error::add_carets(message, self.source_info, content);
                 color_red_multiline_string(&message)
             }
+            RunnerErrorKind::InvalidUrl { url, message } => {
+                let message = &format!("invalid URL <{url}> ({message})");
+                let message = error::add_carets(message, self.source_info, content);
+                color_red_multiline_string(&message)
+            }
+            RunnerErrorKind::InvalidXPathEval => {
+                let message = "XPath expression is not valid";
+                let message = error::add_carets(message, self.source_info, content);
+                color_red_multiline_string(&message)
+            }
+            RunnerErrorKind::NoFilterResult => {
+                let message = "a filter didn't return any result";
+                let message = error::add_carets(message, self.source_info, content);
+                color_red_multiline_string(&message)
+            }
             RunnerErrorKind::NoQueryResult => {
-                let message = "The query didn't return any result";
+                let message = "query didn't return any result";
+                let message = error::add_carets(message, self.source_info, content);
+                color_red_multiline_string(&message)
+            }
+            RunnerErrorKind::PossibleLoggedSecret => {
+                let message = "redacted secret not authorized in verbose";
                 let message = error::add_carets(message, self.source_info, content);
                 color_red_multiline_string(&message)
             }
@@ -275,27 +304,17 @@ impl DisplaySourceError for RunnerError {
                 color_red_multiline_string(&message)
             }
             RunnerErrorKind::QueryInvalidJson => {
-                let message = "the HTTP response is not a valid JSON";
+                let message = "HTTP response is not a valid JSON";
                 let message = error::add_carets(message, self.source_info, content);
                 color_red_multiline_string(&message)
             }
             RunnerErrorKind::QueryInvalidJsonpathExpression { value } => {
-                let message = &format!("the JSONPath expression '{value}' is not valid");
+                let message = &format!("JSONPath expression '{value}' is not valid");
                 let message = error::add_carets(message, self.source_info, content);
                 color_red_multiline_string(&message)
             }
             RunnerErrorKind::QueryInvalidXml => {
-                let message = "the HTTP response is not a valid XML";
-                let message = error::add_carets(message, self.source_info, content);
-                color_red_multiline_string(&message)
-            }
-            RunnerErrorKind::QueryInvalidXpathEval => {
-                let message = "the XPath expression is not valid";
-                let message = error::add_carets(message, self.source_info, content);
-                color_red_multiline_string(&message)
-            }
-            RunnerErrorKind::ReadOnlySecret { name } => {
-                let message = &format!("secret '{name}' can't be reassigned");
+                let message = "HTTP response is not a valid XML";
                 let message = error::add_carets(message, self.source_info, content);
                 color_red_multiline_string(&message)
             }
@@ -317,6 +336,11 @@ impl DisplaySourceError for RunnerError {
                 let message = error::add_carets(message, self.source_info, content);
                 color_red_multiline_string(&message)
             }
+            RunnerErrorKind::UnsupportedSecretType(kind) => {
+                let message = &format!("secret must be string, actual value is <{kind}>");
+                let message = error::add_carets(message, self.source_info, content);
+                color_red_multiline_string(&message)
+            }
         }
     }
 
@@ -431,7 +455,7 @@ mod tests {
             "\n 1 | GET http://unknown\n   |     ^^^^^^^^^^^^^^ (6) Could not resolve host: unknown\n   |"
         );
         assert_eq!(
-            error.to_string(
+            error.render(
                 filename,
                 content,
                 Some(entry_source_info),
@@ -448,6 +472,7 @@ mod tests {
 
     #[test]
     fn test_assert_error_status() {
+        // For the crate colored to output ANSI escape code in test environment.
         hurl_core::text::init_crate_colored();
 
         let content = r#"GET http://unknown
@@ -472,7 +497,7 @@ HTTP/1.0 200
         );
 
         assert_eq!(
-            error.to_string(
+            error.render(
                 filename,
                 content,
                 Some(entry_source_info),
@@ -499,17 +524,13 @@ xpath "strong(//head/title)" == "Hello"
         let filename = "test.hurl";
         let error_source_info = SourceInfo::new(Pos::new(4, 7), Pos::new(4, 29));
         let entry_source_info = SourceInfo::new(Pos::new(1, 1), Pos::new(1, 22));
-        let error = RunnerError::new(
-            error_source_info,
-            RunnerErrorKind::QueryInvalidXpathEval,
-            true,
-        );
+        let error = RunnerError::new(error_source_info, RunnerErrorKind::InvalidXPathEval, true);
         assert_eq!(
         &error.message(&lines).to_string(Format::Plain),
-        "\n 4 | xpath \"strong(//head/title)\" == \"Hello\"\n   |       ^^^^^^^^^^^^^^^^^^^^^^ the XPath expression is not valid\n   |"
+        "\n 4 | xpath \"strong(//head/title)\" == \"Hello\"\n   |       ^^^^^^^^^^^^^^^^^^^^^^ XPath expression is not valid\n   |"
     );
         assert_eq!(
-            error.to_string(
+            error.render(
                 filename,
                 content,
                 Some(entry_source_info),
@@ -521,7 +542,7 @@ xpath "strong(//head/title)" == "Hello"
    | GET http://example.com
    | ...
  4 | xpath "strong(//head/title)" == "Hello"
-   |       ^^^^^^^^^^^^^^^^^^^^^^ the XPath expression is not valid
+   |       ^^^^^^^^^^^^^^^^^^^^^^ XPath expression is not valid
    |"#
         );
     }
@@ -540,8 +561,8 @@ jsonpath "$.count" >= 5
         let error = RunnerError {
             source_info: error_source_info,
             kind: RunnerErrorKind::AssertFailure {
-                actual: "int <2>".to_string(),
-                expected: "greater than int <5>".to_string(),
+                actual: "integer <2>".to_string(),
+                expected: "greater than integer <5>".to_string(),
                 type_mismatch: false,
             },
             assert: true,
@@ -551,13 +572,13 @@ jsonpath "$.count" >= 5
             error.message(&lines).to_string(Format::Plain),
             r#"
  4 | jsonpath "$.count" >= 5
-   |   actual:   int <2>
-   |   expected: greater than int <5>
+   |   actual:   integer <2>
+   |   expected: greater than integer <5>
    |"#
         );
 
         assert_eq!(
-            error.to_string(
+            error.render(
                 filename,
                 content,
                 Some(entry_source_info),
@@ -569,8 +590,8 @@ jsonpath "$.count" >= 5
    | GET http://api
    | ...
  4 | jsonpath "$.count" >= 5
-   |   actual:   int <2>
-   |   expected: greater than int <5>
+   |   actual:   integer <2>
+   |   expected: greater than integer <5>
    |"#
         );
     }
@@ -598,7 +619,7 @@ HTTP/1.0 200
             "\n 4 | <p>Hello</p>\n   |   +\n   |"
         );
         assert_eq!(
-            error.to_string(
+            error.render(
                 filename,
                 content,
                 Some(entry_source_info),
