@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,30 +17,41 @@
  */
 use hurl_core::ast::{Filter, FilterValue};
 
-use super::count::eval_count;
+use crate::runner::filter::base64_decode::eval_base64_decode;
+use crate::runner::filter::base64_encode::eval_base64_encode;
+use crate::runner::filter::base64_url_safe_decode::eval_base64_url_safe_decode;
+use crate::runner::filter::base64_url_safe_encode::eval_base64_url_safe_encode;
+use crate::runner::filter::count::eval_count;
 use crate::runner::filter::days_after_now::eval_days_after_now;
 use crate::runner::filter::days_before_now::eval_days_before_now;
 use crate::runner::filter::decode::eval_decode;
+use crate::runner::filter::first::eval_first;
 use crate::runner::filter::format::eval_format;
 use crate::runner::filter::html_escape::eval_html_escape;
 use crate::runner::filter::html_unescape::eval_html_unescape;
 use crate::runner::filter::jsonpath::eval_jsonpath;
+use crate::runner::filter::last::eval_last;
+use crate::runner::filter::location::eval_location;
 use crate::runner::filter::nth::eval_nth;
 use crate::runner::filter::regex::eval_regex;
 use crate::runner::filter::replace::eval_replace;
+use crate::runner::filter::replace_regex::eval_replace_regex;
 use crate::runner::filter::split::eval_split;
 use crate::runner::filter::to_date::eval_to_date;
 use crate::runner::filter::to_float::eval_to_float;
+use crate::runner::filter::to_hex::eval_to_hex;
 use crate::runner::filter::to_int::eval_to_int;
+use crate::runner::filter::to_string::eval_to_string;
 use crate::runner::filter::url_decode::eval_url_decode;
 use crate::runner::filter::url_encode::eval_url_encode;
+use crate::runner::filter::url_query_param::eval_url_query_param;
 use crate::runner::filter::xpath::eval_xpath;
 use crate::runner::{RunnerError, RunnerErrorKind, Value, VariableSet};
 
 /// Apply successive `filter` to an input `value`.
 /// Specify whether they are executed  `in_assert` or not.
 pub fn eval_filters(
-    filters: &[Filter],
+    filters: &[&Filter],
     value: &Value,
     variables: &VariableSet,
     in_assert: bool,
@@ -68,12 +79,21 @@ pub fn eval_filter(
     in_assert: bool,
 ) -> Result<Option<Value>, RunnerError> {
     match &filter.value {
+        FilterValue::Base64Decode => eval_base64_decode(value, filter.source_info, in_assert),
+        FilterValue::Base64Encode => eval_base64_encode(value, filter.source_info, in_assert),
+        FilterValue::Base64UrlSafeDecode => {
+            eval_base64_url_safe_decode(value, filter.source_info, in_assert)
+        }
+        FilterValue::Base64UrlSafeEncode => {
+            eval_base64_url_safe_encode(value, filter.source_info, in_assert)
+        }
         FilterValue::Count => eval_count(value, filter.source_info, in_assert),
         FilterValue::DaysAfterNow => eval_days_after_now(value, filter.source_info, in_assert),
         FilterValue::DaysBeforeNow => eval_days_before_now(value, filter.source_info, in_assert),
         FilterValue::Decode { encoding, .. } => {
             eval_decode(value, encoding, variables, filter.source_info, in_assert)
         }
+        FilterValue::First => eval_first(value, filter.source_info, in_assert),
         FilterValue::Format { fmt, .. } => {
             eval_format(value, fmt, variables, filter.source_info, in_assert)
         }
@@ -82,10 +102,12 @@ pub fn eval_filter(
         FilterValue::JsonPath { expr, .. } => {
             eval_jsonpath(value, expr, variables, filter.source_info, in_assert)
         }
+        FilterValue::Last => eval_last(value, filter.source_info, in_assert),
+        FilterValue::Location => eval_location(value, filter.source_info, in_assert),
         FilterValue::Regex {
             value: regex_value, ..
         } => eval_regex(value, regex_value, variables, filter.source_info, in_assert),
-        FilterValue::Nth { n, .. } => eval_nth(value, filter.source_info, in_assert, *n),
+        FilterValue::Nth { n, .. } => eval_nth(value, n, variables, filter.source_info, in_assert),
         FilterValue::Replace {
             old_value,
             new_value,
@@ -98,6 +120,16 @@ pub fn eval_filter(
             old_value,
             new_value,
         ),
+        FilterValue::ReplaceRegex {
+            pattern, new_value, ..
+        } => eval_replace_regex(
+            value,
+            variables,
+            filter.source_info,
+            in_assert,
+            pattern,
+            new_value,
+        ),
         FilterValue::Split { sep, .. } => {
             eval_split(value, variables, filter.source_info, in_assert, sep)
         }
@@ -105,9 +137,14 @@ pub fn eval_filter(
             eval_to_date(value, fmt, variables, filter.source_info, in_assert)
         }
         FilterValue::ToFloat => eval_to_float(value, filter.source_info, in_assert),
+        FilterValue::ToHex => eval_to_hex(value, filter.source_info, in_assert),
         FilterValue::ToInt => eval_to_int(value, filter.source_info, in_assert),
+        FilterValue::ToString => eval_to_string(value, filter.source_info, in_assert),
         FilterValue::UrlDecode => eval_url_decode(value, filter.source_info, in_assert),
         FilterValue::UrlEncode => eval_url_encode(value, filter.source_info, in_assert),
+        FilterValue::UrlQueryParam { param, .. } => {
+            eval_url_query_param(value, param, variables, filter.source_info, in_assert)
+        }
         FilterValue::XPath { expr, .. } => {
             eval_xpath(value, expr, variables, filter.source_info, in_assert)
         }
@@ -115,7 +152,7 @@ pub fn eval_filter(
 }
 
 #[cfg(test)]
-pub mod tests {
+mod tests {
     use hurl_core::ast::{Filter, FilterValue, SourceInfo};
     use hurl_core::reader::Pos;
 
@@ -123,12 +160,12 @@ pub mod tests {
     use crate::runner::{Number, Value, VariableSet};
 
     #[test]
-    pub fn test_filters() {
+    fn test_filters() {
         let variables = VariableSet::new();
 
         assert_eq!(
             eval_filters(
-                &[Filter {
+                &[&Filter {
                     source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6)),
                     value: FilterValue::Count,
                 }],
