@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,22 +20,33 @@ pub use eval::eval_filters;
 pub use jsonpath::eval_jsonpath_json;
 pub use xpath::eval_xpath_doc;
 
+mod base64_decode;
+mod base64_encode;
+mod base64_url_safe_decode;
+mod base64_url_safe_encode;
 mod count;
 mod days_after_now;
 mod days_before_now;
 mod decode;
 mod eval;
+mod first;
 mod format;
 mod html_escape;
 mod html_unescape;
 mod jsonpath;
+mod last;
+mod location;
 mod nth;
 mod regex;
 mod replace;
+mod replace_regex;
 mod split;
 mod to_date;
 mod to_float;
+mod to_hex;
 mod to_int;
+mod to_string;
 mod url_decode;
 mod url_encode;
+mod url_query_param;
 mod xpath;
