@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,52 +20,53 @@ use hurl_core::ast::{SourceInfo, Template};
 use crate::runner::template::eval_template;
 use crate::runner::{RunnerError, RunnerErrorKind, Value, VariableSet};
 
+/// Splits the string `value` to a list of strings around occurrences of the specified `delimiter`.
 pub fn eval_split(
     value: &Value,
     variables: &VariableSet,
     source_info: SourceInfo,
     assert: bool,
-    sep: &Template,
+    delimiter: &Template,
 ) -> Result<Option<Value>, RunnerError> {
     match value {
         Value::String(s) => {
-            let sep = eval_template(sep, variables)?;
+            let delimiter = eval_template(delimiter, variables)?;
             let values = s
-                .split(&sep)
+                .split(&delimiter)
                 .map(|v| Value::String(v.to_string()))
                 .collect();
             Ok(Some(Value::List(values)))
         }
         v => {
-            let kind = RunnerErrorKind::FilterInvalidInput(v.display());
+            let kind = RunnerErrorKind::FilterInvalidInput(v.repr());
             Err(RunnerError::new(source_info, kind, assert))
         }
     }
 }
 
 #[cfg(test)]
-pub mod tests {
-
+mod tests {
     use hurl_core::ast::{Filter, FilterValue, SourceInfo, Template, TemplateElement, Whitespace};
     use hurl_core::reader::Pos;
+    use hurl_core::typing::ToSource;
 
     use crate::runner::filter::eval::eval_filter;
     use crate::runner::{Value, VariableSet};
 
     #[test]
-    pub fn eval_filter_split() {
+    fn eval_filter_split() {
         let variables = VariableSet::new();
         let filter = Filter {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
             value: FilterValue::Split {
-                sep: Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                sep: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: ",".to_string(),
-                        encoded: ",".to_string(),
+                        source: ",".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                },
+                    SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                ),
                 space0: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
