@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@ use hurl_core::ast::SourceInfo;
 
 use crate::runner::{Number, RunnerError, RunnerErrorKind, Value};
 
+/// Converts `value` to float number.
 pub fn eval_to_float(
     value: &Value,
     source_info: SourceInfo,
@@ -30,28 +31,35 @@ pub fn eval_to_float(
         Value::String(v) => match v.parse::<f64>() {
             Ok(f) => Ok(Some(Value::Number(Number::Float(f)))),
             _ => {
-                let kind = RunnerErrorKind::FilterInvalidInput(value.display());
+                let kind = RunnerErrorKind::FilterInvalidInput(value.repr());
                 Err(RunnerError::new(source_info, kind, assert))
             }
         },
+        Value::Number(Number::BigInteger(_)) => {
+            let kind = RunnerErrorKind::FilterInvalidInput(format!(
+                "{} is too big to be cast as a float",
+                value.repr()
+            ));
+            Err(RunnerError::new(source_info, kind, assert))
+        }
         v => {
-            let kind = RunnerErrorKind::FilterInvalidInput(v.display());
+            let kind = RunnerErrorKind::FilterInvalidInput(v.repr());
             Err(RunnerError::new(source_info, kind, assert))
         }
     }
 }
 
 #[cfg(test)]
-pub mod tests {
-
+mod tests {
     use hurl_core::ast::{Filter, FilterValue, SourceInfo};
     use hurl_core::reader::Pos;
 
     use crate::runner::filter::eval::eval_filter;
     use crate::runner::{Number, RunnerErrorKind, Value, VariableSet};
 
+    #[allow(clippy::approx_constant)]
     #[test]
-    pub fn eval_filter_to_float() {
+    fn eval_filter_to_float() {
         let variable = VariableSet::new();
         let filter = Filter {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
@@ -104,7 +112,7 @@ pub mod tests {
     }
 
     #[test]
-    pub fn eval_filter_to_float_error() {
+    fn eval_filter_to_float_error() {
         let variables = VariableSet::new();
         let filter = Filter {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
@@ -127,7 +135,7 @@ pub mod tests {
             .unwrap();
         assert_eq!(
             err.kind,
-            RunnerErrorKind::FilterInvalidInput("bool <true>".to_string())
+            RunnerErrorKind::FilterInvalidInput("boolean <true>".to_string())
         );
     }
 }
