@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,9 +19,8 @@ use std::thread;
 use std::time::Instant;
 
 use chrono::Utc;
-use hurl_core::ast::VersionValue::VersionAnyLegacy;
 use hurl_core::ast::{Entry, OptionKind, SourceInfo};
-use hurl_core::error::DisplaySourceError;
+use hurl_core::error::{DisplaySourceError, OutputFormat};
 use hurl_core::input::Input;
 use hurl_core::parser;
 use hurl_core::typing::Count;
@@ -40,6 +39,8 @@ use crate::util::term::{Stderr, Stdout, WriteMode};
 /// etc...) can be read in the [`HurlResult`] `success` field. If `content` is not syntactically
 /// correct, a parsing error is returned. This is the only possible way for this function to fail.
 ///
+/// The caller must ensure that `content` do not start with [BOM](https://en.wikipedia.org/wiki/Byte_order_mark).
+///
 /// `filename` indicates an optional file source, used when displaying errors.
 ///
 /// # Example
@@ -69,7 +70,7 @@ use crate::util::term::{Stderr, Stdout, WriteMode};
 ///
 /// // Set variables
 /// let mut variables = VariableSet::new();
-/// variables.insert("name".to_string(), Value::String("toto".to_string())).unwrap();
+/// variables.insert("name".to_string(), Value::String("toto".to_string()));
 ///
 /// // Run the Hurl sample
 /// let result = runner::run(
@@ -95,15 +96,23 @@ pub fn run(
 
     // We also create a common logger for this run (logger verbosity can eventually be mutated on
     // each entry).
-    let mut logger = Logger::new(logger_options, stderr);
+    let secrets = variables.secrets();
+    let mut logger = Logger::new(logger_options, stderr, &secrets);
 
     // Try to parse the content
     let hurl_file = parser::parse_hurl_file(content);
     let hurl_file = match hurl_file {
         Ok(h) => h,
-        Err(e) => {
-            logger.error_parsing_rich(content, filename, &e);
-            return Err(e.description());
+        Err(error) => {
+            let filename = filename.map_or(String::new(), |f| f.to_string());
+            let message = error.render(
+                &filename,
+                content,
+                None,
+                OutputFormat::Terminal(logger.color),
+            );
+            logger.error_rich(&message);
+            return Err(error.description());
         }
     };
 
@@ -154,6 +163,22 @@ pub fn run_entries(
     let start = Instant::now();
     let timestamp = Utc::now().timestamp();
 
+    // Warn deprecations
+    if runner_options.pre_entry.is_some() {
+        logger.warning(
+            "--interactive mode is now deprecated, it will be removed in next Hurl versions",
+        );
+    }
+
+    if entries
+        .iter()
+        .any(|e| e.use_multiline_string_body_with_attributes())
+    {
+        logger.warning(
+            "multilines string attributes are now deprecated, they will be removed in next Hurl versions",
+        );
+    }
+
     log_run_info(entries, runner_options, &variables, logger);
 
     // Main loop processing each entry.
@@ -185,8 +210,6 @@ pub fn run_entries(
 
         log_run_entry(entry_index, logger);
 
-        warn_deprecated(entry, filename, logger);
-
         // We can report the progression of the run for --test mode.
         if let Some(listener) = listener {
             listener.on_running(entry_index - 1, n);
@@ -256,7 +279,7 @@ pub fn run_entries(
             logger,
         );
 
-        let has_error = results.last().map_or(false, |r| !r.errors.is_empty());
+        let has_error = results.last().is_some_and(|r| !r.errors.is_empty());
 
         entries_result.extend(results);
 
@@ -294,7 +317,7 @@ pub fn run_entries(
     }
 
     let duration = start.elapsed();
-    let cookies = http_client.cookie_storage();
+    let cookies = http_client.cookie_storage(logger);
     let success = is_success(&entries_result);
     HurlResult {
         entries: entries_result,
@@ -302,6 +325,7 @@ pub fn run_entries(
         success,
         cookies,
         timestamp,
+        variables,
     }
 }
 
@@ -394,7 +418,7 @@ fn run_request(
 fn get_output_source_info(entry: &Entry) -> SourceInfo {
     let mut source_info = entry.source_info();
     for option_entry in entry.request.options() {
-        if let OptionKind::Output(value) = option_entry.kind {
+        if let OptionKind::Output(value) = &option_entry.kind {
             source_info = value.source_info;
         }
     }
@@ -429,21 +453,6 @@ fn is_success(entries: &[EntryResult]) -> bool {
     true
 }
 
-/// Logs deprecated syntax and provides alternatives.
-fn warn_deprecated(entry: &Entry, filename: Option<&Input>, logger: &mut Logger) {
-    let filename = filename.map_or(String::new(), |f| f.to_string());
-    // HTTP/* is used instead of HTTP.
-    if let Some(response) = &entry.response {
-        let version = &response.version;
-        let source_info = &version.source_info;
-        let line = &source_info.start.line;
-        let column = &source_info.start.column;
-        if version.value == VersionAnyLegacy {
-            logger.warning(&format!("{filename}:{line}:{column} 'HTTP/*' keyword is deprecated, please use 'HTTP' instead"));
-        }
-    }
-}
-
 // Returns the list of options that have non-default values.
 fn get_non_default_options(options: &RunnerOptions) -> Vec<(&'static str, String)> {
     let default_options = RunnerOptions::default();
@@ -514,10 +523,14 @@ fn log_run_info(
         }
     }
 
+    let variables = variables
+        .iter()
+        .filter(|(_, variable)| !variable.is_secret())
+        .collect::<Vec<_>>();
     if !variables.is_empty() {
         logger.debug_important("Variables:");
-        for (name, value) in variables.iter() {
-            logger.debug(&format!("    {name}: {value}"));
+        for (name, variable) in variables.iter() {
+            logger.debug(&format!("    {name}: {}", variable.value()));
         }
     }
     if let Some(to_entry) = runner_options.to_entry {
@@ -543,11 +556,21 @@ fn log_errors(
 
     if logger.error_format == ErrorFormat::Long {
         if let Some(Call { response, .. }) = entry_result.calls.last() {
+            logger.info_curl_cmd(&entry_result.curl_cmd.to_string());
+            logger.info("");
+
             response.log_info_all(logger);
         }
     }
     entry_result.errors.iter().for_each(|error| {
-        logger.error_runtime_rich(content, filename, error, entry_result.source_info);
+        let filename = filename.map_or(String::new(), |f| f.to_string());
+        let message = error.render(
+            &filename,
+            content,
+            Some(entry_result.source_info),
+            OutputFormat::Terminal(logger.color),
+        );
+        logger.error_rich(&message);
     });
 }
 
