@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ use hurl_core::ast::{
 use hurl_core::parser::{parse_json_boolean, parse_json_null, parse_json_number};
 use hurl_core::reader::Reader;
 
+use super::template::eval_template;
 use crate::runner::error::{RunnerError, RunnerErrorKind};
 use crate::runner::{expr, VariableSet};
 
@@ -112,11 +113,12 @@ fn eval_json_object_element(
     variables: &VariableSet,
     keep_whitespace: bool,
 ) -> Result<String, RunnerError> {
+    let name = eval_template(&element.name, variables)?;
     let value = eval_json_value(&element.value, variables, keep_whitespace)?;
     if keep_whitespace {
         Ok(format!(
             "{}\"{}\"{}:{}{}{}",
-            element.space0, element.name, element.space1, element.space2, value, element.space3
+            element.space0, name, element.space1, element.space2, value, element.space3
         ))
     } else {
         Ok(format!("\"{}\":{}", element.name, value))
@@ -151,7 +153,7 @@ fn eval_json_template_element(
     variables: &VariableSet,
 ) -> Result<String, RunnerError> {
     match template_element {
-        TemplateElement::String { encoded, .. } => Ok(encoded.clone()),
+        TemplateElement::String { source, .. } => Ok(source.to_string()),
         TemplateElement::Placeholder(Placeholder { expr, .. }) => {
             let s = expr::render(expr, variables)?;
             Ok(encode_json_string(&s))
@@ -176,21 +178,22 @@ fn encode_json_char(c: char) -> String {
 
 #[cfg(test)]
 mod tests {
-    use crate::runner::Value;
     use hurl_core::ast::*;
     use hurl_core::reader::Pos;
+    use hurl_core::typing::ToSource;
 
     use super::super::error::RunnerErrorKind;
     use super::*;
+    use crate::runner::Value;
 
     pub fn json_hello_world_value() -> JsonValue {
         // "hello\u0020{{name}}!"
-        JsonValue::String(Template {
-            delimiter: Some('"'),
-            elements: vec![
+        JsonValue::String(Template::new(
+            Some('"'),
+            vec![
                 TemplateElement::String {
                     value: "Hello ".to_string(),
-                    encoded: "Hello\\u0020".to_string(),
+                    source: "Hello\\u0020".to_source(),
                 },
                 TemplateElement::Placeholder(Placeholder {
                     space0: Whitespace {
@@ -211,11 +214,11 @@ mod tests {
                 }),
                 TemplateElement::String {
                     value: "!".to_string(),
-                    encoded: "!".to_string(),
+                    source: "!".to_source(),
                 },
             ],
-            source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 22)),
-        })
+            SourceInfo::new(Pos::new(1, 2), Pos::new(1, 22)),
+        ))
     }
 
     pub fn json_person_value() -> JsonValue {
@@ -223,24 +226,24 @@ mod tests {
             space0: "\n    ".to_string(),
             elements: vec![JsonObjectElement {
                 space0: String::new(),
-                name: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
+                name: Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "firstName".to_string(),
-                        encoded: "firstName".to_string(),
+                        source: "firstName".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
-                },
+                    SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
+                ),
                 space1: String::new(),
                 space2: " ".to_string(),
-                value: JsonValue::String(Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
+                value: JsonValue::String(Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "John".to_string(),
-                        encoded: "John".to_string(),
+                        source: "John".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
-                }),
+                    SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
+                )),
                 space3: "\n".to_string(),
             }],
         }
@@ -249,9 +252,7 @@ mod tests {
     #[test]
     fn test_scalar_value() {
         let mut variables = VariableSet::new();
-        variables
-            .insert("name".to_string(), Value::String("Bob".to_string()))
-            .unwrap();
+        variables.insert("name".to_string(), Value::String("Bob".to_string()));
         assert_eq!(
             eval_json_value(&JsonValue::Null, &variables, true).unwrap(),
             "null".to_string()
@@ -291,9 +292,7 @@ mod tests {
     #[test]
     fn test_list_value() {
         let mut variables = VariableSet::new();
-        variables
-            .insert("name".to_string(), Value::String("Bob".to_string()))
-            .unwrap();
+        variables.insert("name".to_string(), Value::String("Bob".to_string()));
         assert_eq!(
             eval_json_value(
                 &JsonValue::List {
@@ -336,14 +335,14 @@ mod tests {
             "[1, -2, 3.0]".to_string()
         );
 
-        let template = Template {
-            delimiter: Some('"'),
-            elements: vec![TemplateElement::String {
-                encoded: "Hi".to_string(),
+        let template = Template::new(
+            Some('"'),
+            vec![TemplateElement::String {
                 value: "Hi".to_string(),
+                source: "Hi".to_source(),
             }],
-            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-        };
+            SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+        );
         assert_eq!(
             eval_json_value(
                 &JsonValue::List {
@@ -398,14 +397,14 @@ mod tests {
         let variables = VariableSet::new();
         assert_eq!(
             eval_json_value(
-                &JsonValue::String(Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
+                &JsonValue::String(Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "\n".to_string(),
-                        encoded: "\\n".to_string(),
+                        source: "\\n".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
-                }),
+                    SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1))
+                )),
                 &variables,
                 true
             )
@@ -419,14 +418,14 @@ mod tests {
         let variables = VariableSet::new();
         assert_eq!(
             eval_json_template(
-                &Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
+                &Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "\n".to_string(),
-                        encoded: "\\n".to_string(),
+                        source: "\\n".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
-                },
+                    SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
+                ),
                 &variables,
             )
             .unwrap(),
@@ -434,17 +433,15 @@ mod tests {
         );
 
         let mut variables = VariableSet::new();
-        variables
-            .insert("quote".to_string(), Value::String("\"".to_string()))
-            .unwrap();
+        variables.insert("quote".to_string(), Value::String("\"".to_string()));
         assert_eq!(
             eval_json_template(
-                &Template {
-                    delimiter: Some('"'),
-                    elements: vec![
+                &Template::new(
+                    Some('"'),
+                    vec![
                         TemplateElement::String {
                             value: "Hello ".to_string(),
-                            encoded: "Hello ".to_string(),
+                            source: "Hello ".to_source(),
                         },
                         TemplateElement::Placeholder(Placeholder {
                             space0: whitespace(),
@@ -458,8 +455,8 @@ mod tests {
                             space1: whitespace(),
                         }),
                     ],
-                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                },
+                    SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                ),
                 &variables,
             )
             .unwrap(),
