@@ -1,6 +1,7 @@
+use chrono::Utc;
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,44 +28,48 @@ use crate::runner::cache::BodyCache;
 use crate::runner::error::{RunnerError, RunnerErrorKind};
 use crate::runner::template::eval_template;
 use crate::runner::xpath::{Document, Format};
-use crate::runner::{filter, Number, Value, VariableSet};
+use crate::runner::{filter, HttpResponse, Number, Value, VariableSet};
 
 pub type QueryResult = Result<Option<Value>, RunnerError>;
 
-/// Evaluates this `query` and returns a [`QueryResult`], using the HTTP `response` and `variables`.
+/// Evaluates this `query` and returns a [`QueryResult`], using a list of HTTP `responses` and `variables`.
 pub fn eval_query(
     query: &Query,
     variables: &VariableSet,
-    response: &http::Response,
+    responses: &[&http::Response],
     cache: &mut BodyCache,
 ) -> QueryResult {
+    let last_response = responses.last().unwrap();
     match &query.value {
-        QueryValue::Status => eval_query_status(response),
-        QueryValue::Url => eval_query_url(response),
-        QueryValue::Header { name, .. } => eval_query_header(response, name, variables),
+        QueryValue::Status => eval_query_status(last_response),
+        QueryValue::Version => eval_query_version(last_response),
+        QueryValue::Url => eval_query_url(last_response),
+        QueryValue::Header { name, .. } => eval_query_header(last_response, name, variables),
         QueryValue::Cookie {
             expr: CookiePath { name, attribute },
             ..
-        } => eval_query_cookie(response, name, attribute, variables),
-        QueryValue::Body => eval_query_body(response, query.source_info),
+        } => eval_query_cookie(last_response, name, attribute, variables),
+        QueryValue::Body => eval_query_body(last_response, query.source_info),
         QueryValue::Xpath { expr, .. } => {
-            eval_query_xpath(response, cache, expr, variables, query.source_info)
+            eval_query_xpath(last_response, cache, expr, variables, query.source_info)
         }
         QueryValue::Jsonpath { expr, .. } => {
-            eval_query_jsonpath(response, cache, expr, variables, query.source_info)
+            eval_query_jsonpath(last_response, cache, expr, variables, query.source_info)
         }
         QueryValue::Regex { value, .. } => {
-            eval_query_regex(response, value, variables, query.source_info)
+            eval_query_regex(last_response, value, variables, query.source_info)
         }
         QueryValue::Variable { name, .. } => eval_query_variable(name, variables),
-        QueryValue::Duration => eval_query_duration(response),
-        QueryValue::Bytes => eval_query_bytes(response, query.source_info),
-        QueryValue::Sha256 => eval_query_sha256(response, query.source_info),
-        QueryValue::Md5 => eval_query_md5(response, query.source_info),
+        QueryValue::Duration => eval_query_duration(last_response),
+        QueryValue::Bytes => eval_query_bytes(last_response, query.source_info),
+        QueryValue::Sha256 => eval_query_sha256(last_response, query.source_info),
+        QueryValue::Md5 => eval_query_md5(last_response, query.source_info),
         QueryValue::Certificate {
             attribute_name: field,
             ..
-        } => eval_query_certificate(response, *field),
+        } => eval_query_certificate(last_response, *field),
+        QueryValue::Ip => eval_ip(last_response),
+        QueryValue::Redirects => eval_redirects(responses),
     }
 }
 
@@ -75,6 +80,18 @@ fn eval_query_status(response: &http::Response) -> QueryResult {
     )))))
 }
 
+/// Evaluates the version on the HTTP `response`
+fn eval_query_version(response: &http::Response) -> QueryResult {
+    Ok(Some(Value::String(
+        response
+            .version
+            .to_string()
+            .strip_prefix("HTTP/")
+            .unwrap()
+            .to_string(),
+    )))
+}
+
 /// Evaluates the final URL of the HTTP `response`.
 fn eval_query_url(response: &http::Response) -> QueryResult {
     Ok(Some(Value::String(response.url.to_string())))
@@ -109,6 +126,7 @@ fn eval_query_cookie(
     attribute: &Option<CookieAttribute>,
     variables: &VariableSet,
 ) -> QueryResult {
+    let query_source_info = name.source_info;
     let name = eval_template(name, variables)?;
     match response.get_cookie(&name) {
         None => Ok(None),
@@ -118,7 +136,7 @@ fn eval_query_cookie(
             } else {
                 CookieAttributeName::Value("Value".to_string())
             };
-            Ok(eval_cookie_attribute_name(attribute_name, cookie))
+            eval_cookie_attribute_name(attribute_name, cookie, query_source_info)
         }
     }
 }
@@ -290,8 +308,8 @@ fn eval_query_regex(
 /// Evaluates a variable, given a set of `variables`.
 fn eval_query_variable(name: &Template, variables: &VariableSet) -> QueryResult {
     let name = eval_template(name, variables)?;
-    if let Some(value) = variables.get(name.as_str()) {
-        Ok(Some(value.clone()))
+    if let Some(variable) = variables.get(&name) {
+        Ok(Some(variable.value().clone()))
     } else {
         Ok(None)
     }
@@ -379,42 +397,79 @@ fn eval_query_certificate(
     }
 }
 
+/// Evaluates the ip address of the HTTP `response`.
+fn eval_ip(response: &http::Response) -> QueryResult {
+    Ok(Some(Value::String(response.ip_addr.to_string())))
+}
+
+/// Evaluates the redirects within a list of HTTP `responses`
+fn eval_redirects(responses: &[&http::Response]) -> QueryResult {
+    let mut it = responses.iter().peekable();
+    let mut values: Vec<Value> = vec![];
+    while let Some(r) = it.next() {
+        // We peek the next response in the response chain to use the next URL response
+        // as the current response location. The URLs' response chain has already been resolved
+        // we don't need to reconstruct the location from response headers.
+        let location = it.peek().map(|r| r.url.clone());
+        // We're only interested to redirection:
+        if location.is_some() {
+            let response = Value::HttpResponse(HttpResponse::new(location, r.status));
+            values.push(response);
+        }
+    }
+    Ok(Some(Value::List(values)))
+}
+
 fn eval_cookie_attribute_name(
     cookie_attribute_name: CookieAttributeName,
     cookie: http::ResponseCookie,
-) -> Option<Value> {
+    query_source_info: SourceInfo,
+) -> QueryResult {
     match cookie_attribute_name {
-        CookieAttributeName::Value(_) => Some(Value::String(cookie.value)),
+        CookieAttributeName::Value(_) => Ok(Some(Value::String(cookie.value))),
         CookieAttributeName::Expires(_) => {
             if let Some(s) = cookie.expires() {
-                match chrono::DateTime::parse_from_rfc2822(s.as_str()) {
-                    Ok(v) => Some(Value::Date(v.with_timezone(&chrono::Utc))),
-                    Err(_) => todo!(),
+                if let Ok(v) = chrono::DateTime::parse_from_rfc2822(s.as_str()) {
+                    Ok(Some(Value::Date(v.with_timezone(&chrono::Utc))))
+
+                // support format with dash such as Wed, 13-Jan-2021 22:23:01 GMT
+                // TODO: search for for other possible date format used in the wild
+                } else if let Ok(v) = chrono::NaiveDateTime::parse_from_str(
+                    s.as_str(),
+                    "%a, %d-%b-%Y %H:%M:%S%.3f GMT",
+                ) {
+                    Ok(Some(Value::Date(v.and_local_timezone(Utc).unwrap())))
+                } else {
+                    Err(RunnerError::new(
+                        query_source_info,
+                        RunnerErrorKind::Http(http::HttpError::CouldNotParseCookieExpires(s)),
+                        true,
+                    ))
                 }
             } else {
-                None
+                Ok(None)
             }
         }
         CookieAttributeName::MaxAge(_) => {
-            cookie.max_age().map(|v| Value::Number(Number::Integer(v)))
+            Ok(cookie.max_age().map(|v| Value::Number(Number::Integer(v))))
         }
-        CookieAttributeName::Domain(_) => cookie.domain().map(Value::String),
-        CookieAttributeName::Path(_) => cookie.path().map(Value::String),
+        CookieAttributeName::Domain(_) => Ok(cookie.domain().map(Value::String)),
+        CookieAttributeName::Path(_) => Ok(cookie.path().map(Value::String)),
         CookieAttributeName::Secure(_) => {
             if cookie.has_secure() {
-                Some(Value::Unit)
+                Ok(Some(Value::Unit))
             } else {
-                None
+                Ok(None)
             }
         }
         CookieAttributeName::HttpOnly(_) => {
             if cookie.has_httponly() {
-                Some(Value::Unit)
+                Ok(Some(Value::Unit))
             } else {
-                None
+                Ok(None)
             }
         }
-        CookieAttributeName::SameSite(_) => cookie.samesite().map(Value::String),
+        CookieAttributeName::SameSite(_) => Ok(cookie.samesite().map(Value::String)),
     }
 }
 
@@ -450,9 +505,11 @@ impl Value {
 
 #[cfg(test)]
 pub mod tests {
-    use hex_literal::hex;
+    use std::num::ParseIntError;
+
     use hurl_core::ast::{SourceInfo, TemplateElement, Whitespace};
     use hurl_core::reader::Pos;
+    use hurl_core::typing::ToSource;
 
     use super::*;
     use crate::http::{HeaderVec, HttpError, HttpVersion};
@@ -466,6 +523,7 @@ pub mod tests {
             duration: Default::default(),
             url: "http://localhost".parse().unwrap(),
             certificate: None,
+            ip_addr: Default::default(),
         }
     }
 
@@ -479,14 +537,14 @@ pub mod tests {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 13)),
             value: QueryValue::Xpath {
                 space0: whitespace,
-                expr: Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                expr: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "???".to_string(),
-                        encoded: "???".to_string(),
+                        source: "???".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 10)),
-                },
+                    SourceInfo::new(Pos::new(1, 7), Pos::new(1, 10)),
+                ),
             },
         }
     }
@@ -500,14 +558,14 @@ pub mod tests {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 13)),
             value: QueryValue::Xpath {
                 space0: whitespace,
-                expr: Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                expr: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "count(//user)".to_string(),
-                        encoded: "count(//user)".to_string(),
+                        source: "count(//user)".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                },
+                    SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                ),
             },
         }
     }
@@ -521,14 +579,14 @@ pub mod tests {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 13)),
             value: QueryValue::Xpath {
                 space0: whitespace,
-                expr: Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                expr: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "//user".to_string(),
-                        encoded: "/user".to_string(),
+                        source: "/user".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                },
+                    SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                ),
             },
         }
     }
@@ -542,14 +600,14 @@ pub mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
-                expr: Template {
-                    elements: vec![TemplateElement::String {
-                        value: String::from("$.success"),
-                        encoded: String::from("$.success"),
+                expr: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
+                        value: "$.success".to_string(),
+                        source: "$.success".to_source(),
                     }],
-                    delimiter: Some('"'),
-                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 19)),
-                },
+                    SourceInfo::new(Pos::new(1, 10), Pos::new(1, 19)),
+                ),
             },
         }
     }
@@ -563,14 +621,14 @@ pub mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
-                expr: Template {
-                    elements: vec![TemplateElement::String {
-                        value: String::from("$.errors"),
-                        encoded: String::from("$.errors"),
+                expr: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
+                        value: "$.errors".to_string(),
+                        source: "$.errors".to_source(),
                     }],
-                    delimiter: Some('"'),
-                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 18)),
-                },
+                    SourceInfo::new(Pos::new(1, 10), Pos::new(1, 18)),
+                ),
             },
         }
     }
@@ -584,14 +642,14 @@ pub mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
-                expr: Template {
-                    elements: vec![TemplateElement::String {
-                        value: String::from("$.duration"),
-                        encoded: String::from("$.duration"),
+                expr: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
+                        value: "$.duration".to_string(),
+                        source: "$.duration".to_source(),
                     }],
-                    delimiter: Some('"'),
-                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 18)),
-                },
+                    SourceInfo::new(Pos::new(1, 10), Pos::new(1, 18)),
+                ),
             },
         }
     }
@@ -605,14 +663,14 @@ pub mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)),
                 },
-                value: RegexValue::Template(Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                value: RegexValue::Template(Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "Hello ([a-zA-Z]+)!".to_string(),
-                        encoded: "Hello ([a-zA-Z]+)!".to_string(),
+                        source: "Hello ([a-zA-Z]+)!".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 26)),
-                }),
+                    SourceInfo::new(Pos::new(1, 7), Pos::new(1, 26)),
+                )),
             },
         }
     }
@@ -626,14 +684,14 @@ pub mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)),
                 },
-                value: RegexValue::Template(Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                value: RegexValue::Template(Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "???".to_string(),
-                        encoded: "???".to_string(),
+                        source: "???".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 10)),
-                }),
+                    SourceInfo::new(Pos::new(1, 7), Pos::new(1, 10)),
+                )),
             },
         }
     }
@@ -678,7 +736,7 @@ pub mod tests {
                     value: QueryValue::Status,
                 },
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -700,14 +758,14 @@ pub mod tests {
                     value: String::from(" "),
                     source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 8)),
                 },
-                name: Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                name: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "Custom".to_string(),
-                        encoded: "Custom".to_string(),
+                        source: "Custom".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(2, 8), Pos::new(2, 14)),
-                },
+                    SourceInfo::new(Pos::new(2, 8), Pos::new(2, 14)),
+                ),
             },
         };
         //    let error = query_header.eval(http::hello_http_response()).err().unwrap();
@@ -717,7 +775,7 @@ pub mod tests {
             eval_query(
                 &query_header,
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache
             )
             .unwrap(),
@@ -738,21 +796,21 @@ pub mod tests {
                     value: String::from(" "),
                     source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 8)),
                 },
-                name: Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                name: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "Content-Type".to_string(),
-                        encoded: "Content-Type".to_string(),
+                        source: "Content-Type".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 16)),
-                },
+                    SourceInfo::new(Pos::new(1, 8), Pos::new(1, 16)),
+                ),
             },
         };
         assert_eq!(
             eval_query(
                 &query_header,
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache
             )
             .unwrap()
@@ -778,26 +836,28 @@ pub mod tests {
             ..default_response()
         };
 
+        let responses = vec![&response];
+
         // cookie "LSID"
         let query = Query {
             source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
-                    name: Template {
-                        delimiter: Some('"'),
-                        elements: vec![TemplateElement::String {
+                    name: Template::new(
+                        Some('"'),
+                        vec![TemplateElement::String {
                             value: "LSID".to_string(),
-                            encoded: "LSID".to_string(),
+                            source: "LSID".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                    },
+                        SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                    ),
                     attribute: None,
                 },
             },
         };
         assert_eq!(
-            eval_query(&query, &variables, &response, &mut cache)
+            eval_query(&query, &variables, &responses, &mut cache)
                 .unwrap()
                 .unwrap(),
             Value::String("DQAAAKEaem_vYg".to_string())
@@ -809,14 +869,14 @@ pub mod tests {
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
-                    name: Template {
-                        delimiter: Some('"'),
-                        elements: vec![TemplateElement::String {
+                    name: Template::new(
+                        Some('"'),
+                        vec![TemplateElement::String {
                             value: "LSID".to_string(),
-                            encoded: "LSID".to_string(),
+                            source: "LSID".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                    },
+                        SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                    ),
                     attribute: Some(CookieAttribute {
                         space0: space.clone(),
                         name: CookieAttributeName::Path("Path".to_string()),
@@ -826,7 +886,7 @@ pub mod tests {
             },
         };
         assert_eq!(
-            eval_query(&query, &variables, &response, &mut cache)
+            eval_query(&query, &variables, &responses, &mut cache)
                 .unwrap()
                 .unwrap(),
             Value::String("/accounts".to_string())
@@ -838,14 +898,14 @@ pub mod tests {
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
-                    name: Template {
-                        delimiter: Some('"'),
-                        elements: vec![TemplateElement::String {
+                    name: Template::new(
+                        Some('"'),
+                        vec![TemplateElement::String {
                             value: "LSID".to_string(),
-                            encoded: "LSID".to_string(),
+                            source: "LSID".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                    },
+                        SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                    ),
                     attribute: Some(CookieAttribute {
                         space0: space.clone(),
                         name: CookieAttributeName::Secure("Secure".to_string()),
@@ -855,7 +915,7 @@ pub mod tests {
             },
         };
         assert_eq!(
-            eval_query(&query, &variables, &response, &mut cache)
+            eval_query(&query, &variables, &responses, &mut cache)
                 .unwrap()
                 .unwrap(),
             Value::Unit
@@ -867,14 +927,14 @@ pub mod tests {
             value: QueryValue::Cookie {
                 space0: space.clone(),
                 expr: CookiePath {
-                    name: Template {
-                        delimiter: Some('"'),
-                        elements: vec![TemplateElement::String {
+                    name: Template::new(
+                        Some('"'),
+                        vec![TemplateElement::String {
                             value: "LSID".to_string(),
-                            encoded: "LSID".to_string(),
+                            source: "LSID".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                    },
+                        SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                    ),
                     attribute: Some(CookieAttribute {
                         space0: space.clone(),
                         name: CookieAttributeName::Domain("Domain".to_string()),
@@ -884,7 +944,7 @@ pub mod tests {
             },
         };
         assert_eq!(
-            eval_query(&query, &variables, &response, &mut cache).unwrap(),
+            eval_query(&query, &variables, &responses, &mut cache).unwrap(),
             None
         );
     }
@@ -913,35 +973,53 @@ pub mod tests {
                 },
             ],
         };
+        let query_source_info = SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0));
+
         assert_eq!(
-            eval_cookie_attribute_name(CookieAttributeName::Value("_".to_string()), cookie.clone())
-                .unwrap(),
+            eval_cookie_attribute_name(
+                CookieAttributeName::Value("_".to_string()),
+                cookie.clone(),
+                query_source_info
+            )
+            .unwrap()
+            .unwrap(),
             Value::String("DQAAAKEaem_vYg".to_string())
         );
         assert_eq!(
             eval_cookie_attribute_name(
                 CookieAttributeName::Domain("_".to_string()),
                 cookie.clone(),
-            ),
+                query_source_info
+            )
+            .unwrap(),
             None
         );
         assert_eq!(
-            eval_cookie_attribute_name(CookieAttributeName::Path("_".to_string()), cookie.clone())
-                .unwrap(),
+            eval_cookie_attribute_name(
+                CookieAttributeName::Path("_".to_string()),
+                cookie.clone(),
+                query_source_info
+            )
+            .unwrap()
+            .unwrap(),
             Value::String("/accounts".to_string())
         );
         assert_eq!(
             eval_cookie_attribute_name(
                 CookieAttributeName::MaxAge("_".to_string()),
                 cookie.clone(),
-            ),
+                query_source_info
+            )
+            .unwrap(),
             None
         );
         assert_eq!(
             eval_cookie_attribute_name(
                 CookieAttributeName::Expires("_".to_string()),
                 cookie.clone(),
+                query_source_info
             )
+            .unwrap()
             .unwrap(),
             Value::Date(
                 chrono::DateTime::parse_from_rfc2822("Wed, 13 Jan 2021 22:23:01 GMT")
@@ -953,7 +1031,9 @@ pub mod tests {
             eval_cookie_attribute_name(
                 CookieAttributeName::Secure("_".to_string()),
                 cookie.clone(),
+                query_source_info
             )
+            .unwrap()
             .unwrap(),
             Value::Unit
         );
@@ -961,13 +1041,47 @@ pub mod tests {
             eval_cookie_attribute_name(
                 CookieAttributeName::HttpOnly("_".to_string()),
                 cookie.clone(),
+                query_source_info
             )
+            .unwrap()
             .unwrap(),
             Value::Unit
         );
         assert_eq!(
-            eval_cookie_attribute_name(CookieAttributeName::SameSite("_".to_string()), cookie),
-            None
+            eval_cookie_attribute_name(
+                CookieAttributeName::SameSite("_".to_string()),
+                cookie,
+                query_source_info
+            )
+            .unwrap(),
+            None,
+        );
+    }
+
+    #[test]
+    fn test_eval_cookie_attribute_expires_error() {
+        let cookie = http::ResponseCookie {
+            name: "cookie1".to_string(),
+            value: "value1".to_string(),
+            attributes: vec![http::CookieAttribute {
+                name: "Expires".to_string(),
+                value: Some("???".to_string()),
+            }],
+        };
+        let query_source_info = SourceInfo::new(Pos::new(5, 1), Pos::new(5, 20));
+        let error = eval_cookie_attribute_name(
+            CookieAttributeName::Expires("_".to_string()),
+            cookie.clone(),
+            query_source_info,
+        )
+        .unwrap_err();
+        assert_eq!(
+            error.source_info,
+            SourceInfo::new(Pos::new(5, 1), Pos::new(5, 20))
+        );
+        assert_eq!(
+            error.kind,
+            RunnerErrorKind::Http(HttpError::CouldNotParseCookieExpires("???".to_string()))
         );
     }
 
@@ -983,7 +1097,7 @@ pub mod tests {
                     value: QueryValue::Body,
                 },
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -996,7 +1110,7 @@ pub mod tests {
                 value: QueryValue::Body,
             },
             &variables,
-            &http::bytes_http_response(),
+            &[&http::bytes_http_response()],
             &mut cache,
         )
         .err()
@@ -1022,7 +1136,7 @@ pub mod tests {
             body: vec![200],
             ..default_response()
         };
-        let error = eval_query(&xpath_users(), &variables, &http_response, &mut cache)
+        let error = eval_query(&xpath_users(), &variables, &[&http_response], &mut cache)
             .err()
             .unwrap();
         assert_eq!(error.source_info.start, Pos { line: 1, column: 1 });
@@ -1047,24 +1161,24 @@ pub mod tests {
                     value: String::from(" "),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)),
                 },
-                expr: Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                expr: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "^^^".to_string(),
-                        encoded: "^^^".to_string(),
+                        source: "^^^".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 10)),
-                },
+                    SourceInfo::new(Pos::new(1, 7), Pos::new(1, 10)),
+                ),
             },
         };
         let error = eval_query(
             &query,
             &variables,
-            &http::xml_two_users_http_response(),
+            &[&http::xml_two_users_http_response()],
             &mut cache,
         )
         .unwrap_err();
-        assert_eq!(error.kind, RunnerErrorKind::QueryInvalidXpathEval);
+        assert_eq!(error.kind, RunnerErrorKind::InvalidXPathEval);
         assert_eq!(error.source_info.start, Pos { line: 1, column: 7 });
     }
 
@@ -1077,7 +1191,7 @@ pub mod tests {
             eval_query(
                 &xpath_users(),
                 &variables,
-                &http::xml_two_users_http_response(),
+                &[&http::xml_two_users_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -1088,7 +1202,7 @@ pub mod tests {
             eval_query(
                 &xpath_count_user_query(),
                 &variables,
-                &http::xml_two_users_http_response(),
+                &[&http::xml_two_users_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -1108,14 +1222,14 @@ pub mod tests {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 13)),
             value: QueryValue::Xpath {
                 space0: whitespace,
-                expr: Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                expr: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "normalize-space(/html/head/meta/@charset)".to_string(),
-                        encoded: "normalize-space(/html/head/meta/@charset)".to_string(),
+                        source: "normalize-space(/html/head/meta/@charset)".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                },
+                    SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                ),
             },
         }
     }
@@ -1129,7 +1243,7 @@ pub mod tests {
             eval_query(
                 &xpath_html_charset(),
                 &variables,
-                &http::html_http_response(),
+                &[&http::html_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -1151,21 +1265,21 @@ pub mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
-                expr: Template {
-                    elements: vec![TemplateElement::String {
-                        value: String::from("xxx"),
-                        encoded: String::from("xxx"),
+                expr: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
+                        value: "xxx".to_string(),
+                        source: "xxx".to_source(),
                     }],
-                    delimiter: Some('"'),
-                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 13)),
-                },
+                    SourceInfo::new(Pos::new(1, 10), Pos::new(1, 13)),
+                ),
             },
         };
 
         let error = eval_query(
             &jsonpath_query,
             &variables,
-            &http::json_http_response(),
+            &[&http::json_http_response()],
             &mut cache,
         )
         .unwrap_err();
@@ -1192,9 +1306,14 @@ pub mod tests {
             body: String::into_bytes(String::from("xxx")),
             ..default_response()
         };
-        let error = eval_query(&jsonpath_success(), &variables, &http_response, &mut cache)
-            .err()
-            .unwrap();
+        let error = eval_query(
+            &jsonpath_success(),
+            &variables,
+            &[&http_response],
+            &mut cache,
+        )
+        .err()
+        .unwrap();
         assert_eq!(error.source_info.start, Pos { line: 1, column: 1 });
         assert_eq!(error.kind, RunnerErrorKind::QueryInvalidJson);
     }
@@ -1209,7 +1328,13 @@ pub mod tests {
             ..default_response()
         };
         assert_eq!(
-            eval_query(&jsonpath_success(), &variables, &http_response, &mut cache).unwrap(),
+            eval_query(
+                &jsonpath_success(),
+                &variables,
+                &[&http_response],
+                &mut cache
+            )
+            .unwrap(),
             None
         );
     }
@@ -1223,7 +1348,7 @@ pub mod tests {
             eval_query(
                 &jsonpath_success(),
                 &variables,
-                &http::json_http_response(),
+                &[&http::json_http_response()],
                 &mut cache
             )
             .unwrap()
@@ -1234,7 +1359,7 @@ pub mod tests {
             eval_query(
                 &jsonpath_errors(),
                 &variables,
-                &http::json_http_response(),
+                &[&http::json_http_response()],
                 &mut cache
             )
             .unwrap()
@@ -1261,7 +1386,7 @@ pub mod tests {
             eval_query(
                 &regex_name(),
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache
             )
             .unwrap()
@@ -1272,7 +1397,7 @@ pub mod tests {
         let error = eval_query(
             &regex_invalid(),
             &variables,
-            &http::hello_http_response(),
+            &[&http::hello_http_response()],
             &mut cache,
         )
         .err()
@@ -1296,7 +1421,7 @@ pub mod tests {
                     value: QueryValue::Bytes,
                 },
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -1305,6 +1430,13 @@ pub mod tests {
         );
     }
 
+    fn decode_hex(s: &str) -> Result<Vec<u8>, ParseIntError> {
+        (0..s.len())
+            .step_by(2)
+            .map(|i| u8::from_str_radix(&s[i..i + 2], 16))
+            .collect()
+    }
+
     #[test]
     fn test_query_sha256() {
         let variables = VariableSet::new();
@@ -1317,16 +1449,17 @@ pub mod tests {
                     value: QueryValue::Sha256 {},
                 },
                 &variables,
-                &http::Response {
+                &[&http::Response {
                     body: vec![0xff],
                     ..default_response()
-                },
+                }],
                 &mut cache,
             )
             .unwrap()
             .unwrap(),
             Value::Bytes(
-                hex!("a8100ae6aa1940d0b663bb31cd466142ebbdbd5187131b92d93818987832eb89").to_vec()
+                decode_hex("a8100ae6aa1940d0b663bb31cd466142ebbdbd5187131b92d93818987832eb89")
+                    .unwrap()
             )
         );
     }
