@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,7 +33,7 @@ pub fn eval_version_status_asserts(
     let mut asserts = vec![];
 
     let version = &response.version;
-    asserts.push(AssertResult::Version {
+    asserts.push(AssertResult::ImplicitVersion {
         actual: http_response.version.to_string(),
         expected: version.value.to_string(),
         source_info: version.source_info,
@@ -41,7 +41,7 @@ pub fn eval_version_status_asserts(
 
     let status = &response.status;
     if let StatusValue::Specific(v) = status.value {
-        asserts.push(AssertResult::Status {
+        asserts.push(AssertResult::ImplicitStatus {
             actual: http_response.status as u64,
             expected: v,
             source_info: status.source_info,
@@ -50,7 +50,7 @@ pub fn eval_version_status_asserts(
     asserts
 }
 
-/// Returns a list of assert results, given a set of `variables`, an actual `http_response` and a spec `response`.
+/// Returns a list of assert results, given a set of `variables`, a list of actual `http_responses` and a spec `response`.
 ///
 /// Asserts on status and version and not run in this function, there are run with `eval_version_status_asserts`
 /// as they're semantically stronger.
@@ -60,17 +60,18 @@ pub fn eval_version_status_asserts(
 pub fn eval_asserts(
     response: &Response,
     variables: &VariableSet,
-    http_response: &http::Response,
+    http_responses: &[&http::Response],
     cache: &mut BodyCache,
     context_dir: &ContextDir,
 ) -> Vec<AssertResult> {
     let mut asserts = vec![];
+    let last_response = http_responses.last().unwrap();
 
     // First, evaluates implicit asserts on response headers.
     for header in &response.headers {
         match template::eval_template(&header.value, variables) {
             Err(e) => {
-                let result = AssertResult::Header {
+                let result = AssertResult::ImplicitHeader {
                     actual: Err(e),
                     expected: String::new(),
                     source_info: header.key.source_info,
@@ -80,9 +81,9 @@ pub fn eval_asserts(
             Ok(expected) => {
                 match template::eval_template(&header.key, variables) {
                     Ok(header_name) => {
-                        let actuals = http_response.headers.values(&header_name);
+                        let actuals = last_response.headers.values(&header_name);
                         if actuals.is_empty() {
-                            let result = AssertResult::Header {
+                            let result = AssertResult::ImplicitHeader {
                                 actual: Err(RunnerError::new(
                                     header.key.source_info,
                                     RunnerErrorKind::QueryHeaderNotFound,
@@ -94,7 +95,7 @@ pub fn eval_asserts(
                             asserts.push(result);
                         } else if actuals.len() == 1 {
                             let actual = actuals.first().unwrap().to_string();
-                            let result = AssertResult::Header {
+                            let result = AssertResult::ImplicitHeader {
                                 actual: Ok(actual),
                                 expected,
                                 source_info: header.value.source_info,
@@ -118,7 +119,7 @@ pub fn eval_asserts(
                                     break;
                                 }
                             }
-                            let result = AssertResult::Header {
+                            let result = AssertResult::ImplicitHeader {
                                 actual: Ok(actual),
                                 expected,
                                 source_info: header.value.source_info,
@@ -127,7 +128,7 @@ pub fn eval_asserts(
                         }
                     }
                     Err(e) => {
-                        let result = AssertResult::Header {
+                        let result = AssertResult::ImplicitHeader {
                             actual: Err(e),
                             expected,
                             source_info: header.value.source_info,
@@ -141,14 +142,14 @@ pub fn eval_asserts(
 
     // Second, evaluates implicit asserts on response body.
     if let Some(body) = &response.body {
-        let assert = eval_implicit_body_asserts(body, variables, http_response, context_dir);
+        let assert = eval_implicit_body_asserts(body, variables, last_response, context_dir);
         asserts.push(assert);
     }
 
     // Then, checks all the explicit asserts.
     for assert in response.asserts() {
         let assert_result =
-            assert::eval_explicit_assert(assert, variables, http_response, cache, context_dir);
+            assert::eval_explicit_assert(assert, variables, http_responses, cache, context_dir);
         asserts.push(assert_result);
     }
     asserts
@@ -181,7 +182,7 @@ fn eval_implicit_body_asserts(
                     ))
                 }
             };
-            AssertResult::Body {
+            AssertResult::ImplicitBody {
                 actual,
                 expected,
                 source_info: spec_body.space0.source_info,
@@ -203,7 +204,7 @@ fn eval_implicit_body_asserts(
                     ))
                 }
             };
-            AssertResult::Body {
+            AssertResult::ImplicitBody {
                 actual,
                 expected,
                 source_info: spec_body.space0.source_info,
@@ -228,7 +229,7 @@ fn eval_implicit_body_asserts(
                     ))
                 }
             };
-            AssertResult::Body {
+            AssertResult::ImplicitBody {
                 actual,
                 expected,
                 source_info: value.source_info,
@@ -253,7 +254,7 @@ fn eval_implicit_body_asserts(
                     ))
                 }
             };
-            AssertResult::Body {
+            AssertResult::ImplicitBody {
                 actual,
                 expected,
                 source_info: multi.value().source_info,
@@ -280,7 +281,7 @@ fn eval_implicit_body_asserts(
                     ))
                 }
             };
-            AssertResult::Body {
+            AssertResult::ImplicitBody {
                 actual,
                 expected,
                 source_info: SourceInfo {
@@ -310,7 +311,7 @@ fn eval_implicit_body_asserts(
                     ))
                 }
             };
-            AssertResult::Body {
+            AssertResult::ImplicitBody {
                 actual,
                 expected,
                 source_info: SourceInfo {
@@ -338,7 +339,7 @@ fn eval_implicit_body_asserts(
                     ))
                 }
             };
-            AssertResult::Body {
+            AssertResult::ImplicitBody {
                 actual,
                 expected,
                 source_info: spec_body.space0.source_info,
@@ -347,23 +348,35 @@ fn eval_implicit_body_asserts(
     }
 }
 
-/// Evaluates captures from this HTTP `http_response`, given a set of `variables`.
+/// Evaluates captures from a list of HTTP `http_responses`, given a set of `variables`.
 pub fn eval_captures(
     response: &Response,
-    http_response: &http::Response,
+    http_responses: &[&http::Response],
     cache: &mut BodyCache,
     variables: &mut VariableSet,
 ) -> Result<Vec<CaptureResult>, RunnerError> {
     let mut captures = vec![];
     for capture in response.captures() {
-        let capture_result = capture::eval_capture(capture, variables, http_response, cache)?;
+        let capture_result = capture::eval_capture(capture, variables, http_responses, cache)?;
         // Update variables now so the captures set is ready in case
         // the next captures reference this new variable.
         let name = capture_result.name.clone();
         let value = capture_result.value.clone();
-        if let Err(error) = variables.insert(name, value) {
-            let source_info = capture.name.source_info;
-            return Err(error.to_runner_error(source_info));
+
+        // If the capture is redacted, we insert it in the variable set. Only secrets strings
+        // are supported so all other `Value` variants will trigger an error.
+        if capture.redacted {
+            match value {
+                Value::String(secret) => variables.insert_secret(name, secret),
+                _ => {
+                    let source_info = capture.name.source_info;
+                    let kind = RunnerErrorKind::UnsupportedSecretType(value.kind().to_string());
+                    return Err(RunnerError::new(source_info, kind, false));
+                }
+            }
+        } else {
+            // We insert a public capture.
+            variables.insert(name, value);
         }
         captures.push(capture_result);
     }
@@ -437,7 +450,7 @@ mod tests {
             eval_asserts(
                 &user_response(),
                 &variables,
-                &http::xml_two_users_http_response(),
+                &[&http::xml_two_users_http_response()],
                 &mut cache,
                 &context_dir,
             ),
@@ -447,8 +460,8 @@ mod tests {
                 predicate_result: Some(Err(RunnerError::new(
                     SourceInfo::new(Pos::new(1, 0), Pos::new(1, 0)),
                     RunnerErrorKind::AssertFailure {
-                        actual: "int <2>".to_string(),
-                        expected: "int <3>".to_string(),
+                        actual: "integer <2>".to_string(),
+                        expected: "integer <3>".to_string(),
                         type_mismatch: false,
                     },
                     true
@@ -462,12 +475,12 @@ mod tests {
         assert_eq!(
             eval_version_status_asserts(&user_response(), &http::xml_two_users_http_response(),),
             vec![
-                AssertResult::Version {
+                AssertResult::ImplicitVersion {
                     actual: String::from("HTTP/1.0"),
                     expected: String::from("HTTP/1.0"),
                     source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 9)),
                 },
-                AssertResult::Status {
+                AssertResult::ImplicitStatus {
                     actual: 200,
                     expected: 200,
                     source_info: SourceInfo::new(Pos::new(2, 10), Pos::new(2, 13)),
@@ -484,7 +497,7 @@ mod tests {
         assert_eq!(
             eval_captures(
                 &user_response(),
-                &http::xml_two_users_http_response(),
+                &[&http::xml_two_users_http_response()],
                 &mut cache,
                 &mut variables,
             )
