@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,12 +24,12 @@ use crate::http::{Call, Cookie, CurlCmd};
 use crate::runner::error::RunnerError;
 use crate::runner::output::Output;
 use crate::runner::value::Value;
-use crate::runner::RunnerErrorKind;
+use crate::runner::{RunnerErrorKind, VariableSet};
 use crate::util::path::ContextDir;
 use crate::util::term::Stdout;
 
 /// Represents the result of a valid Hurl file execution.
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, Debug, Default, PartialEq, Eq)]
 pub struct HurlResult {
     /// The entries result for this run.
     pub entries: Vec<EntryResult>,
@@ -41,6 +41,8 @@ pub struct HurlResult {
     pub cookies: Vec<Cookie>,
     /// Start of the run (in "UNIX timestamp").
     pub timestamp: i64,
+    /// The set of variables, updated at the end of the run execution.
+    pub variables: VariableSet,
 }
 
 impl HurlResult {
@@ -136,25 +138,25 @@ impl Default for EntryResult {
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum AssertResult {
     /// Implicit HTTP version assert (like HTTP/3, HTTP/2 etc...).
-    Version {
+    ImplicitVersion {
         actual: String,
         expected: String,
         source_info: SourceInfo,
     },
     /// Implicit HTTP status code assert.
-    Status {
+    ImplicitStatus {
         actual: u64,
         expected: u64,
         source_info: SourceInfo,
     },
     /// Implicit HTTP response header assert.
-    Header {
+    ImplicitHeader {
         actual: Result<String, RunnerError>,
         expected: String,
         source_info: SourceInfo,
     },
     /// Implicit HTTP response body assert.
-    Body {
+    ImplicitBody {
         actual: Result<Value, RunnerError>,
         expected: Result<Value, RunnerError>,
         source_info: SourceInfo,
@@ -163,7 +165,7 @@ pub enum AssertResult {
     Explicit {
         actual: Result<Option<Value>, RunnerError>,
         source_info: SourceInfo,
-        predicate_result: Option<PredicateResult>,
+        predicate_result: Option<Result<(), RunnerError>>,
     },
 }
 
@@ -179,8 +181,6 @@ pub struct CaptureResult {
     pub value: Value,
 }
 
-pub type PredicateResult = Result<(), RunnerError>;
-
 impl EntryResult {
     /// Writes the last HTTP response of this entry result to this `output`.
     /// The HTTP response can be decompressed if the entry's `compressed` option has been set.
