@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -47,6 +47,7 @@ fn eval_template_element(
 mod tests {
     use hurl_core::ast::{Expr, ExprKind, SourceInfo, Variable, Whitespace};
     use hurl_core::reader::Pos;
+    use hurl_core::typing::ToSource;
 
     use super::*;
     use crate::runner::{Number, RunnerErrorKind, Value};
@@ -79,7 +80,7 @@ mod tests {
             eval_template_element(
                 &TemplateElement::String {
                     value: "World".to_string(),
-                    encoded: "World".to_string(),
+                    source: "World".to_source(),
                 },
                 &variables
             )
@@ -88,9 +89,7 @@ mod tests {
         );
 
         let mut variables = VariableSet::new();
-        variables
-            .insert("name".to_string(), Value::String("World".to_string()))
-            .unwrap();
+        variables.insert("name".to_string(), Value::String("World".to_string()));
         assert_eq!(
             eval_template_element(&template_element_expression(), &variables).unwrap(),
             "World".to_string()
@@ -100,15 +99,13 @@ mod tests {
     #[test]
     fn test_template_element_error() {
         let mut variables = VariableSet::new();
-        variables
-            .insert(
-                "name".to_string(),
-                Value::List(vec![
-                    Value::Number(Number::Integer(1)),
-                    Value::Number(Number::Integer(2)),
-                ]),
-            )
-            .unwrap();
+        variables.insert(
+            "name".to_string(),
+            Value::List(vec![
+                Value::Number(Number::Integer(1)),
+                Value::Number(Number::Integer(2)),
+            ]),
+        );
         let error = eval_template_element(&template_element_expression(), &variables)
             .err()
             .unwrap();
