@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -168,7 +168,7 @@ fn libxml_eval_xpath(
 
     // libxml2 prints to stdout warning and errors, so we mut it.
     unsafe {
-        libxml::bindings::initGenericErrorDefaultFunc(&mut Some(silentErrorFunc));
+        libxml::bindings::xmlSetGenericErrorFunc(ptr::null_mut(), Some(silentErrorFunc));
     }
 
     if support_ns {
