@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,12 +15,16 @@
  * limitations under the License.
  *
  */
-
+//! Exposes Hurl AST nodes (see [Hurl grammar](https://hurl.dev/docs/grammar.html)).
 pub use self::core::*;
-pub use self::json::{
-    ListElement as JsonListElement, ObjectElement as JsonObjectElement, Value as JsonValue,
-};
+pub use self::json::{JsonListElement, JsonObjectElement, JsonValue};
+pub use self::option::*;
+pub use self::primitive::*;
+pub use self::section::*;
 
 mod core;
-mod display;
 mod json;
+mod option;
+mod primitive;
+mod section;
+pub mod visit;
