@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,7 +22,6 @@ use crate::reader::Reader;
 ///
 /// Encoded
 /// YW55IGNhcm5hbCBwbGVhcw==    any carnal pleas   # [97, 110, 121, 32, 99, 97, 114, 110, 97, 108, 32, 112, 108, 101, 97, 115]
-
 pub fn parse(reader: &mut Reader) -> Vec<u8> {
     let mut bytes = vec![];
     let mut buf = vec![]; // base64 text
@@ -157,20 +156,20 @@ fn padding(reader: &mut Reader) -> String {
 }
 
 fn decode_two_chars(c1: i32, c2: i32) -> Vec<u8> {
-    vec![((c1 << 2 & 255) + (c2 >> 4)) as u8]
+    vec![(((c1 << 2) & 255) + (c2 >> 4)) as u8]
 }
 
 fn decode_three_chars(c1: i32, c2: i32, c3: i32) -> Vec<u8> {
     vec![
-        ((c1 << 2 & 255) + (c2 >> 4)) as u8,
-        ((c2 << 4 & 255) + (c3 >> 2)) as u8,
+        (((c1 << 2) & 255) + (c2 >> 4)) as u8,
+        (((c2 << 4) & 255) + (c3 >> 2)) as u8,
     ]
 }
 
 fn decode_four_chars(c1: i32, c2: i32, c3: i32, c4: i32) -> Vec<u8> {
     vec![
-        ((c1 << 2 & 255) + (c2 >> 4)) as u8,
-        ((c2 << 4 & 255) + (c3 >> 2)) as u8,
+        (((c1 << 2) & 255) + (c2 >> 4)) as u8,
+        (((c2 << 4) & 255) + (c3 >> 2)) as u8,
         (((c3 << 6) & 255) + c4) as u8,
     ]
 }
@@ -178,32 +177,33 @@ fn decode_four_chars(c1: i32, c2: i32, c3: i32, c4: i32) -> Vec<u8> {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::reader::CharPos;
 
     #[test]
     fn test_decode_one_block() {
         let mut reader = Reader::new("");
         assert_eq!(parse(&mut reader), vec![] as Vec<u8>);
-        assert_eq!(reader.cursor().index, 0);
+        assert_eq!(reader.cursor().index, CharPos(0));
 
         let mut reader = Reader::new("AA==;");
         assert_eq!(parse(&mut reader), vec![0]);
-        assert_eq!(reader.cursor().index, 4);
+        assert_eq!(reader.cursor().index, CharPos(4));
 
         let mut reader = Reader::new("AA");
         assert_eq!(parse(&mut reader), vec![0]);
-        assert_eq!(reader.cursor().index, 2);
+        assert_eq!(reader.cursor().index, CharPos(2));
 
         let mut reader = Reader::new("AA;");
         assert_eq!(parse(&mut reader), vec![0]);
-        assert_eq!(reader.cursor().index, 2);
+        assert_eq!(reader.cursor().index, CharPos(2));
 
         let mut reader = Reader::new("TWE=;");
         assert_eq!(parse(&mut reader), vec![77, 97]);
-        assert_eq!(reader.cursor().index, 4);
+        assert_eq!(reader.cursor().index, CharPos(4));
 
         let mut reader = Reader::new("TWFu;");
         assert_eq!(parse(&mut reader), vec![77, 97, 110]);
-        assert_eq!(reader.cursor().index, 4);
+        assert_eq!(reader.cursor().index, CharPos(4));
     }
 
     /*
@@ -246,7 +246,7 @@ mod tests {
     fn test_decode_with_whitespace() {
         let mut reader = Reader::new("TW E=\n;");
         assert_eq!(parse(&mut reader), vec![77, 97]);
-        assert_eq!(reader.cursor().index, 5);
+        assert_eq!(reader.cursor().index, CharPos(5));
     }
 
     #[test]
