@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -77,7 +77,8 @@ mod tests {
     use super::super::error::*;
     use super::*;
     use crate::ast::{JsonListElement, JsonValue, SourceInfo, Template, TemplateElement};
-    use crate::reader::Pos;
+    use crate::reader::{CharPos, Pos};
+    use crate::typing::ToSource;
 
     #[test]
     fn test_bytes_json() {
@@ -105,7 +106,7 @@ mod tests {
                 ],
             })
         );
-        assert_eq!(reader.cursor().index, 7);
+        assert_eq!(reader.cursor().index, CharPos(7));
 
         let mut reader = Reader::new("{ } ");
         assert_eq!(
@@ -115,25 +116,26 @@ mod tests {
                 elements: vec![],
             })
         );
-        assert_eq!(reader.cursor().index, 3);
+        assert_eq!(reader.cursor().index, CharPos(3));
 
         let mut reader = Reader::new("true");
         assert_eq!(
             bytes(&mut reader).unwrap(),
             Bytes::Json(JsonValue::Boolean(true))
         );
-        assert_eq!(reader.cursor().index, 4);
+        assert_eq!(reader.cursor().index, CharPos(4));
 
         let mut reader = Reader::new("\"\" x");
+        let template = Template::new(
+            Some('"'),
+            vec![],
+            SourceInfo::new(Pos::new(1, 2), Pos::new(1, 2)),
+        );
         assert_eq!(
             bytes(&mut reader).unwrap(),
-            Bytes::Json(JsonValue::String(Template {
-                delimiter: Some('"'),
-                elements: vec![],
-                source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 2)),
-            }))
+            Bytes::Json(JsonValue::String(template))
         );
-        assert_eq!(reader.cursor().index, 2);
+        assert_eq!(reader.cursor().index, CharPos(2));
     }
 
     #[test]
@@ -196,17 +198,15 @@ mod tests {
     #[test]
     fn test_bytes_string() {
         let mut reader = Reader::new("`foo`  ");
-        assert_eq!(
-            bytes(&mut reader).unwrap(),
-            Bytes::OnelineString(Template {
-                delimiter: Some('`'),
-                elements: vec![TemplateElement::String {
-                    value: "foo".to_string(),
-                    encoded: "foo".to_string()
-                }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6))
-            })
+        let template = Template::new(
+            Some('`'),
+            vec![TemplateElement::String {
+                value: "foo".to_string(),
+                source: "foo".to_source(),
+            }],
+            SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6)),
         );
-        assert_eq!(reader.cursor().index, 5);
+        assert_eq!(bytes(&mut reader).unwrap(), Bytes::OnelineString(template));
+        assert_eq!(reader.cursor().index.0, 5);
     }
 }
