@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,8 +16,7 @@
  *
  */
 use crate::ast::{Expr, ExprKind, SourceInfo, Variable};
-use crate::parser::ParseResult;
-use crate::parser::{function, ParseError, ParseErrorKind};
+use crate::parser::{function, ParseError, ParseErrorKind, ParseResult};
 use crate::reader::Reader;
 
 /// Parse an expression
