@@ -103,7 +103,7 @@ fn filename_text(reader: &mut Reader) -> String {
         match reader.read() {
             None => break,
             Some(c) => {
-                if ['#', ';', '{', '}', ' ', '\n', '\\'].contains(&c) {
+                if ['#', ';', '{', '}', ' ', '\n', '\r', '\\'].contains(&c) {
                     reader.seek(save);
                     break;
                 } else {
@@ -143,7 +143,7 @@ fn filename_escaped_char(reader: &mut Reader) -> ParseResult<char> {
 mod tests {
     use super::*;
     use crate::ast::{Expr, ExprKind, Placeholder, Variable, Whitespace};
-    use crate::reader::Pos;
+    use crate::reader::{CharPos, Pos};
 
     #[test]
     fn test_filename() {
@@ -159,7 +159,7 @@ mod tests {
                 SourceInfo::new(Pos::new(1, 1), Pos::new(1, 14)),
             )
         );
-        assert_eq!(reader.cursor().index, 13);
+        assert_eq!(reader.cursor().index, CharPos(13));
 
         let mut reader = Reader::new("data.bin");
         assert_eq!(
@@ -173,7 +173,7 @@ mod tests {
                 SourceInfo::new(Pos::new(1, 1), Pos::new(1, 9)),
             )
         );
-        assert_eq!(reader.cursor().index, 8);
+        assert_eq!(reader.cursor().index, CharPos(8));
     }
 
     #[test]
@@ -190,7 +190,7 @@ mod tests {
                 SourceInfo::new(Pos::new(1, 1), Pos::new(1, 19)),
             )
         );
-        assert_eq!(reader.cursor().index, 18);
+        assert_eq!(reader.cursor().index, CharPos(18));
     }
 
     #[test]
@@ -207,7 +207,7 @@ mod tests {
                 SourceInfo::new(Pos::new(1, 1), Pos::new(1, 11))
             )
         );
-        assert_eq!(reader.cursor().index, 10);
+        assert_eq!(reader.cursor().index, CharPos(10));
     }
 
     #[test]
@@ -293,4 +293,20 @@ mod tests {
             )
         );
     }
+
+    #[test]
+    fn test_filename_with_crlf() {
+        let mut reader = Reader::new("foo\r\n");
+        assert_eq!(
+            parse(&mut reader).unwrap(),
+            Template::new(
+                None,
+                vec![TemplateElement::String {
+                    value: "foo".to_string(),
+                    source: "foo".to_source(),
+                }],
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 4)),
+            )
+        );
+    }
 }
