@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,13 +15,13 @@
  * limitations under the License.
  *
  */
-use crate::ast::{Filter, FilterValue, SourceInfo, Whitespace};
+use crate::ast::{Filter, FilterValue, IntegerValue, SourceInfo, Whitespace};
 use crate::combinator::{choice, ParseError as ParseErrorTrait};
-use crate::parser::number::natural;
+use crate::parser::number::integer;
 use crate::parser::primitives::{one_or_more_spaces, try_literal, zero_or_more_spaces};
 use crate::parser::query::regex_value;
 use crate::parser::string::quoted_template;
-use crate::parser::{ParseError, ParseErrorKind, ParseResult};
+use crate::parser::{placeholder, ParseError, ParseErrorKind, ParseResult};
 use crate::reader::Reader;
 
 pub fn filters(reader: &mut Reader) -> ParseResult<Vec<(Whitespace, Filter)>> {
@@ -53,23 +53,34 @@ pub fn filter(reader: &mut Reader) -> ParseResult<Filter> {
     let start = reader.cursor();
     let value = choice(
         &[
+            base64_decode_filter,
+            base64_encode_filter,
+            base64_url_safe_decode_filter,
+            base64_url_safe_encode_filter,
             count_filter,
             days_after_now_filter,
             days_before_now_filter,
             decode_filter,
+            first_filter,
             format_filter,
             html_decode_filter,
             html_encode_filter,
             jsonpath_filter,
+            last_filter,
+            location_filter,
             nth_filter,
             regex_filter,
+            replace_regex_filter,
             replace_filter,
             split_filter,
-            to_float_filter,
-            to_int_filter,
             to_date_filter,
+            to_float_filter,
+            to_hex_filter,
+            to_int_filter,
+            to_string_filter,
             url_decode_filter,
             url_encode_filter,
+            url_query_param_filter,
             xpath_filter,
         ],
         reader,
@@ -92,6 +103,26 @@ pub fn filter(reader: &mut Reader) -> ParseResult<Filter> {
     Ok(Filter { source_info, value })
 }
 
+fn base64_decode_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("base64Decode", reader)?;
+    Ok(FilterValue::Base64Decode)
+}
+
+fn base64_encode_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("base64Encode", reader)?;
+    Ok(FilterValue::Base64Encode)
+}
+
+fn base64_url_safe_decode_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("base64UrlSafeDecode", reader)?;
+    Ok(FilterValue::Base64UrlSafeDecode)
+}
+
+fn base64_url_safe_encode_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("base64UrlSafeEncode", reader)?;
+    Ok(FilterValue::Base64UrlSafeEncode)
+}
+
 fn count_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     try_literal("count", reader)?;
     Ok(FilterValue::Count)
@@ -114,6 +145,11 @@ fn decode_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     Ok(FilterValue::Decode { space0, encoding })
 }
 
+fn first_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("first", reader)?;
+    Ok(FilterValue::First)
+}
+
 fn format_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     try_literal("format", reader)?;
     let space0 = one_or_more_spaces(reader)?;
@@ -138,13 +174,40 @@ fn jsonpath_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     Ok(FilterValue::JsonPath { space0, expr })
 }
 
+fn last_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("last", reader)?;
+    Ok(FilterValue::Last)
+}
+
+fn location_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("location", reader)?;
+    Ok(FilterValue::Location)
+}
+
 fn nth_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     try_literal("nth", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let n = natural(reader)?;
+    let n = integer_value(reader)?;
     Ok(FilterValue::Nth { space0, n })
 }
 
+fn integer_value(reader: &mut Reader) -> ParseResult<IntegerValue> {
+    let start = reader.cursor();
+    match integer(reader) {
+        Ok(v) => Ok(IntegerValue::Literal(v)),
+        Err(_) => {
+            reader.seek(start);
+            let placeholder = placeholder::parse(reader).map_err(|e| {
+                let kind = ParseErrorKind::Expecting {
+                    value: "integer".to_string(),
+                };
+                ParseError::new(e.pos, false, kind)
+            })?;
+            Ok(IntegerValue::Placeholder(placeholder))
+        }
+    }
+}
+
 fn regex_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     try_literal("regex", reader)?;
     let space0 = one_or_more_spaces(reader)?;
@@ -155,7 +218,7 @@ fn regex_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
 fn replace_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     try_literal("replace", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let old_value = regex_value(reader)?;
+    let old_value = quoted_template(reader).map_err(|e| e.to_non_recoverable())?;
     let space1 = one_or_more_spaces(reader)?;
     let new_value = quoted_template(reader).map_err(|e| e.to_non_recoverable())?;
     Ok(FilterValue::Replace {
@@ -166,6 +229,20 @@ fn replace_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     })
 }
 
+fn replace_regex_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("replaceRegex", reader)?;
+    let space0 = one_or_more_spaces(reader)?;
+    let pattern = regex_value(reader)?;
+    let space1 = one_or_more_spaces(reader)?;
+    let new_value = quoted_template(reader).map_err(|e| e.to_non_recoverable())?;
+    Ok(FilterValue::ReplaceRegex {
+        space0,
+        pattern,
+        space1,
+        new_value,
+    })
+}
+
 fn split_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     try_literal("split", reader)?;
     let space0 = one_or_more_spaces(reader)?;
@@ -185,11 +262,21 @@ fn to_float_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     Ok(FilterValue::ToFloat)
 }
 
+fn to_hex_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("toHex", reader)?;
+    Ok(FilterValue::ToHex)
+}
+
 fn to_int_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     try_literal("toInt", reader)?;
     Ok(FilterValue::ToInt)
 }
 
+fn to_string_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("toString", reader)?;
+    Ok(FilterValue::ToString)
+}
+
 fn url_encode_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     try_literal("urlEncode", reader)?;
     Ok(FilterValue::UrlEncode)
@@ -200,6 +287,13 @@ fn url_decode_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     Ok(FilterValue::UrlDecode)
 }
 
+fn url_query_param_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
+    try_literal("urlQueryParam", reader)?;
+    let space0 = one_or_more_spaces(reader)?;
+    let param = quoted_template(reader).map_err(|e| e.to_non_recoverable())?;
+    Ok(FilterValue::UrlQueryParam { space0, param })
+}
+
 fn xpath_filter(reader: &mut Reader) -> ParseResult<FilterValue> {
     try_literal("xpath", reader)?;
     let space0 = one_or_more_spaces(reader)?;
