@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,13 +15,14 @@
  * limitations under the License.
  *
  */
+use super::placeholder;
 use crate::ast::{SourceInfo, Template, TemplateElement};
 use crate::parser::primitives::try_literal;
 use crate::parser::{string, ParseError, ParseErrorKind, ParseResult};
 use crate::reader::Reader;
+use crate::typing::ToSource;
 
-use super::placeholder;
-
+/// Parses a string into a [key string](https://hurl.dev/docs/grammar.html#key-string).
 pub fn parse(reader: &mut Reader) -> ParseResult<Template> {
     let start = reader.cursor();
     let mut elements = vec![];
@@ -39,8 +40,8 @@ pub fn parse(reader: &mut Reader) -> ParseResult<Template> {
                     if value.is_empty() {
                         break;
                     }
-                    let encoded = reader.read_from(start.index);
-                    let element = TemplateElement::String { value, encoded };
+                    let source = reader.read_from(start.index).to_source();
+                    let element = TemplateElement::String { value, source };
                     elements.push(element);
                 } else {
                     return Err(e);
@@ -54,8 +55,8 @@ pub fn parse(reader: &mut Reader) -> ParseResult<Template> {
         };
         return Err(ParseError::new(start.pos, false, kind));
     }
-    if let Some(TemplateElement::String { encoded, .. }) = elements.first() {
-        if encoded.starts_with('[') {
+    if let Some(TemplateElement::String { source, .. }) = elements.first() {
+        if source.starts_with('[') {
             let kind = ParseErrorKind::Expecting {
                 value: "key-string".to_string(),
             };
@@ -64,11 +65,8 @@ pub fn parse(reader: &mut Reader) -> ParseResult<Template> {
     }
 
     let end = reader.cursor();
-    Ok(Template {
-        delimiter: None,
-        elements,
-        source_info: SourceInfo::new(start.pos, end.pos),
-    })
+    let template = Template::new(None, elements, SourceInfo::new(start.pos, end.pos));
+    Ok(template)
 }
 
 fn key_string_content(reader: &mut Reader) -> ParseResult<String> {
@@ -148,93 +146,93 @@ fn key_string_escaped_char(reader: &mut Reader) -> ParseResult<char> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::reader::Pos;
+    use crate::reader::{CharPos, Pos};
 
     #[test]
     fn test_key_string() {
         let mut reader = Reader::new("aaa\\: ");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "aaa:".to_string(),
-                    encoded: "aaa\\:".to_string(),
+                    source: "aaa\\:".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6)),
+            )
         );
-        assert_eq!(reader.cursor().index, 5);
+        assert_eq!(reader.cursor().index, CharPos(5));
 
         let mut reader = Reader::new("$top:");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "$top".to_string(),
-                    encoded: "$top".to_string(),
+                    source: "$top".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 5)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 5)),
+            )
         );
-        assert_eq!(reader.cursor().index, 4);
+        assert_eq!(reader.cursor().index, CharPos(4));
 
         let mut reader = Reader::new("key\\u{20}\\u{3a} :");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "key :".to_string(),
-                    encoded: "key\\u{20}\\u{3a}".to_string(),
+                    source: "key\\u{20}\\u{3a}".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 16)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 16))
+            )
         );
-        assert_eq!(reader.cursor().index, 15);
+        assert_eq!(reader.cursor().index, CharPos(15));
 
         let mut reader = Reader::new("values\\u{5b}0\\u{5d} :");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "values[0]".to_string(),
-                    encoded: "values\\u{5b}0\\u{5d}".to_string(),
+                    source: "values\\u{5b}0\\u{5d}".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 20)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 20))
+            )
         );
-        assert_eq!(reader.cursor().index, 19);
+        assert_eq!(reader.cursor().index, CharPos(19));
 
         let mut reader = Reader::new("values[0] :");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "values[0]".to_string(),
-                    encoded: "values[0]".to_string(),
+                    source: "values[0]".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 10)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 10)),
+            )
         );
-        assert_eq!(reader.cursor().index, 9);
+        assert_eq!(reader.cursor().index, CharPos(9));
 
         let mut reader = Reader::new("\\u{5b}0\\u{5d}");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "[0]".to_string(),
-                    encoded: "\\u{5b}0\\u{5d}".to_string(),
+                    source: "\\u{5b}0\\u{5d}".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 14)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 14)),
+            )
         );
-        assert_eq!(reader.cursor().index, 13);
+        assert_eq!(reader.cursor().index, CharPos(13));
     }
 
     #[test]
@@ -253,7 +251,7 @@ mod tests {
         let mut reader = Reader::new("[0]:");
         let error = parse(&mut reader).err().unwrap();
         assert!(!error.recoverable);
-        assert_eq!(reader.cursor().index, 3);
+        assert_eq!(reader.cursor().index, CharPos(3));
 
         let mut reader = Reader::new("\\l");
         let error = parse(&mut reader).err().unwrap();
@@ -281,18 +279,18 @@ mod tests {
     fn test_key_string_text() {
         let mut reader = Reader::new("aaa\\:");
         assert_eq!(key_string_text(&mut reader), "aaa");
-        assert_eq!(reader.cursor().index, 3);
+        assert_eq!(reader.cursor().index, CharPos(3));
     }
 
     #[test]
     fn test_key_string_escaped_char() {
         let mut reader = Reader::new("\\u{0a}");
         assert_eq!(key_string_escaped_char(&mut reader).unwrap(), '\n');
-        assert_eq!(reader.cursor().index, 6);
+        assert_eq!(reader.cursor().index, CharPos(6));
 
         let mut reader = Reader::new("\\:");
         assert_eq!(key_string_escaped_char(&mut reader).unwrap(), ':');
-        assert_eq!(reader.cursor().index, 2);
+        assert_eq!(reader.cursor().index, CharPos(2));
 
         let mut reader = Reader::new("x");
         let error = key_string_escaped_char(&mut reader).err().unwrap();
@@ -304,6 +302,6 @@ mod tests {
             }
         );
         assert!(error.recoverable);
-        assert_eq!(reader.cursor().index, 0);
+        assert_eq!(reader.cursor().index, CharPos(0));
     }
 }
