@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
  */
 use crate::ast::{
     GraphQl, GraphQlVariables, MultilineString, MultilineStringAttribute, MultilineStringKind,
-    SourceInfo, Template, Text, Whitespace,
+    SourceInfo, Template, Whitespace,
 };
 use crate::combinator::{choice, optional, zero_or_more};
 use crate::parser::json::object_value;
@@ -32,41 +32,65 @@ pub fn multiline_string(reader: &mut Reader) -> ParseResult<MultilineString> {
     choice(&[json_text, xml_text, graphql, plain_text], reader)
 }
 
-fn text(lang: &str, reader: &mut Reader) -> ParseResult<(Text, Vec<MultilineStringAttribute>)> {
-    try_literal(lang, reader)?;
-    drop(try_literal(",", reader));
+fn text(
+    lang: Option<&str>,
+    reader: &mut Reader,
+) -> ParseResult<(
+    Vec<MultilineStringAttribute>,
+    Whitespace,
+    Whitespace,
+    Template,
+)> {
+    if let Some(lang) = lang {
+        try_literal(lang, reader)?;
+        drop(try_literal(",", reader));
+    }
     let attributes = multiline_string_attributes(reader)?;
     let escape = attributes.contains(&MultilineStringAttribute::Escape);
     let space = zero_or_more_spaces(reader)?;
     let newline = newline(reader)?;
     let value = multiline_string_value(reader, escape)?;
-    Ok((
-        Text {
-            space,
-            newline,
-            value,
-        },
+    Ok((attributes, space, newline, value))
+}
+
+fn plain_text(reader: &mut Reader) -> ParseResult<MultilineString> {
+    let (attributes, space, newline, text) = text(None, reader)?;
+    let kind = MultilineStringKind::Text(text);
+    Ok(MultilineString {
         attributes,
-    ))
+        space,
+        newline,
+        kind,
+    })
 }
 
 fn json_text(reader: &mut Reader) -> ParseResult<MultilineString> {
-    let (text, attributes) = text("json", reader)?;
+    let (attributes, space, newline, text) = text(Some("json"), reader)?;
     let kind = MultilineStringKind::Json(text);
-    Ok(MultilineString { kind, attributes })
+    Ok(MultilineString {
+        attributes,
+        space,
+        newline,
+        kind,
+    })
 }
 
 fn xml_text(reader: &mut Reader) -> ParseResult<MultilineString> {
-    let (text, attributes) = text("xml", reader)?;
+    let (attributes, space, newline, text) = text(Some("xml"), reader)?;
     let kind = MultilineStringKind::Xml(text);
-    Ok(MultilineString { kind, attributes })
+    Ok(MultilineString {
+        attributes,
+        space,
+        newline,
+        kind,
+    })
 }
 
 fn graphql(reader: &mut Reader) -> ParseResult<MultilineString> {
     try_literal("graphql", reader)?;
-    let space = zero_or_more_spaces(reader)?;
     drop(try_literal(",", reader));
     let attributes = multiline_string_attributes(reader)?;
+    let space = zero_or_more_spaces(reader)?;
     let newline = newline(reader)?;
 
     let mut chars = vec![];
@@ -90,19 +114,19 @@ fn graphql(reader: &mut Reader) -> ParseResult<MultilineString> {
                     };
 
                     let elements = template::templatize(encoded_string)?;
-                    let template = Template {
-                        delimiter: None,
-                        elements,
-                        source_info: SourceInfo::new(start.pos, end.pos),
-                    };
+                    let template =
+                        Template::new(None, elements, SourceInfo::new(start.pos, end.pos));
                     let kind = MultilineStringKind::GraphQl(GraphQl {
-                        space,
-                        newline,
                         value: template,
                         variables: Some(variables),
                     });
                     let attributes = vec![];
-                    return Ok(MultilineString { kind, attributes });
+                    return Ok(MultilineString {
+                        attributes,
+                        space,
+                        newline,
+                        kind,
+                    });
                 }
             }
         }
@@ -116,30 +140,32 @@ fn graphql(reader: &mut Reader) -> ParseResult<MultilineString> {
     };
 
     let elements = template::templatize(encoded_string)?;
-    let template = Template {
-        delimiter: None,
-        elements,
-        source_info: SourceInfo::new(start.pos, end.pos),
-    };
-
+    let template = Template::new(None, elements, SourceInfo::new(start.pos, end.pos));
     let kind = MultilineStringKind::GraphQl(GraphQl {
-        space,
-        newline,
         value: template,
         variables: None,
     });
-    Ok(MultilineString { kind, attributes })
+    Ok(MultilineString {
+        attributes,
+        space,
+        newline,
+        kind,
+    })
 }
 
 fn multiline_string_attributes(reader: &mut Reader) -> ParseResult<Vec<MultilineStringAttribute>> {
     let mut attributes = vec![];
-    zero_or_more_spaces(reader)?;
 
     while !reader.is_eof() && reader.peek() != Some('\n') && reader.peek() != Some('\r') {
-        let pos = reader.cursor().pos;
+        let start = reader.cursor();
+        zero_or_more_spaces(reader)?;
+        let c = reader.peek();
+        if c == Some('\r') || c == Some('\n') {
+            reader.seek(start);
+            break;
+        }
         let attribute = reader
-            .read_while(|c| c != ',' && c != '\r' && c != '\n')
-            .trim()
+            .read_while(|c| c != ',' && c != '\r' && c != '\n' && !c.is_whitespace())
             .to_string();
         if attribute == "escape" {
             attributes.push(MultilineStringAttribute::Escape);
@@ -149,7 +175,7 @@ fn multiline_string_attributes(reader: &mut Reader) -> ParseResult<Vec<Multiline
             let kind = ParseErrorKind::MultilineAttribute(attribute);
             return Err(ParseError {
                 kind,
-                pos,
+                pos: start.pos,
                 recoverable: false,
             });
         }
@@ -212,21 +238,6 @@ fn graphql_variables(reader: &mut Reader) -> ParseResult<GraphQlVariables> {
     })
 }
 
-fn plain_text(reader: &mut Reader) -> ParseResult<MultilineString> {
-    let space = zero_or_more_spaces(reader)?;
-    drop(try_literal(",", reader));
-    let attributes = multiline_string_attributes(reader)?;
-    let escape = attributes.contains(&MultilineStringAttribute::Escape);
-    let newline = newline(reader)?;
-    let value = multiline_string_value(reader, escape)?;
-    let kind = MultilineStringKind::Text(Text {
-        space,
-        newline,
-        value,
-    });
-    Ok(MultilineString { kind, attributes })
-}
-
 fn multiline_string_value(reader: &mut Reader, escape: bool) -> ParseResult<Template> {
     let mut chars = vec![];
 
@@ -259,20 +270,16 @@ fn multiline_string_value(reader: &mut Reader, escape: bool) -> ParseResult<Temp
     };
 
     let elements = template::templatize(encoded_string)?;
-
-    Ok(Template {
-        delimiter: None,
-        elements,
-        source_info: SourceInfo::new(start.pos, end.pos),
-    })
+    let template = Template::new(None, elements, SourceInfo::new(start.pos, end.pos));
+    Ok(template)
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::ast::{JsonObjectElement, JsonValue, TemplateElement};
-    use crate::reader::Pos;
-
     use super::*;
+    use crate::ast::{JsonObjectElement, JsonValue, TemplateElement};
+    use crate::reader::{CharPos, Pos};
+    use crate::typing::ToSource;
 
     #[test]
     fn test_multiline_string_text() {
@@ -280,25 +287,23 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-                kind: MultilineStringKind::Text(Text {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
-                    },
-                    newline: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
-                            value: "line1\nline2\nline3\n".to_string(),
-                            encoded: "line1\nline2\nline3\n".to_string(),
-                        }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
-                    },
-                }),
-                attributes: vec![]
+                attributes: vec![],
+                space: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
+                },
+                kind: MultilineStringKind::Text(Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "line1\nline2\nline3\n".to_string(),
+                        source: "line1\nline2\nline3\n".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1))
+                )),
             }
         );
 
@@ -306,25 +311,50 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-                kind: MultilineStringKind::Text(Text {
-                    space: Whitespace {
-                        value: "         ".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 13)),
-                    },
-                    newline: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 13), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
-                            value: "line1\nline2\nline3\n".to_string(),
-                            encoded: "line1\nline2\nline3\n".to_string(),
-                        }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
-                    },
-                }),
-                attributes: vec![]
+                attributes: vec![],
+                space: Whitespace {
+                    value: "         ".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 13)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 13), Pos::new(2, 1)),
+                },
+                kind: MultilineStringKind::Text(Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "line1\nline2\nline3\n".to_string(),
+                        source: "line1\nline2\nline3\n".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1))
+                )),
+            }
+        );
+
+        let mut reader = Reader::new("```escape,novariable         \nline1\nline2\nline3\n```");
+        assert_eq!(
+            multiline_string(&mut reader).unwrap(),
+            MultilineString {
+                attributes: vec![
+                    MultilineStringAttribute::Escape,
+                    MultilineStringAttribute::NoVariable
+                ],
+                space: Whitespace {
+                    value: "         ".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 21), Pos::new(1, 30)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 30), Pos::new(2, 1)),
+                },
+                kind: MultilineStringKind::Text(Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "line1\nline2\nline3\n".to_string(),
+                        source: "line1\nline2\nline3\n".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1))
+                )),
             }
         );
     }
@@ -335,25 +365,23 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-                kind: MultilineStringKind::Json(Text {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 8)),
-                    },
-                    newline: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
-                            value: "line1\nline2\nline3\n".to_string(),
-                            encoded: "line1\nline2\nline3\n".to_string(),
-                        }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
-                    },
-                }),
-                attributes: vec![]
+                attributes: vec![],
+                space: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 8)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(2, 1)),
+                },
+                kind: MultilineStringKind::Json(Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "line1\nline2\nline3\n".to_string(),
+                        source: "line1\nline2\nline3\n".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
+                )),
             }
         );
 
@@ -367,25 +395,23 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-                kind: MultilineStringKind::Json(Text {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(1, 15)),
-                    },
-                    newline: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
-                            value: "{\n  \"g_clef\": \"𝄞\"\n}\n".to_string(),
-                            encoded: "{\n  \"g_clef\": \"\\u{1D11E}\"\n}\n".to_string(),
-                        }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
-                    },
-                }),
-                attributes: vec![MultilineStringAttribute::Escape]
+                attributes: vec![MultilineStringAttribute::Escape],
+                space: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(1, 15)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(2, 1)),
+                },
+                kind: MultilineStringKind::Json(Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "{\n  \"g_clef\": \"𝄞\"\n}\n".to_string(),
+                        source: "{\n  \"g_clef\": \"\\u{1D11E}\"\n}\n".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
+                )),
             }
         );
     }
@@ -396,26 +422,26 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
+                attributes: vec![],
+                space: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 11)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(2, 1)),
+                },
                 kind: MultilineStringKind::GraphQl(GraphQl {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 11)),
-                    },
-                    newline: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
+                    value: Template::new(
+                        None,
+                        vec![TemplateElement::String {
                             value: "line1\nline2\nline3\n".to_string(),
-                            encoded: "line1\nline2\nline3\n".to_string(),
+                            source: "line1\nline2\nline3\n".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
-                    },
+                        SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1))
+                    ),
                     variables: None,
                 }),
-                attributes: vec![]
             }
         );
 
@@ -423,26 +449,26 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
+                attributes: vec![],
+                space: Whitespace {
+                    value: "      ".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 17)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 17), Pos::new(2, 1)),
+                },
                 kind: MultilineStringKind::GraphQl(GraphQl {
-                    space: Whitespace {
-                        value: "      ".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 17)),
-                    },
-                    newline: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 17), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
+                    value: Template::new(
+                        None,
+                        vec![TemplateElement::String {
                             value: "line1\nline2\nline3\n".to_string(),
-                            encoded: "line1\nline2\nline3\n".to_string(),
+                            source: "line1\nline2\nline3\n".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
-                    },
+                        SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
+                    ),
                     variables: None,
                 }),
-                attributes: vec![]
             }
         );
     }
@@ -466,44 +492,40 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-                kind: MultilineStringKind::Text(Text {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
-                    },
-                    newline: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 1)),
-                    },
-                }),
-                attributes: vec![]
+                attributes: vec![],
+                space: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
+                },
+                kind: MultilineStringKind::Text(Template::new(
+                    None,
+                    vec![],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(2, 1))
+                )),
             }
         );
         let mut reader = Reader::new("```\r\n```");
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-                kind: MultilineStringKind::Text(Text {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
-                    },
-                    newline: Whitespace {
-                        value: "\r\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 1)),
-                    },
-                }),
-                attributes: vec![]
+                attributes: vec![],
+                space: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
+                },
+                newline: Whitespace {
+                    value: "\r\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
+                },
+                kind: MultilineStringKind::Text(Template::new(
+                    None,
+                    vec![],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(2, 1)),
+                )),
             }
         );
     }
@@ -515,7 +537,7 @@ mod tests {
         assert_eq!(error.pos, Pos::new(1, 4));
         assert_eq!(
             error.kind,
-            ParseErrorKind::MultilineAttribute("Hello World!```".to_string())
+            ParseErrorKind::MultilineAttribute("Hello".to_string())
         );
     }
 
@@ -525,25 +547,23 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-                kind: MultilineStringKind::Text(Text {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
-                    },
-                    newline: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
-                            value: "line1\nline2\nline3\n".to_string(),
-                            encoded: "line1\nline2\nline3\n".to_string(),
-                        }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
-                    },
-                }),
-                attributes: vec![]
+                attributes: vec![],
+                space: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
+                },
+                kind: MultilineStringKind::Text(Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "line1\nline2\nline3\n".to_string(),
+                        source: "line1\nline2\nline3\n".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(5, 1)),
+                )),
             }
         );
     }
@@ -554,25 +574,23 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-                kind: MultilineStringKind::Text(Text {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
-                    },
-                    newline: Whitespace {
+                kind: MultilineStringKind::Text(Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
-                            value: "\n".to_string(),
-                            encoded: "\n".to_string(),
-                        }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(3, 1)),
-                    },
-                }),
-                attributes: vec![]
+                        source: "\n".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(3, 1)),
+                )),
+                attributes: vec![],
+                space: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
+                },
             }
         );
 
@@ -581,25 +599,23 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-                kind: MultilineStringKind::Text(Text {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
-                    },
-                    newline: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
-                            value: "\r\n".to_string(),
-                            encoded: "\r\n".to_string(),
-                        }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(3, 1)),
-                    },
-                }),
-                attributes: vec![]
+                attributes: vec![],
+                space: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(2, 1)),
+                },
+                kind: MultilineStringKind::Text(Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "\r\n".to_string(),
+                        source: "\r\n".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(3, 1)),
+                )),
             }
         );
     }
@@ -611,15 +627,15 @@ mod tests {
             multiline_string_attributes(&mut reader).unwrap(),
             vec![MultilineStringAttribute::Escape]
         );
-        assert_eq!(reader.cursor().index, 6);
+        assert_eq!(reader.cursor().index, CharPos(6));
 
         let mut reader = Reader::new("\n```");
         assert_eq!(multiline_string_attributes(&mut reader).unwrap(), vec![]);
-        assert_eq!(reader.cursor().index, 0);
+        assert_eq!(reader.cursor().index, CharPos(0));
 
         let mut reader = Reader::new("\r\n```");
         assert_eq!(multiline_string_attributes(&mut reader).unwrap(), vec![]);
-        assert_eq!(reader.cursor().index, 0);
+        assert_eq!(reader.cursor().index, CharPos(0));
 
         let mut reader = Reader::new("toto\n```");
         let error = multiline_string_attributes(&mut reader).unwrap_err();
@@ -644,25 +660,23 @@ mod tests {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-                kind: MultilineStringKind::Text(Text {
-                    space: Whitespace {
-                        value: String::new(),
-                        source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 4)),
-                    },
-                    newline: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(2, 1)),
-                    },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
-                            value: "\t\n".to_string(),
-                            encoded: "\\t\n".to_string(),
-                        }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(3, 1)),
-                    },
-                }),
-                attributes: vec![MultilineStringAttribute::Escape]
+                attributes: vec![MultilineStringAttribute::Escape],
+                space: Whitespace {
+                    value: String::new(),
+                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 10)),
+                },
+                newline: Whitespace {
+                    value: "\n".to_string(),
+                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(2, 1)),
+                },
+                kind: MultilineStringKind::Text(Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "\t\n".to_string(),
+                        source: "\\t\n".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(2, 1), Pos::new(3, 1)),
+                )),
             }
         );
     }
@@ -706,27 +720,27 @@ mod tests {
         let mut reader = Reader::new("```");
         assert_eq!(
             multiline_string_value(&mut reader, false).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
-            }
+            Template::new(
+                None,
+                vec![],
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1))
+            )
         );
-        assert_eq!(reader.cursor().index, 3);
+        assert_eq!(reader.cursor().index, CharPos(3));
 
         let mut reader = Reader::new("hello```");
         assert_eq!(
             multiline_string_value(&mut reader, false).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "hello".to_string(),
-                    encoded: "hello".to_string(),
+                    source: "hello".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6))
+            )
         );
-        assert_eq!(reader.cursor().index, 8);
+        assert_eq!(reader.cursor().index, CharPos(8));
     }
 
     #[test]
@@ -748,7 +762,7 @@ variables {
         assert_eq!(
             multiline_string(&mut reader).unwrap(),
             MultilineString {
-            kind: MultilineStringKind::GraphQl(GraphQl {
+                attributes: vec![],
                 space: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 11)),
@@ -757,55 +771,57 @@ variables {
                     value: "\n".to_string(),
                     source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(2, 1)),
                 },
-                value: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
-                        value: "query Human($name: String!) {\n  human(name: $name) {\n    name\n    height(unit: FOOT)\n}\n\n".to_string(),
-                        encoded:
-                            "query Human($name: String!) {\n  human(name: $name) {\n    name\n    height(unit: FOOT)\n}\n\n".to_string()
-                    }],
-                    source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(8, 1)),
-                },
-                variables: Some(GraphQlVariables {
-                    space: Whitespace {
-                        value: " ".to_string(),
-                        source_info: SourceInfo::new(Pos::new(8, 10), Pos::new(8, 11)),
-                    },
-                    value: JsonValue::Object {
-                        space0: "\n  ".to_string(),
-                        elements: vec![JsonObjectElement {
-                            space0: String::new(),
-                            name: Template {
-                                delimiter: Some('"'),
-                                elements: vec![
-                                    TemplateElement::String {
-                                        value: "name".to_string(),
-                                        encoded: "name".to_string()
-                                    }
-                                ],
-                                source_info: SourceInfo::new(Pos::new(9, 4), Pos::new(9, 8))
-                            },
-                            space1: String::new(),
-                            space2: " ".to_string(),
-                            value: JsonValue::String(Template {
-                                delimiter: Some('"'),
-                                elements: vec![
-                                    TemplateElement::String {
-                                        value: "Han Solo".to_string(),
-                                        encoded: "Han Solo".to_string()
-                                    }
-                                ],
-                                source_info: SourceInfo::new(Pos::new(9, 12), Pos::new(9, 20))
-                            }),
-                            space3: "\n".to_string()
-                        }]
-                    },
-                    whitespace: Whitespace {
-                        value: "\n".to_string(),
-                        source_info: SourceInfo::new(Pos::new(10, 2), Pos::new(11, 1))
-                    }
-                })
-            }), attributes: vec![]}
+                kind: MultilineStringKind::GraphQl(GraphQl {
+                    value: Template::new(
+                        None,
+                        vec![TemplateElement::String {
+                            value: "query Human($name: String!) {\n  human(name: $name) {\n    name\n    height(unit: FOOT)\n}\n\n".to_string(),
+                            source:
+                            "query Human($name: String!) {\n  human(name: $name) {\n    name\n    height(unit: FOOT)\n}\n\n".to_source()
+                        }],
+                        SourceInfo::new(Pos::new(2, 1), Pos::new(8, 1)),
+                    ),
+                    variables: Some(GraphQlVariables {
+                        space: Whitespace {
+                            value: " ".to_string(),
+                            source_info: SourceInfo::new(Pos::new(8, 10), Pos::new(8, 11)),
+                        },
+                        value: JsonValue::Object {
+                            space0: "\n  ".to_string(),
+                            elements: vec![JsonObjectElement {
+                                space0: String::new(),
+                                name: Template::new(
+                                    Some('"'),
+                                    vec![
+                                        TemplateElement::String {
+                                            value: "name".to_string(),
+                                            source: "name".to_source()
+                                        }
+                                    ],
+                                    SourceInfo::new(Pos::new(9, 4), Pos::new(9, 8))
+                                ),
+                                space1: String::new(),
+                                space2: " ".to_string(),
+                                value: JsonValue::String(Template::new(
+                                    Some('"'),
+                                    vec![
+                                        TemplateElement::String {
+                                            value: "Han Solo".to_string(),
+                                            source: "Han Solo".to_source()
+                                        }
+                                    ],
+                                    SourceInfo::new(Pos::new(9, 12), Pos::new(9, 20))
+                                )),
+                                space3: "\n".to_string()
+                            }]
+                        },
+                        whitespace: Whitespace {
+                            value: "\n".to_string(),
+                            source_info: SourceInfo::new(Pos::new(10, 2), Pos::new(11, 1))
+                        }
+                    })
+                }),
+            }
         );
     }
 
