@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
  * limitations under the License.
  *
  */
+use super::placeholder;
 use crate::ast::{
     is_variable_reserved, BooleanOption, CountOption, DurationOption, EntryOption, NaturalOption,
     OptionKind, SourceInfo, VariableDefinition, VariableValue,
@@ -31,8 +32,6 @@ use crate::parser::{filename, filename_password, ParseError, ParseErrorKind, Par
 use crate::reader::Reader;
 use crate::typing::Count;
 
-use super::placeholder;
-
 /// Parse an option in an `[Options]` section.
 pub fn parse(reader: &mut Reader) -> ParseResult<EntryOption> {
     let line_terminators = optional_line_terminators(reader)?;
@@ -55,6 +54,7 @@ pub fn parse(reader: &mut Reader) -> ParseResult<EntryOption> {
         "connect-timeout" => option_connect_timeout(reader)?,
         "delay" => option_delay(reader)?,
         "insecure" => option_insecure(reader)?,
+        "header" => option_header(reader)?,
         "http1.0" => option_http_10(reader)?,
         "http1.1" => option_http_11(reader)?,
         "http2" => option_http_2(reader)?,
@@ -66,11 +66,13 @@ pub fn parse(reader: &mut Reader) -> ParseResult<EntryOption> {
         "location" => option_follow_location(reader)?,
         "location-trusted" => option_follow_location_trusted(reader)?,
         "max-redirs" => option_max_redirect(reader)?,
+        "max-time" => option_max_time(reader)?,
         "netrc" => option_netrc(reader)?,
         "netrc-file" => option_netrc_file(reader)?,
         "netrc-optional" => option_netrc_optional(reader)?,
         "output" => option_output(reader)?,
         "path-as-is" => option_path_as_is(reader)?,
+        "pinnedpubkey" => option_pinned_pub_key(reader)?,
         "proxy" => option_proxy(reader)?,
         "repeat" => option_repeat(reader)?,
         "resolve" => option_resolve(reader)?,
@@ -147,6 +149,11 @@ fn option_follow_location_trusted(reader: &mut Reader) -> ParseResult<OptionKind
     Ok(OptionKind::FollowLocationTrusted(value))
 }
 
+fn option_header(reader: &mut Reader) -> ParseResult<OptionKind> {
+    let value = unquoted_template(reader)?;
+    Ok(OptionKind::Header(value))
+}
+
 fn option_http_10(reader: &mut Reader) -> ParseResult<OptionKind> {
     let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::Http10(value))
@@ -197,6 +204,11 @@ fn option_max_redirect(reader: &mut Reader) -> ParseResult<OptionKind> {
     Ok(OptionKind::MaxRedirect(value))
 }
 
+fn option_max_time(reader: &mut Reader) -> ParseResult<OptionKind> {
+    let value = duration_option(reader)?;
+    Ok(OptionKind::MaxTime(value))
+}
+
 fn option_netrc(reader: &mut Reader) -> ParseResult<OptionKind> {
     let value = non_recover(boolean_option, reader)?;
     Ok(OptionKind::NetRc(value))
@@ -222,6 +234,11 @@ fn option_path_as_is(reader: &mut Reader) -> ParseResult<OptionKind> {
     Ok(OptionKind::PathAsIs(value))
 }
 
+fn option_pinned_pub_key(reader: &mut Reader) -> ParseResult<OptionKind> {
+    let value = unquoted_template(reader)?;
+    Ok(OptionKind::PinnedPublicKey(value))
+}
+
 fn option_proxy(reader: &mut Reader) -> ParseResult<OptionKind> {
     let value = unquoted_template(reader)?;
     Ok(OptionKind::Proxy(value))
@@ -280,10 +297,10 @@ fn option_very_verbose(reader: &mut Reader) -> ParseResult<OptionKind> {
 fn count(reader: &mut Reader) -> ParseResult<Count> {
     let start = reader.cursor();
     let value = non_recover(integer, reader)?;
-    if value == -1 {
+    if value.as_i64() == -1 {
         Ok(Count::Infinite)
-    } else if value >= 0 {
-        Ok(Count::Finite(value as usize))
+    } else if value.as_i64() >= 0 {
+        Ok(Count::Finite(value.as_i64() as usize))
     } else {
         let kind = ParseErrorKind::Expecting {
             value: "Expecting a count value".to_string(),
@@ -439,8 +456,9 @@ fn variable_value(reader: &mut Reader) -> ParseResult<VariableValue> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::ast::{LineTerminator, Number, Template, TemplateElement, Whitespace};
+    use crate::ast::{LineTerminator, Number, Template, TemplateElement, Whitespace, I64};
     use crate::reader::Pos;
+    use crate::typing::ToSource;
 
     #[test]
     fn test_option_insecure() {
@@ -547,20 +565,20 @@ mod tests {
                         end: Pos { line: 1, column: 9 },
                     },
                 },
-                kind: OptionKind::CaCertificate(Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
+                kind: OptionKind::CaCertificate(Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "/home/foo/cert.pem".to_string(),
-                        encoded: "/home/foo/cert.pem".to_string()
+                        source: "/home/foo/cert.pem".to_source()
                     }],
-                    source_info: SourceInfo {
+                    SourceInfo {
                         start: Pos { line: 1, column: 9 },
                         end: Pos {
                             line: 1,
                             column: 27,
                         },
-                    },
-                }),
+                    }
+                )),
                 line_terminator0: LineTerminator {
                     space0: Whitespace {
                         value: String::new(),
@@ -607,20 +625,20 @@ mod tests {
 
         assert_eq!(
             option_cert(&mut reader).unwrap(),
-            OptionKind::ClientCert(Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            OptionKind::ClientCert(Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "/etc/client-cert.pem".to_string(),
-                    encoded: "/etc/client-cert.pem".to_string()
+                    source: "/etc/client-cert.pem".to_source()
                 }],
-                source_info: SourceInfo {
+                SourceInfo {
                     start: Pos { line: 1, column: 1 },
                     end: Pos {
                         line: 1,
                         column: 21,
                     },
                 },
-            }),
+            )),
         );
     }
 
@@ -654,7 +672,7 @@ mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 3)),
                 },
-                value: VariableValue::Number(Number::Integer(1)),
+                value: VariableValue::Number(Number::Integer(I64::new(1, "1".to_source()))),
             }
         );
     }
@@ -673,38 +691,32 @@ mod tests {
         let mut reader = Reader::new("1");
         assert_eq!(
             variable_value(&mut reader).unwrap(),
-            VariableValue::Number(Number::Integer(1))
+            VariableValue::Number(Number::Integer(I64::new(1, "1".to_source())))
         );
 
         let mut reader = Reader::new("toto");
         assert_eq!(
             variable_value(&mut reader).unwrap(),
-            VariableValue::String(Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            VariableValue::String(Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "toto".to_string(),
-                    encoded: "toto".to_string(),
+                    source: "toto".to_source(),
                 }],
-                source_info: SourceInfo {
-                    start: Pos { line: 1, column: 1 },
-                    end: Pos { line: 1, column: 5 },
-                },
-            })
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 5)),
+            ))
         );
         let mut reader = Reader::new("\"123\"");
         assert_eq!(
             variable_value(&mut reader).unwrap(),
-            VariableValue::String(Template {
-                delimiter: Some('"'),
-                elements: vec![TemplateElement::String {
+            VariableValue::String(Template::new(
+                Some('"'),
+                vec![TemplateElement::String {
                     value: "123".to_string(),
-                    encoded: "123".to_string(),
+                    source: "123".to_source(),
                 }],
-                source_info: SourceInfo {
-                    start: Pos { line: 1, column: 1 },
-                    end: Pos { line: 1, column: 6 },
-                },
-            })
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 6))
+            ))
         );
     }
 }
