@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,12 +17,11 @@
  */
 use crate::ast::VersionValue::VersionAny;
 use crate::ast::{
-    Body, Entry, HurlFile, Method, Request, Response, SourceInfo, Status, StatusValue, Version,
-    VersionValue,
+    Body, Entry, HurlFile, Method, Request, Response, Section, SourceInfo, Status, StatusValue,
+    Version, VersionValue,
 };
 use crate::combinator::{optional, zero_or_more};
 use crate::parser::bytes::bytes;
-use crate::parser::number::natural;
 use crate::parser::primitives::{
     eof, key_value, line_terminator, one_or_more_spaces, optional_line_terminators, try_literal,
     zero_or_more_spaces,
@@ -55,39 +54,27 @@ fn request(reader: &mut Reader) -> ParseResult<Request> {
     let start = reader.cursor();
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
-    let m = method(reader)?;
+    let method = method(reader)?;
     let space1 = one_or_more_spaces(reader)?;
     let url = unquoted_template(reader)?;
     let line_terminator0 = line_terminator(reader)?;
     let headers = zero_or_more(key_value, reader)?;
     let sections = request_sections(reader)?;
-    let b = optional(body, reader)?;
+    let body = optional(body, reader)?;
     let source_info = SourceInfo::new(start.pos, reader.cursor().pos);
 
-    // Check duplicated section
-    let mut section_names = vec![];
-    for section in sections.clone() {
-        if section_names.contains(&section.name().to_string()) {
-            return Err(ParseError::new(
-                section.source_info.start,
-                false,
-                ParseErrorKind::DuplicateSection,
-            ));
-        } else {
-            section_names.push(section.name().to_string());
-        }
-    }
+    check_duplicated_sections(&sections)?;
 
     Ok(Request {
         line_terminators,
         space0,
-        method: m,
+        method,
         space1,
         url,
         line_terminator0,
         headers,
         sections,
-        body: b,
+        body,
         source_info,
     })
 }
@@ -96,27 +83,47 @@ fn response(reader: &mut Reader) -> ParseResult<Response> {
     let start = reader.cursor();
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
-    let _version = version(reader)?;
+    let version = version(reader)?;
     let space1 = one_or_more_spaces(reader)?;
-    let _status = status(reader)?;
+    let status = status(reader)?;
     let line_terminator0 = line_terminator(reader)?;
     let headers = zero_or_more(key_value, reader)?;
     let sections = response_sections(reader)?;
-    let b = optional(body, reader)?;
+    let body = optional(body, reader)?;
+    let source_info = SourceInfo::new(start.pos, reader.cursor().pos);
+
+    check_duplicated_sections(&sections)?;
+
     Ok(Response {
         line_terminators,
         space0,
-        version: _version,
+        version,
         space1,
-        status: _status,
+        status,
         line_terminator0,
         headers,
         sections,
-        body: b,
-        source_info: SourceInfo::new(start.pos, reader.cursor().pos),
+        body,
+        source_info,
     })
 }
 
+fn check_duplicated_sections(sections: &[Section]) -> Result<(), ParseError> {
+    let mut section_names = vec![];
+    for section in sections {
+        if section_names.contains(&section.identifier()) {
+            return Err(ParseError::new(
+                section.source_info.start,
+                false,
+                ParseErrorKind::DuplicateSection,
+            ));
+        } else {
+            section_names.push(section.identifier());
+        }
+    }
+    Ok(())
+}
+
 fn method(reader: &mut Reader) -> ParseResult<Method> {
     if reader.is_eof() {
         let kind = ParseErrorKind::Method {
@@ -130,7 +137,7 @@ fn method(reader: &mut Reader) -> ParseResult<Method> {
         let kind = ParseErrorKind::Method { name };
         Err(ParseError::new(start.pos, false, kind))
     } else {
-        Ok(Method(name))
+        Ok(Method::new(&name))
     }
 }
 
@@ -146,12 +153,11 @@ fn version(reader: &mut Reader) -> ParseResult<Version> {
                 ("/1.1", VersionValue::Version11),
                 ("/2", VersionValue::Version2),
                 ("/3", VersionValue::Version3),
-                ("/*", VersionValue::VersionAnyLegacy),
             ];
             for (s, value) in available_version.iter() {
                 if try_literal(s, reader).is_ok() {
                     return Ok(Version {
-                        value: value.clone(),
+                        value: *value,
                         source_info: SourceInfo::new(start.pos, reader.cursor().pos),
                     });
                 }
@@ -170,10 +176,24 @@ fn status(reader: &mut Reader) -> ParseResult<Status> {
     let start = reader.cursor();
     let value = match try_literal("*", reader) {
         Ok(_) => StatusValue::Any,
-        Err(_) => match natural(reader) {
-            Ok(value) => StatusValue::Specific(value),
-            Err(_) => return Err(ParseError::new(start.pos, false, ParseErrorKind::Status)),
-        },
+        Err(_) => {
+            if reader.is_eof() {
+                let kind = ParseErrorKind::Status;
+                return Err(ParseError::new(start.pos, false, kind));
+            }
+            let s = reader.read_while(|c| c.is_ascii_digit());
+            if s.is_empty() {
+                let kind = ParseErrorKind::Status;
+                return Err(ParseError::new(start.pos, false, kind));
+            }
+            match s.to_string().parse() {
+                Ok(value) => StatusValue::Specific(value),
+                Err(_) => {
+                    let kind = ParseErrorKind::Status;
+                    return Err(ParseError::new(start.pos, false, kind));
+                }
+            }
+        }
     };
     let end = reader.cursor();
     Ok(Status {
@@ -201,9 +221,10 @@ mod tests {
     use super::*;
     use crate::ast::{
         Bytes, Comment, JsonListElement, JsonValue, LineTerminator, MultilineString,
-        MultilineStringKind, Template, TemplateElement, Text, Whitespace,
+        MultilineStringKind, Template, TemplateElement, Whitespace,
     };
-    use crate::reader::Pos;
+    use crate::reader::{CharPos, Pos};
+    use crate::typing::ToSource;
 
     #[test]
     fn test_hurl_file() {
@@ -216,8 +237,8 @@ mod tests {
     fn test_entry() {
         let mut reader = Reader::new("GET http://google.fr");
         let e = entry(&mut reader).unwrap();
-        assert_eq!(e.request.method, Method("GET".to_string()));
-        assert_eq!(reader.cursor().index, 20);
+        assert_eq!(e.request.method, Method::new("GET"));
+        assert_eq!(reader.cursor().index, CharPos(20));
     }
 
     #[test]
@@ -225,26 +246,26 @@ mod tests {
         let mut reader = Reader::new("GET http://google.fr\nGET http://google.fr");
 
         let e = entry(&mut reader).unwrap();
-        assert_eq!(e.request.method, Method("GET".to_string()));
-        assert_eq!(reader.cursor().index, 21);
+        assert_eq!(e.request.method, Method::new("GET"));
+        assert_eq!(reader.cursor().index, CharPos(21));
         assert_eq!(reader.cursor().pos.line, 2);
 
         let e = entry(&mut reader).unwrap();
-        assert_eq!(e.request.method, Method("GET".to_string()));
-        assert_eq!(reader.cursor().index, 41);
+        assert_eq!(e.request.method, Method::new("GET"));
+        assert_eq!(reader.cursor().index, CharPos(41));
         assert_eq!(reader.cursor().pos.line, 2);
 
         let mut reader =
             Reader::new("GET http://google.fr # comment1\nGET http://google.fr # comment2");
 
         let e = entry(&mut reader).unwrap();
-        assert_eq!(e.request.method, Method("GET".to_string()));
-        assert_eq!(reader.cursor().index, 32);
+        assert_eq!(e.request.method, Method::new("GET"));
+        assert_eq!(reader.cursor().index, CharPos(32));
         assert_eq!(reader.cursor().pos.line, 2);
 
         let e = entry(&mut reader).unwrap();
-        assert_eq!(e.request.method, Method("GET".to_string()));
-        assert_eq!(reader.cursor().index, 63);
+        assert_eq!(e.request.method, Method::new("GET"));
+        assert_eq!(reader.cursor().index, CharPos(63));
         assert_eq!(reader.cursor().pos.line, 2);
     }
 
@@ -252,7 +273,7 @@ mod tests {
     fn test_entry_with_response() {
         let mut reader = Reader::new("GET http://google.fr\nHTTP/1.1 200");
         let e = entry(&mut reader).unwrap();
-        assert_eq!(e.request.method, Method("GET".to_string()));
+        assert_eq!(e.request.method, Method::new("GET"));
         assert_eq!(e.response.unwrap().status.value, StatusValue::Specific(200));
     }
 
@@ -265,19 +286,19 @@ mod tests {
                 value: String::new(),
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
             },
-            method: Method("GET".to_string()),
+            method: Method::new("GET"),
             space1: Whitespace {
                 value: " ".to_string(),
                 source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 5)),
             },
-            url: Template {
-                elements: vec![TemplateElement::String {
-                    value: String::from("http://google.fr"),
-                    encoded: String::from("http://google.fr"),
+            url: Template::new(
+                None,
+                vec![TemplateElement::String {
+                    value: "http://google.fr".to_string(),
+                    source: "http://google.fr".to_source(),
                 }],
-                delimiter: None,
-                source_info: SourceInfo::new(Pos::new(1, 5), Pos::new(1, 21)),
-            },
+                SourceInfo::new(Pos::new(1, 5), Pos::new(1, 21)),
+            ),
             line_terminator0: LineTerminator {
                 space0: Whitespace {
                     value: String::new(),
@@ -295,7 +316,7 @@ mod tests {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 21)),
         };
         assert_eq!(request(&mut reader).unwrap(), default_request);
-        assert_eq!(reader.cursor().index, 20);
+        assert_eq!(reader.cursor().index, CharPos(20));
 
         let mut reader = Reader::new("GET  http://google.fr # comment");
         let default_request = Request {
@@ -304,19 +325,19 @@ mod tests {
                 value: String::new(),
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
             },
-            method: Method("GET".to_string()),
+            method: Method::new("GET"),
             space1: Whitespace {
                 value: "  ".to_string(),
                 source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 6)),
             },
-            url: Template {
-                elements: vec![TemplateElement::String {
-                    value: String::from("http://google.fr"),
-                    encoded: String::from("http://google.fr"),
+            url: Template::new(
+                None,
+                vec![TemplateElement::String {
+                    value: "http://google.fr".to_string(),
+                    source: "http://google.fr".to_source(),
                 }],
-                delimiter: None,
-                source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 22)),
-            },
+                SourceInfo::new(Pos::new(1, 6), Pos::new(1, 22)),
+            ),
             line_terminator0: LineTerminator {
                 space0: Whitespace {
                     value: " ".to_string(),
@@ -337,14 +358,14 @@ mod tests {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 32)),
         };
         assert_eq!(request(&mut reader).unwrap(), default_request);
-        assert_eq!(reader.cursor().index, 31);
+        assert_eq!(reader.cursor().index, CharPos(31));
 
         let mut reader = Reader::new("GET http://google.fr\nGET http://google.fr");
         let r = request(&mut reader).unwrap();
-        assert_eq!(r.method, Method("GET".to_string()));
-        assert_eq!(reader.cursor().index, 21);
+        assert_eq!(r.method, Method::new("GET"));
+        assert_eq!(reader.cursor().index, CharPos(21));
         let r = request(&mut reader).unwrap();
-        assert_eq!(r.method, Method("GET".to_string()));
+        assert_eq!(r.method, Method::new("GET"));
     }
 
     #[test]
@@ -364,25 +385,23 @@ mod tests {
                     source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 1)),
                 },
                 value: Bytes::MultilineString(MultilineString {
-                    kind: MultilineStringKind::Text(Text {
-                        space: Whitespace {
-                            value: String::new(),
-                            source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(2, 4)),
-                        },
-                        newline: Whitespace {
-                            source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(3, 1)),
-                            value: "\n".to_string(),
-                        },
-                        value: Template {
-                            elements: vec![TemplateElement::String {
-                                value: String::from("Hello World!\n"),
-                                encoded: String::from("Hello World!\n"),
-                            }],
-                            delimiter: None,
-                            source_info: SourceInfo::new(Pos::new(3, 1), Pos::new(4, 1)),
-                        },
-                    }),
-                    attributes: vec![]
+                    attributes: vec![],
+                    space: Whitespace {
+                        value: String::new(),
+                        source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(2, 4)),
+                    },
+                    newline: Whitespace {
+                        source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(3, 1)),
+                        value: "\n".to_string(),
+                    },
+                    kind: MultilineStringKind::Text(Template::new(
+                        None,
+                        vec![TemplateElement::String {
+                            value: "Hello World!\n".to_string(),
+                            source: "Hello World!\n".to_source(),
+                        }],
+                        SourceInfo::new(Pos::new(3, 1), Pos::new(4, 1)),
+                    )),
                 }),
                 line_terminator0: LineTerminator {
                     space0: Whitespace {
@@ -403,7 +422,7 @@ mod tests {
     fn test_request_post_json() {
         let mut reader = Reader::new("POST http://localhost:8000/post-json-array\n[1,2,3]");
         let r = request(&mut reader).unwrap();
-        assert_eq!(r.method, Method("POST".to_string()));
+        assert_eq!(r.method, Method::new("POST"));
         assert_eq!(
             r.body.unwrap().value,
             Bytes::Json(JsonValue::List {
@@ -430,22 +449,22 @@ mod tests {
 
         let mut reader = Reader::new("POST http://localhost:8000/post-json-string\n\"Hello\"");
         let r = request(&mut reader).unwrap();
-        assert_eq!(r.method, Method("POST".to_string()));
+        assert_eq!(r.method, Method::new("POST"));
         assert_eq!(
             r.body.unwrap().value,
-            Bytes::Json(JsonValue::String(Template {
-                delimiter: Some('"'),
-                elements: vec![TemplateElement::String {
+            Bytes::Json(JsonValue::String(Template::new(
+                Some('"'),
+                vec![TemplateElement::String {
                     value: "Hello".to_string(),
-                    encoded: "Hello".to_string(),
+                    source: "Hello".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(2, 2), Pos::new(2, 7)),
-            }))
+                SourceInfo::new(Pos::new(2, 2), Pos::new(2, 7)),
+            )))
         );
 
         let mut reader = Reader::new("POST http://localhost:8000/post-json-number\n100");
         let r = request(&mut reader).unwrap();
-        assert_eq!(r.method, Method("POST".to_string()));
+        assert_eq!(r.method, Method::new("POST"));
         assert_eq!(
             r.body.unwrap().value,
             Bytes::Json(JsonValue::Number("100".to_string()))
@@ -474,31 +493,31 @@ mod tests {
         let mut reader = Reader::new("xxx ");
         let error = method(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(reader.cursor().index, 3);
+        assert_eq!(reader.cursor().index, CharPos(3));
 
         let mut reader = Reader::new("");
         let error = method(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(reader.cursor().index, 0);
+        assert_eq!(reader.cursor().index, CharPos(0));
 
         let mut reader = Reader::new("GET ");
-        assert_eq!(method(&mut reader).unwrap(), Method("GET".to_string()));
-        assert_eq!(reader.cursor().index, 3);
+        assert_eq!(method(&mut reader).unwrap(), Method::new("GET"));
+        assert_eq!(reader.cursor().index, CharPos(3));
 
         let mut reader = Reader::new("CUSTOM");
-        assert_eq!(method(&mut reader).unwrap(), Method("CUSTOM".to_string()));
-        assert_eq!(reader.cursor().index, 6);
+        assert_eq!(method(&mut reader).unwrap(), Method::new("CUSTOM"));
+        assert_eq!(reader.cursor().index, CharPos(6));
     }
 
     #[test]
     fn test_version() {
         let mut reader = Reader::new("HTTP 200");
         assert_eq!(version(&mut reader).unwrap().value, VersionAny);
-        assert_eq!(reader.cursor().index, 4);
+        assert_eq!(reader.cursor().index, CharPos(4));
 
         let mut reader = Reader::new("HTTP\t200");
         assert_eq!(version(&mut reader).unwrap().value, VersionAny);
-        assert_eq!(reader.cursor().index, 4);
+        assert_eq!(reader.cursor().index, CharPos(4));
 
         let mut reader = Reader::new("HTTP/1.1 200");
         assert_eq!(version(&mut reader).unwrap().value, VersionValue::Version11);
@@ -551,7 +570,7 @@ mod tests {
                 ],
             })
         );
-        assert_eq!(reader.cursor().index, 8);
+        assert_eq!(reader.cursor().index, CharPos(8));
 
         let mut reader = Reader::new("{}");
         let b = body(&mut reader).unwrap();
@@ -563,7 +582,7 @@ mod tests {
                 elements: vec![],
             })
         );
-        assert_eq!(reader.cursor().index, 2);
+        assert_eq!(reader.cursor().index, CharPos(2));
 
         let mut reader = Reader::new("# comment\n {} # comment\nxxx");
         let b = body(&mut reader).unwrap();
@@ -575,7 +594,7 @@ mod tests {
                 elements: vec![],
             })
         );
-        assert_eq!(reader.cursor().index, 24);
+        assert_eq!(reader.cursor().index, CharPos(24));
 
         let mut reader = Reader::new("{x");
         let error = body(&mut reader).err().unwrap();
