@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,9 +20,7 @@ use crate::ast::{
 };
 use crate::combinator::choice;
 use crate::parser::predicate_value::predicate_value;
-use crate::parser::primitives::{
-    one_or_more_spaces, try_literal, try_literals, zero_or_more_spaces,
-};
+use crate::parser::primitives::{one_or_more_spaces, try_literal, zero_or_more_spaces};
 use crate::parser::{ParseError, ParseErrorKind, ParseResult};
 use crate::reader::Reader;
 
@@ -94,6 +92,9 @@ fn predicate_func_value(reader: &mut Reader) -> ParseResult<PredicateFuncValue>
             exist_predicate,
             is_empty_predicate,
             is_number_predicate,
+            is_ipv4_predicate,
+            is_ipv6_predicate,
+            is_uuid_predicate,
         ],
         reader,
     ) {
@@ -122,59 +123,26 @@ impl PredicateValue {
 }
 
 fn equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
-    let operator = try_literals("equals", "==", reader)? == "==";
-    if !operator {
-        eprintln!("'equals' predicate is now deprecated. Use '==' instead");
-    }
-    let space0 = if operator {
-        zero_or_more_spaces(reader)?
-    } else {
-        one_or_more_spaces(reader)?
-    };
+    try_literal("==", reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
     let value = predicate_value(reader)?;
-    Ok(PredicateFuncValue::Equal {
-        space0,
-        value,
-        operator,
-    })
+    Ok(PredicateFuncValue::Equal { space0, value })
 }
 
 fn not_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
-    let operator = try_literals("notEquals", "!=", reader)? == "!=";
-    if !operator {
-        eprintln!("'notEquals' predicate is now deprecated. Use '!=' instead");
-    }
-    let space0 = if operator {
-        zero_or_more_spaces(reader)?
-    } else {
-        one_or_more_spaces(reader)?
-    };
+    try_literal("!=", reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
     let value = predicate_value(reader)?;
-    Ok(PredicateFuncValue::NotEqual {
-        space0,
-        value,
-        operator,
-    })
+    Ok(PredicateFuncValue::NotEqual { space0, value })
 }
 
 fn greater_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
-    let operator = try_literals("greaterThan", ">", reader)? == ">";
-    if !operator {
-        eprintln!("'greaterThan' predicate is now deprecated. Use '>' instead");
-    }
-    let space0 = if operator {
-        zero_or_more_spaces(reader)?
-    } else {
-        one_or_more_spaces(reader)?
-    };
+    try_literal(">", reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
     let start = reader.cursor();
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
-        Ok(PredicateFuncValue::GreaterThan {
-            space0,
-            value,
-            operator,
-        })
+        Ok(PredicateFuncValue::GreaterThan { space0, value })
     } else {
         Err(ParseError::new(
             start.pos,
@@ -185,23 +153,12 @@ fn greater_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
 }
 
 fn greater_or_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
-    let operator = try_literals("greaterThanOrEquals", ">=", reader)? == ">=";
-    if !operator {
-        eprintln!("'greaterThanOrEquals' predicate is now deprecated. Use '>=' instead");
-    }
-    let space0 = if operator {
-        zero_or_more_spaces(reader)?
-    } else {
-        one_or_more_spaces(reader)?
-    };
+    try_literal(">=", reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
     let start = reader.cursor();
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
-        Ok(PredicateFuncValue::GreaterThanOrEqual {
-            space0,
-            value,
-            operator,
-        })
+        Ok(PredicateFuncValue::GreaterThanOrEqual { space0, value })
     } else {
         Err(ParseError::new(
             start.pos,
@@ -212,23 +169,12 @@ fn greater_or_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncV
 }
 
 fn less_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
-    let operator = try_literals("lessThan", "<", reader)? == "<";
-    if !operator {
-        eprintln!("'lessThan' predicate is now deprecated. Use '<' instead");
-    }
-    let space0 = if operator {
-        zero_or_more_spaces(reader)?
-    } else {
-        one_or_more_spaces(reader)?
-    };
+    try_literal("<", reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
     let start = reader.cursor();
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
-        Ok(PredicateFuncValue::LessThan {
-            space0,
-            value,
-            operator,
-        })
+        Ok(PredicateFuncValue::LessThan { space0, value })
     } else {
         Err(ParseError::new(
             start.pos,
@@ -239,23 +185,12 @@ fn less_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
 }
 
 fn less_or_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
-    let operator = try_literals("lessThanOrEquals", "<=", reader)? == "<=";
-    if !operator {
-        eprintln!("'lessThanOrEquals' predicate is now deprecated. Use '<=' instead");
-    }
-    let space0 = if operator {
-        zero_or_more_spaces(reader)?
-    } else {
-        one_or_more_spaces(reader)?
-    };
+    try_literal("<=", reader)?;
+    let space0 = zero_or_more_spaces(reader)?;
     let start = reader.cursor();
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
-        Ok(PredicateFuncValue::LessThanOrEqual {
-            space0,
-            value,
-            operator,
-        })
+        Ok(PredicateFuncValue::LessThanOrEqual { space0, value })
     } else {
         Err(ParseError::new(
             start.pos,
@@ -298,15 +233,7 @@ fn end_with_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
 fn contain_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("contains", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let save = reader.cursor();
     let value = predicate_value(reader)?;
-    if !value.is_string() && !value.is_bytearray() {
-        return Err(ParseError::new(
-            save.pos,
-            false,
-            ParseErrorKind::PredicateValue,
-        ));
-    }
     Ok(PredicateFuncValue::Contain { space0, value })
 }
 
@@ -382,13 +309,29 @@ fn is_number_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     Ok(PredicateFuncValue::IsNumber)
 }
 
+fn is_ipv4_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
+    try_literal("isIpv4", reader)?;
+    Ok(PredicateFuncValue::IsIpv4)
+}
+
+fn is_ipv6_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
+    try_literal("isIpv6", reader)?;
+    Ok(PredicateFuncValue::IsIpv6)
+}
+
+fn is_uuid_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
+    try_literal("isUuid", reader)?;
+    Ok(PredicateFuncValue::IsUuid)
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
     use crate::ast::{
-        Expr, ExprKind, Float, Number, Placeholder, Template, TemplateElement, Variable,
+        Expr, ExprKind, Float, Number, Placeholder, Template, TemplateElement, Variable, I64,
     };
     use crate::reader::Pos;
+    use crate::typing::ToSource;
 
     #[test]
     fn test_predicate_not() {
@@ -438,7 +381,6 @@ mod tests {
                             source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 8)),
                         },
                         value: PredicateValue::Bool(true),
-                        operator: true,
                     },
                 },
             }
@@ -465,8 +407,6 @@ mod tests {
                     value: String::from("  "),
                     source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 5)),
                 },
-
-                operator: true,
             }
         );
 
@@ -474,15 +414,11 @@ mod tests {
         assert_eq!(
             equal_predicate(&mut reader).unwrap(),
             PredicateFuncValue::Equal {
-                value: PredicateValue::Number(Number::Float(Float {
-                    value: 1.1,
-                    encoded: "1.1".to_string(),
-                })),
+                value: PredicateValue::Number(Number::Float(Float::new(1.1, "1.1".to_source()))),
                 space0: Whitespace {
                     value: String::from(" "),
                     source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4)),
                 },
-                operator: true,
             }
         );
 
@@ -490,12 +426,11 @@ mod tests {
         assert_eq!(
             equal_predicate(&mut reader).unwrap(),
             PredicateFuncValue::Equal {
-                value: PredicateValue::Number(Number::Integer(2)),
+                value: PredicateValue::Number(Number::Integer(I64::new(2, "2".to_source()))),
                 space0: Whitespace {
                     value: String::from(" "),
                     source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4)),
                 },
-                operator: true,
             },
         );
 
@@ -503,19 +438,18 @@ mod tests {
         assert_eq!(
             equal_predicate(&mut reader).unwrap(),
             PredicateFuncValue::Equal {
-                value: PredicateValue::String(Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                value: PredicateValue::String(Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "Bob".to_string(),
-                        encoded: "Bob".to_string(),
+                        source: "Bob".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 4), Pos::new(1, 9)),
-                }),
+                    SourceInfo::new(Pos::new(1, 4), Pos::new(1, 9)),
+                )),
                 space0: Whitespace {
                     value: String::from(" "),
                     source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4)),
                 },
-                operator: true,
             }
         );
     }
@@ -547,7 +481,6 @@ mod tests {
                     value: String::from(" "),
                     source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 4)),
                 },
-                operator: true,
             }
         );
     }
