@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@ use crate::combinator::{one_or_more, optional, recover, zero_or_more};
 use crate::parser::string::unquoted_template;
 use crate::parser::{base64, filename, key_string, ParseError, ParseErrorKind, ParseResult};
 use crate::reader::Reader;
+use crate::typing::{SourceString, ToSource};
 
 pub fn space(reader: &mut Reader) -> ParseResult<Whitespace> {
     let start = reader.cursor();
@@ -167,27 +168,6 @@ pub fn try_literal(s: &str, reader: &mut Reader) -> ParseResult<()> {
     }
 }
 
-/// Returns the literal string
-pub fn try_literals(s1: &str, s2: &str, reader: &mut Reader) -> ParseResult<String> {
-    let start = reader.cursor();
-    match literal(s1, reader) {
-        Ok(_) => Ok(s1.to_string()),
-        Err(_) => {
-            reader.seek(start);
-            match literal(s2, reader) {
-                Ok(_) => Ok(s2.to_string()),
-                Err(_) => {
-                    reader.seek(start);
-                    let kind = ParseErrorKind::Expecting {
-                        value: format!("<{s1}> or <{s2}>"),
-                    };
-                    Err(ParseError::new(start.pos, true, kind))
-                }
-            }
-        }
-    }
-}
-
 pub fn newline(reader: &mut Reader) -> ParseResult<Whitespace> {
     let start = reader.cursor();
     match try_literal("\r\n", reader) {
@@ -261,14 +241,14 @@ pub fn hex(reader: &mut Reader) -> ParseResult<Hex> {
             ParseErrorKind::OddNumberOfHexDigits,
         ));
     }
-    let encoded = reader.read_from(start.index);
+    let source = reader.read_from(start.index).to_source();
     let space1 = zero_or_more_spaces(reader)?;
     literal(";", reader)?;
 
     Ok(Hex {
         space0,
         value,
-        encoded,
+        source,
         space1,
     })
 }
@@ -276,7 +256,9 @@ pub fn hex(reader: &mut Reader) -> ParseResult<Hex> {
 pub fn regex(reader: &mut Reader) -> ParseResult<Regex> {
     try_literal("/", reader)?;
     let start = reader.cursor();
-    let mut s = String::new();
+    let mut unescaped = String::new();
+    let mut source = SourceString::new();
+    source.push('/');
 
     // Hurl escaping /
     // in order to avoid terminating the regex
@@ -285,27 +267,30 @@ pub fn regex(reader: &mut Reader) -> ParseResult<Regex> {
     // Other escaped sequences such as \* are part of the regex expression
     // They are not part of the syntax of Hurl itself.
     loop {
-        match reader.read() {
-            None => {
-                let kind = ParseErrorKind::RegexExpr {
-                    message: "unexpected end of file".to_string(),
-                };
-                return Err(ParseError::new(reader.cursor().pos, false, kind));
-            }
-            Some('/') => break,
-            Some('\\') => {
+        let c = reader.read();
+        let Some(c) = c else {
+            let kind = ParseErrorKind::RegexExpr {
+                message: "unexpected end of file".to_string(),
+            };
+            return Err(ParseError::new(reader.cursor().pos, false, kind));
+        };
+        source.push(c);
+        match c {
+            '/' => break,
+            '\\' => {
                 if let Some('/') = reader.peek() {
                     reader.read();
-                    s.push('/');
+                    source.push('/');
+                    unescaped.push('/');
                 } else {
-                    s.push('\\');
+                    unescaped.push('\\');
                 }
             }
-            Some(c) => s.push(c),
+            c => unescaped.push(c),
         }
     }
-    match regex::Regex::new(s.as_str()) {
-        Ok(inner) => Ok(Regex { inner }),
+    match regex::Regex::new(unescaped.as_str()) {
+        Ok(inner) => Ok(Regex { inner, source }),
         Err(e) => {
             let message = match e {
                 regex::Error::Syntax(s) => {
@@ -382,13 +367,13 @@ pub(crate) fn base64(reader: &mut Reader) -> ParseResult<Base64> {
     let value = base64::parse(reader);
     let count = reader.cursor().index - save_state.index;
     reader.seek(save_state);
-    let encoded = reader.read_n(count);
+    let source = reader.read_n(count).to_source();
     let space1 = zero_or_more_spaces(reader)?;
     literal(";", reader)?;
     Ok(Base64 {
         space0,
         value,
-        encoded,
+        source,
         space1,
     })
 }
@@ -441,14 +426,15 @@ pub fn hex_digit(reader: &mut Reader) -> ParseResult<u32> {
 mod tests {
     use super::*;
     use crate::ast::{Expr, ExprKind, Placeholder, Template, TemplateElement, Variable};
-    use crate::reader::Pos;
+    use crate::reader::{CharPos, Pos};
+    use crate::typing::ToSource;
 
     #[test]
     fn test_space() {
         let mut reader = Reader::new("x");
         let error = space(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
-        assert_eq!(reader.cursor().index, 1);
+        assert_eq!(reader.cursor().index, CharPos(1));
 
         let mut reader = Reader::new("  ");
         assert_eq!(
@@ -458,7 +444,7 @@ mod tests {
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 2)),
             }),
         );
-        assert_eq!(reader.cursor().index, 1);
+        assert_eq!(reader.cursor().index, CharPos(1));
     }
 
     #[test]
@@ -487,7 +473,7 @@ mod tests {
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 3)),
             })
         );
-        assert_eq!(reader.cursor().index, 2);
+        assert_eq!(reader.cursor().index, CharPos(2));
 
         let mut reader = Reader::new("xxx");
         assert_eq!(
@@ -497,7 +483,7 @@ mod tests {
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
             })
         );
-        assert_eq!(reader.cursor().index, 0);
+        assert_eq!(reader.cursor().index, CharPos(0));
 
         let mut reader = Reader::new(" xxx");
         assert_eq!(
@@ -507,7 +493,7 @@ mod tests {
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 2)),
             })
         );
-        assert_eq!(reader.cursor().index, 1);
+        assert_eq!(reader.cursor().index, CharPos(1));
     }
 
     #[test]
@@ -538,7 +524,7 @@ mod tests {
                 source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 10)),
             })
         );
-        assert_eq!(reader.cursor().index, 9);
+        assert_eq!(reader.cursor().index, CharPos(9));
 
         let mut reader = Reader::new("xxx");
         let error = comment(&mut reader).err().unwrap();
@@ -550,7 +536,7 @@ mod tests {
     fn test_literal() {
         let mut reader = Reader::new("hello");
         assert_eq!(literal("hello", &mut reader), Ok(()));
-        assert_eq!(reader.cursor().index, 5);
+        assert_eq!(reader.cursor().index, CharPos(5));
 
         let mut reader = Reader::new("");
         let error = literal("hello", &mut reader).err().unwrap();
@@ -561,7 +547,7 @@ mod tests {
                 value: String::from("hello")
             }
         );
-        assert_eq!(reader.cursor().index, 0);
+        assert_eq!(reader.cursor().index, CharPos(0));
 
         let mut reader = Reader::new("hi");
         let error = literal("hello", &mut reader).err().unwrap();
@@ -572,7 +558,7 @@ mod tests {
                 value: String::from("hello")
             }
         );
-        assert_eq!(reader.cursor().index, 2);
+        assert_eq!(reader.cursor().index, CharPos(2));
 
         let mut reader = Reader::new("he");
         let error = literal("hello", &mut reader).err().unwrap();
@@ -583,7 +569,7 @@ mod tests {
                 value: String::from("hello")
             }
         );
-        assert_eq!(reader.cursor().index, 2);
+        assert_eq!(reader.cursor().index, CharPos(2));
     }
 
     #[test]
@@ -609,14 +595,14 @@ mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                 },
-                key: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
+                key: Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "message".to_string(),
-                        encoded: "message".to_string(),
+                        source: "message".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 8)),
-                },
+                    SourceInfo::new(Pos::new(1, 1), Pos::new(1, 8)),
+                ),
                 space1: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 8)),
@@ -625,12 +611,12 @@ mod tests {
                     value: " ".to_string(),
                     source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
-                value: Template {
-                    delimiter: None,
-                    elements: vec![
+                value: Template::new(
+                    None,
+                    vec![
                         TemplateElement::String {
                             value: "hello ".to_string(),
-                            encoded: "hello ".to_string(),
+                            source: "hello ".to_source(),
                         },
                         TemplateElement::Placeholder(Placeholder {
                             space0: Whitespace {
@@ -651,11 +637,11 @@ mod tests {
                         }),
                         TemplateElement::String {
                             value: "!".to_string(),
-                            encoded: "!".to_string(),
+                            source: "!".to_source(),
                         },
                     ],
-                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 25)),
-                },
+                    SourceInfo::new(Pos::new(1, 10), Pos::new(1, 25))
+                ),
                 line_terminator0: LineTerminator {
                     space0: Whitespace {
                         value: " ".to_string(),
@@ -685,9 +671,9 @@ mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 1)),
                 },
-                key: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::Placeholder(Placeholder {
+                key: Template::new(
+                    None,
+                    vec![TemplateElement::Placeholder(Placeholder {
                         space0: Whitespace {
                             value: String::new(),
                             source_info: SourceInfo::new(Pos::new(1, 3), Pos::new(1, 3)),
@@ -704,8 +690,8 @@ mod tests {
                             source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 6)),
                         },
                     })],
-                    source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 8)),
-                },
+                    SourceInfo::new(Pos::new(1, 1), Pos::new(1, 8)),
+                ),
                 space1: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 8)),
@@ -714,14 +700,14 @@ mod tests {
                     value: " ".to_string(),
                     source_info: SourceInfo::new(Pos::new(1, 9), Pos::new(1, 10)),
                 },
-                value: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
+                value: Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "value".to_string(),
-                        encoded: "value".to_string(),
+                        source: "value".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 10), Pos::new(1, 15)),
-                },
+                    SourceInfo::new(Pos::new(1, 10), Pos::new(1, 15)),
+                ),
                 line_terminator0: LineTerminator {
                     space0: Whitespace {
                         value: String::new(),
@@ -735,7 +721,7 @@ mod tests {
                 },
             }
         );
-        assert_eq!(reader.cursor().index, 14);
+        assert_eq!(reader.cursor().index, CharPos(14));
     }
 
     #[test]
@@ -744,13 +730,13 @@ mod tests {
         let error = key_value(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 6 });
         assert!(error.recoverable);
-        assert_eq!(reader.cursor().index, 5); // does not reset cursor
+        assert_eq!(reader.cursor().index, CharPos(5)); // does not reset cursor
 
         let mut reader = Reader::new("GET ®http://google.fr");
         let error = key_value(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 5 });
         assert!(error.recoverable);
-        assert_eq!(reader.cursor().index, 5); // does not reset cursor
+        assert_eq!(reader.cursor().index, CharPos(5)); // does not reset cursor
     }
 
     #[test]
@@ -802,7 +788,7 @@ mod tests {
                     source_info: SourceInfo::new(Pos::new(1, 5), Pos::new(1, 6)),
                 },
                 value: vec![255],
-                encoded: "ff".to_string(),
+                source: "ff".to_source(),
                 space1: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 8)),
@@ -819,7 +805,7 @@ mod tests {
                     source_info: SourceInfo::new(Pos::new(1, 5), Pos::new(1, 5)),
                 },
                 value: vec![1, 2, 3],
-                encoded: "010203".to_string(),
+                source: "010203".to_source(),
                 space1: Whitespace {
                     value: " ".to_string(),
                     source_info: SourceInfo::new(Pos::new(1, 11), Pos::new(1, 12)),
@@ -842,7 +828,8 @@ mod tests {
         assert_eq!(
             regex(&mut reader).unwrap(),
             Regex {
-                inner: regex::Regex::new(r#"a{3}"#).unwrap()
+                inner: regex::Regex::new(r#"a{3}"#).unwrap(),
+                source: r#"/a{3}/"#.to_source(),
             }
         );
 
@@ -850,7 +837,8 @@ mod tests {
         assert_eq!(
             regex(&mut reader).unwrap(),
             Regex {
-                inner: regex::Regex::new(r#"a/b"#).unwrap()
+                inner: regex::Regex::new(r#"a/b"#).unwrap(),
+                source: r"/a\/b/".to_source(),
             }
         );
 
@@ -858,7 +846,8 @@ mod tests {
         assert_eq!(
             regex(&mut reader).unwrap(),
             Regex {
-                inner: regex::Regex::new(r"a\.b").unwrap()
+                inner: regex::Regex::new(r"a\.b").unwrap(),
+                source: r"/a\.b/".to_source(),
             }
         );
 
@@ -866,7 +855,8 @@ mod tests {
         assert_eq!(
             regex(&mut reader).unwrap(),
             Regex {
-                inner: regex::Regex::new(r"\d{4}-\d{2}-\d{2}").unwrap()
+                inner: regex::Regex::new(r"\d{4}-\d{2}-\d{2}").unwrap(),
+                source: r"/\d{4}-\d{2}-\d{2}/".to_source(),
             }
         );
     }
@@ -911,14 +901,14 @@ mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 6)),
                 },
-                filename: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
-                        value: String::from("data.xml"),
-                        encoded: String::from("data.xml"),
+                filename: Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "data.xml".to_string(),
+                        source: "data.xml".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 14)),
-                },
+                    SourceInfo::new(Pos::new(1, 6), Pos::new(1, 14))
+                ),
                 space1: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 14), Pos::new(1, 14)),
@@ -934,14 +924,14 @@ mod tests {
                     value: String::from(" "),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)),
                 },
-                filename: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
-                        value: String::from("filename1"),
-                        encoded: String::from("filename1"),
+                filename: Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "filename1".to_string(),
+                        source: "filename1".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 16)),
-                },
+                    SourceInfo::new(Pos::new(1, 7), Pos::new(1, 16)),
+                ),
                 space1: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 16), Pos::new(1, 16)),
@@ -957,14 +947,14 @@ mod tests {
                     value: String::from(" "),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)),
                 },
-                filename: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
-                        value: String::from("tmp/filename1"),
-                        encoded: String::from("tmp/filename1"),
+                filename: Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "tmp/filename1".to_string(),
+                        source: "tmp/filename1".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 20)),
-                },
+                    SourceInfo::new(Pos::new(1, 7), Pos::new(1, 20))
+                ),
                 space1: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 20), Pos::new(1, 20)),
@@ -980,14 +970,14 @@ mod tests {
                     value: String::from(" "),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 7)),
                 },
-                filename: Template {
-                    elements: vec![TemplateElement::String {
-                        value: String::from("tmp/filename with spaces.txt"),
-                        encoded: String::from("tmp/filename\\ with\\ spaces.txt"),
+                filename: Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "tmp/filename with spaces.txt".to_string(),
+                        source: "tmp/filename\\ with\\ spaces.txt".to_source(),
                     }],
-                    delimiter: None,
-                    source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 37)),
-                },
+                    SourceInfo::new(Pos::new(1, 7), Pos::new(1, 37))
+                ),
                 space1: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 37), Pos::new(1, 37)),
@@ -1049,13 +1039,13 @@ mod tests {
                     source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 10)),
                 },
                 value: vec![77, 97],
-                encoded: String::from("T WE="),
+                source: "T WE=".to_source(),
                 space1: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(1, 15)),
                 },
             }
         );
-        assert_eq!(reader.cursor().index, 15);
+        assert_eq!(reader.cursor().index, CharPos(15));
     }
 }
