@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@ use crate::ast::{Placeholder, SourceInfo, TemplateElement};
 use crate::parser::primitives::zero_or_more_spaces;
 use crate::parser::{error, expr, ParseResult};
 use crate::reader::{Pos, Reader};
+use crate::typing::SourceString;
 
 pub struct EncodedString {
     pub source_info: SourceInfo,
@@ -36,7 +37,7 @@ pub fn templatize(encoded_string: EncodedString) -> ParseResult<Vec<TemplateElem
     let mut elements = vec![];
 
     let mut value = String::new();
-    let mut encoded = String::new();
+    let mut source = SourceString::new();
     let mut state = State::String;
     let mut expression_start = None;
 
@@ -47,24 +48,24 @@ pub fn templatize(encoded_string: EncodedString) -> ParseResult<Vec<TemplateElem
                     state = State::FirstOpenBracket;
                 } else {
                     value.push(c);
-                    encoded.push_str(&s.clone());
+                    source.push_str(&s);
                 }
             }
 
             State::FirstOpenBracket => {
                 if s.as_str() == "{" {
                     if !value.is_empty() {
-                        elements.push(TemplateElement::String { value, encoded });
+                        elements.push(TemplateElement::String { value, source });
                         value = String::new();
-                        encoded = String::new();
+                        source = SourceString::new();
                     }
                     state = State::Template;
                 } else {
                     value.push('{');
-                    encoded.push('{');
+                    source.push('{');
 
                     value.push(c);
-                    encoded.push_str(&s.clone());
+                    source.push_str(&s);
                     state = State::String;
                 }
             }
@@ -77,13 +78,13 @@ pub fn templatize(encoded_string: EncodedString) -> ParseResult<Vec<TemplateElem
                     state = State::FirstCloseBracket;
                 } else {
                     value.push(c);
-                    encoded.push_str(&s.clone());
+                    source.push_str(&s);
                 }
             }
 
             State::FirstCloseBracket => {
                 if s.as_str() == "}" {
-                    let mut reader = Reader::with_pos(encoded.as_str(), expression_start.unwrap());
+                    let mut reader = Reader::with_pos(source.as_str(), expression_start.unwrap());
                     let space0 = zero_or_more_spaces(&mut reader)?;
                     let expr = expr::parse(&mut reader)?;
                     let space1 = zero_or_more_spaces(&mut reader)?;
@@ -94,14 +95,14 @@ pub fn templatize(encoded_string: EncodedString) -> ParseResult<Vec<TemplateElem
                     };
                     elements.push(TemplateElement::Placeholder(placeholder));
                     value = String::new();
-                    encoded = String::new();
+                    source = SourceString::new();
                     expression_start = None;
                     state = State::String;
                 } else {
                     value.push('}');
                     value.push(c);
-                    encoded.push('}');
-                    encoded.push_str(&s.clone());
+                    source.push('}');
+                    source.push_str(&s);
                 }
             }
         }
@@ -111,7 +112,7 @@ pub fn templatize(encoded_string: EncodedString) -> ParseResult<Vec<TemplateElem
         State::String => {}
         State::FirstOpenBracket => {
             value.push('{');
-            encoded.push('{');
+            source.push('{');
         }
         State::Template | State::FirstCloseBracket => {
             let kind = error::ParseErrorKind::Expecting {
@@ -126,7 +127,7 @@ pub fn templatize(encoded_string: EncodedString) -> ParseResult<Vec<TemplateElem
     }
 
     if !value.is_empty() {
-        elements.push(TemplateElement::String { value, encoded });
+        elements.push(TemplateElement::String { value, source });
     }
     Ok(elements)
 }
@@ -136,6 +137,7 @@ mod tests {
 
     use super::*;
     use crate::ast::{Expr, ExprKind, Placeholder, Variable, Whitespace};
+    use crate::typing::ToSource;
 
     #[test]
     fn test_templatize_empty_string() {
@@ -227,7 +229,7 @@ mod tests {
             vec![
                 TemplateElement::String {
                     value: "Hi ".to_string(),
-                    encoded: "Hi\\u0020".to_string(),
+                    source: "Hi\\u0020".to_source(),
                 },
                 TemplateElement::Placeholder(Placeholder {
                     space0: Whitespace {
@@ -248,7 +250,7 @@ mod tests {
                 }),
                 TemplateElement::String {
                     value: "!".to_string(),
-                    encoded: "!".to_string(),
+                    source: "!".to_source(),
                 },
             ]
         );
@@ -319,15 +321,15 @@ mod tests {
         let encoded_string = EncodedString {
             source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 4)),
             chars: vec![
-                ('{', "\\{".to_string(), Pos { line: 1, column: 1 }),
-                ('{', "\\{".to_string(), Pos { line: 1, column: 2 }),
+                ('{', "\\{".to_string(), Pos::new(1, 1)),
+                ('{', "\\{".to_string(), Pos::new(1, 2)),
             ],
         };
         assert_eq!(
             templatize(encoded_string).unwrap(),
             vec![TemplateElement::String {
                 value: "{{".to_string(),
-                encoded: "\\{\\{".to_string(),
+                source: "\\{\\{".to_source(),
             },]
         );
     }
