@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,8 +17,11 @@
  */
 //! Hurl common types.
 use core::fmt;
+use std::borrow::Borrow;
 use std::str::FromStr;
 
+use crate::ast::U64;
+
 /// Represents a count operation, either finite or infinite.
 #[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub enum Count {
@@ -26,27 +29,6 @@ pub enum Count {
     Infinite,
 }
 
-/// Represent a duration
-#[derive(Copy, Clone, Debug, PartialEq, Eq)]
-pub struct Duration {
-    pub value: u64,
-    pub unit: Option<DurationUnit>,
-}
-
-impl Duration {
-    pub fn new(value: u64, unit: Option<DurationUnit>) -> Duration {
-        Duration { value, unit }
-    }
-}
-
-/// Represents a duration unit
-#[derive(Copy, Clone, Debug, PartialEq, Eq)]
-pub enum DurationUnit {
-    MilliSecond,
-    Second,
-    Minute,
-}
-
 impl fmt::Display for Count {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
@@ -60,6 +42,19 @@ impl fmt::Display for Count {
     }
 }
 
+/// Represent a duration
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub struct Duration {
+    pub value: U64,
+    pub unit: Option<DurationUnit>,
+}
+
+impl Duration {
+    pub fn new(value: U64, unit: Option<DurationUnit>) -> Duration {
+        Duration { value, unit }
+    }
+}
+
 impl fmt::Display for Duration {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let unit = if let Some(value) = self.unit {
@@ -71,6 +66,14 @@ impl fmt::Display for Duration {
     }
 }
 
+/// Represents a duration unit
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+pub enum DurationUnit {
+    MilliSecond,
+    Second,
+    Minute,
+}
+
 impl fmt::Display for DurationUnit {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
@@ -103,3 +106,74 @@ impl fmt::Display for BytesPerSec {
         write!(f, "{}", self.0)
     }
 }
+
+/// Represents a source string, in Hurl file format.
+///
+/// For instance, with this Hurl file:
+///
+/// ```hurl
+/// GET https://example.org/api
+/// HTTP 200
+/// [Asserts]
+/// jsonpath "$.slideshow.title" == "A beautiful \u{2708}!"
+/// ```
+///
+/// `"A beautiful \u{2708}!"` is the source string, using a [Hurl Unicode literal](https://hurl.dev/docs/hurl-file.html#special-characters-in-strings).
+/// It's string value is `A beautiful ✈!`.
+#[derive(Clone, Debug, Default, PartialEq, Eq)]
+pub struct SourceString(String);
+
+impl SourceString {
+    /// Creates a new empty [`SourceString`].
+    pub fn new() -> Self {
+        SourceString(String::new())
+    }
+
+    /// Appends a given string slice onto the end of this [`SourceString`].
+    pub fn push_str(&mut self, string: &str) {
+        self.0.push_str(string);
+    }
+
+    /// Appends the given char to the end of this [`SourceString`].
+    pub fn push(&mut self, c: char) {
+        self.0.push(c);
+    }
+
+    /// Extracts a string slice containing the entire [`SourceString`].
+    pub fn as_str(&self) -> &str {
+        self.0.as_str()
+    }
+
+    /// Returns `true` if this [`SourceString`] starts with the char `c`.
+    pub fn starts_with(&self, c: char) -> bool {
+        self.0.starts_with(c)
+    }
+}
+
+pub trait ToSource {
+    fn to_source(&self) -> SourceString;
+}
+
+impl ToSource for String {
+    fn to_source(&self) -> SourceString {
+        SourceString(self.clone())
+    }
+}
+
+impl ToSource for &str {
+    fn to_source(&self) -> SourceString {
+        SourceString(self.to_string())
+    }
+}
+
+impl fmt::Display for SourceString {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{}", self.0)
+    }
+}
+
+impl Borrow<str> for SourceString {
+    fn borrow(&self) -> &str {
+        &self.0
+    }
+}
