@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -58,6 +58,10 @@ pub fn url(arg_matches: &ArgMatches) -> String {
     }
 }
 
+pub fn cookies(arg_matches: &ArgMatches) -> Vec<String> {
+    get_strings(arg_matches, "cookies").unwrap_or_default()
+}
+
 pub fn headers(arg_matches: &ArgMatches) -> Vec<String> {
     let mut headers = get_strings(arg_matches, "headers").unwrap_or_default();
     if !has_content_type(&headers) {
@@ -88,6 +92,9 @@ pub fn options(arg_matches: &ArgMatches) -> Vec<HurlOption> {
     if let Some(value) = get::<i32>(arg_matches, "retry") {
         options.push(HurlOption::new("retry", value.to_string().as_str()));
     }
+    if has_flag(arg_matches, "verbose") {
+        options.push(HurlOption::new("verbose", "true"));
+    }
     options
 }
 
