@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,18 +15,19 @@
  * limitations under the License.
  *
  */
-use crate::format::serialize_json::JValue;
 use base64::engine::general_purpose;
 use base64::Engine;
 use hurl_core::ast::{
     Assert, Base64, Body, BooleanOption, Bytes, Capture, CertificateAttributeName, Comment, Cookie,
-    CountOption, DurationOption, Entry, EntryOption, File, FileParam, Filter, FilterValue, Header,
-    Hex, HurlFile, JsonListElement, JsonValue, KeyValue, MultilineString, MultilineStringKind,
+    CountOption, DurationOption, Entry, EntryOption, File, FilenameParam, Filter, FilterValue, Hex,
+    HurlFile, JsonListElement, JsonValue, KeyValue, MultilineString, MultilineStringKind,
     MultipartParam, NaturalOption, OptionKind, Placeholder, Predicate, PredicateFuncValue,
     PredicateValue, Query, QueryValue, Regex, RegexValue, Request, Response, StatusValue,
     VersionValue,
 };
-use hurl_core::typing::{Count, Duration};
+use hurl_core::typing::{Count, Duration, ToSource};
+
+use crate::format::serialize_json::JValue;
 
 pub fn format(hurl_file: &HurlFile) -> String {
     hurl_file.to_json().format()
@@ -139,7 +140,7 @@ impl ToJson for Response {
     }
 }
 
-fn add_headers(attributes: &mut Vec<(String, JValue)>, headers: &[Header]) {
+fn add_headers(attributes: &mut Vec<(String, JValue)>, headers: &[KeyValue]) {
     if !headers.is_empty() {
         let headers = JValue::List(headers.iter().map(|h| h.to_json()).collect());
         attributes.push(("headers".to_string(), headers));
@@ -252,7 +253,6 @@ fn get_json_version(version_value: &VersionValue) -> Option<String> {
         VersionValue::Version2 => Some("HTTP/2".to_string()),
         VersionValue::Version3 => Some("HTTP/3".to_string()),
         VersionValue::VersionAny => None,
-        VersionValue::VersionAnyLegacy => None,
     }
 }
 
@@ -270,12 +270,12 @@ impl ToJson for MultipartParam {
     fn to_json(&self) -> JValue {
         match self {
             MultipartParam::Param(param) => param.to_json(),
-            MultipartParam::FileParam(param) => param.to_json(),
+            MultipartParam::FilenameParam(param) => param.to_json(),
         }
     }
 }
 
-impl ToJson for FileParam {
+impl ToJson for FilenameParam {
     fn to_json(&self) -> JValue {
         let mut attributes = vec![
             ("name".to_string(), JValue::String(self.key.to_string())),
@@ -284,8 +284,11 @@ impl ToJson for FileParam {
                 JValue::String(self.value.filename.to_string()),
             ),
         ];
-        if let Some(content_type) = self.value.content_type.clone() {
-            attributes.push(("content_type".to_string(), JValue::String(content_type)));
+        if let Some(content_type) = &self.value.content_type {
+            attributes.push((
+                "content_type".to_string(),
+                JValue::String(content_type.to_string()),
+            ));
         }
         JValue::Object(attributes)
     }
@@ -314,6 +317,7 @@ impl ToJson for EntryOption {
             OptionKind::Delay(value) => value.to_json(),
             OptionKind::FollowLocation(value) => value.to_json(),
             OptionKind::FollowLocationTrusted(value) => value.to_json(),
+            OptionKind::Header(value) => JValue::String(value.to_string()),
             OptionKind::Http10(value) => value.to_json(),
             OptionKind::Http11(value) => value.to_json(),
             OptionKind::Http2(value) => value.to_json(),
@@ -323,11 +327,13 @@ impl ToJson for EntryOption {
             OptionKind::IpV6(value) => value.to_json(),
             OptionKind::LimitRate(value) => value.to_json(),
             OptionKind::MaxRedirect(value) => value.to_json(),
+            OptionKind::MaxTime(value) => value.to_json(),
             OptionKind::NetRc(value) => value.to_json(),
             OptionKind::NetRcFile(filename) => JValue::String(filename.to_string()),
             OptionKind::NetRcOptional(value) => value.to_json(),
             OptionKind::Output(filename) => JValue::String(filename.to_string()),
             OptionKind::PathAsIs(value) => value.to_json(),
+            OptionKind::PinnedPublicKey(value) => JValue::String(value.to_string()),
             OptionKind::Proxy(value) => JValue::String(value.to_string()),
             OptionKind::Repeat(value) => value.to_json(),
             OptionKind::Resolve(value) => JValue::String(value.to_string()),
@@ -337,7 +343,7 @@ impl ToJson for EntryOption {
             OptionKind::UnixSocket(value) => JValue::String(value.to_string()),
             OptionKind::User(value) => JValue::String(value.to_string()),
             OptionKind::Variable(value) => {
-                JValue::String(format!("{}={}", value.name, value.value))
+                JValue::String(format!("{}={}", value.name, value.value.to_source()))
             }
             OptionKind::Verbose(value) => value.to_json(),
             OptionKind::VeryVerbose(value) => value.to_json(),
@@ -349,14 +355,14 @@ impl ToJson for EntryOption {
         let attributes = if let JValue::Object(mut attributes) = value {
             attributes.push((
                 "name".to_string(),
-                JValue::String(self.kind.name().to_string()),
+                JValue::String(self.kind.identifier().to_string()),
             ));
             attributes
         } else {
             vec![
                 (
                     "name".to_string(),
-                    JValue::String(self.kind.name().to_string()),
+                    JValue::String(self.kind.identifier().to_string()),
                 ),
                 ("value".to_string(), value),
             ]
@@ -424,6 +430,9 @@ impl ToJson for Capture {
             let filters = JValue::List(self.filters.iter().map(|(_, f)| f.to_json()).collect());
             attributes.push(("filters".to_string(), filters));
         }
+        if self.redacted {
+            attributes.push(("redact".to_string(), JValue::Boolean(true)));
+        }
         JValue::Object(attributes)
     }
 }
@@ -449,62 +458,35 @@ impl ToJson for Query {
 
 fn query_value_attributes(query_value: &QueryValue) -> Vec<(String, JValue)> {
     let mut attributes = vec![];
+    let att_type = JValue::String(query_value.identifier().to_string());
+    attributes.push(("type".to_string(), att_type));
+
     match query_value {
-        QueryValue::Status => {
-            attributes.push(("type".to_string(), JValue::String("status".to_string())));
-        }
-        QueryValue::Url => {
-            attributes.push(("type".to_string(), JValue::String("url".to_string())));
-        }
-        QueryValue::Body => {
-            attributes.push(("type".to_string(), JValue::String("body".to_string())));
-        }
         QueryValue::Jsonpath { expr, .. } => {
-            attributes.push(("type".to_string(), JValue::String("jsonpath".to_string())));
             attributes.push(("expr".to_string(), JValue::String(expr.to_string())));
         }
         QueryValue::Header { name, .. } => {
-            attributes.push(("type".to_string(), JValue::String("header".to_string())));
             attributes.push(("name".to_string(), JValue::String(name.to_string())));
         }
         QueryValue::Cookie { expr, .. } => {
-            attributes.push(("type".to_string(), JValue::String("cookie".to_string())));
             attributes.push(("expr".to_string(), JValue::String(expr.to_string())));
         }
         QueryValue::Xpath { expr, .. } => {
-            attributes.push(("type".to_string(), JValue::String("xpath".to_string())));
             attributes.push(("expr".to_string(), JValue::String(expr.to_string())));
         }
         QueryValue::Regex { value, .. } => {
-            attributes.push(("type".to_string(), JValue::String("regex".to_string())));
             attributes.push(("expr".to_string(), value.to_json()));
         }
         QueryValue::Variable { name, .. } => {
-            attributes.push(("type".to_string(), JValue::String("variable".to_string())));
             attributes.push(("name".to_string(), JValue::String(name.to_string())));
         }
-        QueryValue::Duration => {
-            attributes.push(("type".to_string(), JValue::String("duration".to_string())));
-        }
-        QueryValue::Bytes => {
-            attributes.push(("type".to_string(), JValue::String("bytes".to_string())));
-        }
-        QueryValue::Sha256 => {
-            attributes.push(("type".to_string(), JValue::String("sha256".to_string())));
-        }
-        QueryValue::Md5 => {
-            attributes.push(("type".to_string(), JValue::String("md5".to_string())));
-        }
         QueryValue::Certificate {
             attribute_name: field,
             ..
         } => {
-            attributes.push((
-                "type".to_string(),
-                JValue::String("certificate".to_string()),
-            ));
             attributes.push(("expr".to_string(), field.to_json()));
         }
+        _ => {}
     };
     attributes
 }
@@ -530,14 +512,7 @@ impl ToJson for Regex {
 
 impl ToJson for CertificateAttributeName {
     fn to_json(&self) -> JValue {
-        let value = match self {
-            CertificateAttributeName::Subject => "Subject",
-            CertificateAttributeName::Issuer => "Issuer",
-            CertificateAttributeName::StartDate => "Start-Date",
-            CertificateAttributeName::ExpireDate => "Expire-Date",
-            CertificateAttributeName::SerialNumber => "Serial-Number",
-        };
-        JValue::String(value.to_string())
+        JValue::String(self.identifier().to_string())
     }
 }
 
@@ -547,96 +522,60 @@ impl ToJson for Predicate {
         if self.not {
             attributes.push(("not".to_string(), JValue::Boolean(true)));
         }
-        match self.predicate_func.value.clone() {
-            PredicateFuncValue::Equal { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("equal".to_string())));
-                add_predicate_value(&mut attributes, value);
-            }
+        let identifier = self.predicate_func.value.identifier();
+        attributes.push(("type".to_string(), JValue::String(identifier.to_string())));
+
+        match &self.predicate_func.value {
+            PredicateFuncValue::Equal { value, .. } => add_predicate_value(&mut attributes, value),
             PredicateFuncValue::NotEqual { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("not-equal".to_string())));
                 add_predicate_value(&mut attributes, value);
             }
             PredicateFuncValue::GreaterThan { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("greater".to_string())));
                 add_predicate_value(&mut attributes, value);
             }
             PredicateFuncValue::GreaterThanOrEqual { value, .. } => {
-                attributes.push((
-                    "type".to_string(),
-                    JValue::String("greater-or-equal".to_string()),
-                ));
                 add_predicate_value(&mut attributes, value);
             }
             PredicateFuncValue::LessThan { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("less".to_string())));
                 add_predicate_value(&mut attributes, value);
             }
             PredicateFuncValue::LessThanOrEqual { value, .. } => {
-                attributes.push((
-                    "type".to_string(),
-                    JValue::String("less-or-equal".to_string()),
-                ));
                 add_predicate_value(&mut attributes, value);
             }
             PredicateFuncValue::StartWith { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("start-with".to_string())));
                 add_predicate_value(&mut attributes, value);
             }
             PredicateFuncValue::EndWith { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("end-with".to_string())));
                 add_predicate_value(&mut attributes, value);
             }
             PredicateFuncValue::Contain { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("contain".to_string())));
                 add_predicate_value(&mut attributes, value);
             }
             PredicateFuncValue::Include { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("include".to_string())));
                 add_predicate_value(&mut attributes, value);
             }
             PredicateFuncValue::Match { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("match".to_string())));
                 add_predicate_value(&mut attributes, value);
             }
-            PredicateFuncValue::IsInteger => {
-                attributes.push(("type".to_string(), JValue::String("isInteger".to_string())));
-            }
-            PredicateFuncValue::IsFloat => {
-                attributes.push(("type".to_string(), JValue::String("isFloat".to_string())));
-            }
-            PredicateFuncValue::IsBoolean => {
-                attributes.push(("type".to_string(), JValue::String("isBoolean".to_string())));
-            }
-            PredicateFuncValue::IsString => {
-                attributes.push(("type".to_string(), JValue::String("isString".to_string())));
-            }
-            PredicateFuncValue::IsCollection => {
-                attributes.push((
-                    "type".to_string(),
-                    JValue::String("isCollection".to_string()),
-                ));
-            }
-            PredicateFuncValue::IsDate => {
-                attributes.push(("type".to_string(), JValue::String("isDate".to_string())));
-            }
-            PredicateFuncValue::IsIsoDate => {
-                attributes.push(("type".to_string(), JValue::String("isIsoDate".to_string())));
-            }
-            PredicateFuncValue::Exist => {
-                attributes.push(("type".to_string(), JValue::String("exist".to_string())));
-            }
-            PredicateFuncValue::IsEmpty => {
-                attributes.push(("type".to_string(), JValue::String("isEmpty".to_string())));
-            }
-            PredicateFuncValue::IsNumber => {
-                attributes.push(("type".to_string(), JValue::String("isNumber".to_string())));
-            }
+            PredicateFuncValue::IsInteger
+            | PredicateFuncValue::IsFloat
+            | PredicateFuncValue::IsBoolean
+            | PredicateFuncValue::IsString
+            | PredicateFuncValue::IsCollection
+            | PredicateFuncValue::IsDate
+            | PredicateFuncValue::IsIsoDate
+            | PredicateFuncValue::Exist
+            | PredicateFuncValue::IsEmpty
+            | PredicateFuncValue::IsNumber
+            | PredicateFuncValue::IsIpv4
+            | PredicateFuncValue::IsIpv6
+            | PredicateFuncValue::IsUuid => {}
         }
         JValue::Object(attributes)
     }
 }
 
-fn add_predicate_value(attributes: &mut Vec<(String, JValue)>, predicate_value: PredicateValue) {
+fn add_predicate_value(attributes: &mut Vec<(String, JValue)>, predicate_value: &PredicateValue) {
     let (value, encoding) = json_predicate_value(predicate_value);
     attributes.push(("value".to_string(), value));
     if let Some(encoding) = encoding {
@@ -644,20 +583,20 @@ fn add_predicate_value(attributes: &mut Vec<(String, JValue)>, predicate_value:
     }
 }
 
-fn json_predicate_value(predicate_value: PredicateValue) -> (JValue, Option<String>) {
+fn json_predicate_value(predicate_value: &PredicateValue) -> (JValue, Option<String>) {
     match predicate_value {
         PredicateValue::String(value) => (JValue::String(value.to_string()), None),
         PredicateValue::MultilineString(value) => (JValue::String(value.value().to_string()), None),
-        PredicateValue::Bool(value) => (JValue::Boolean(value), None),
+        PredicateValue::Bool(value) => (JValue::Boolean(*value), None),
         PredicateValue::Null => (JValue::Null, None),
         PredicateValue::Number(value) => (JValue::Number(value.to_string()), None),
         PredicateValue::File(value) => (value.to_json(), None),
         PredicateValue::Hex(value) => {
-            let base64_string = general_purpose::STANDARD.encode(value.value);
+            let base64_string = general_purpose::STANDARD.encode(value.value.clone());
             (JValue::String(base64_string), Some("base64".to_string()))
         }
         PredicateValue::Base64(value) => {
-            let base64_string = general_purpose::STANDARD.encode(value.value);
+            let base64_string = general_purpose::STANDARD.encode(value.value.clone());
             (JValue::String(base64_string), Some("base64".to_string()))
         }
         PredicateValue::Placeholder(value) => (JValue::String(value.to_string()), None),
@@ -703,49 +642,24 @@ impl ToJson for Filter {
 impl ToJson for FilterValue {
     fn to_json(&self) -> JValue {
         let mut attributes = vec![];
+        let att_name = "type".to_string();
+        let att_value = JValue::String(self.identifier().to_string());
+        attributes.push((att_name, att_value));
+
         match self {
-            FilterValue::Count => {
-                attributes.push(("type".to_string(), JValue::String("count".to_string())));
-            }
-            FilterValue::DaysAfterNow => {
-                attributes.push((
-                    "type".to_string(),
-                    JValue::String("daysAfterNow".to_string()),
-                ));
-            }
-            FilterValue::DaysBeforeNow => {
-                attributes.push((
-                    "type".to_string(),
-                    JValue::String("daysBeforeNow".to_string()),
-                ));
-            }
             FilterValue::Decode { encoding, .. } => {
-                attributes.push(("type".to_string(), JValue::String("decode".to_string())));
                 attributes.push(("encoding".to_string(), JValue::String(encoding.to_string())));
             }
             FilterValue::Format { fmt, .. } => {
-                attributes.push(("type".to_string(), JValue::String("format".to_string())));
                 attributes.push(("fmt".to_string(), JValue::String(fmt.to_string())));
             }
             FilterValue::JsonPath { expr, .. } => {
-                attributes.push(("type".to_string(), JValue::String("jsonpath".to_string())));
                 attributes.push(("expr".to_string(), JValue::String(expr.to_string())));
             }
             FilterValue::Nth { n, .. } => {
-                attributes.push(("type".to_string(), JValue::String("nth".to_string())));
                 attributes.push(("n".to_string(), JValue::Number(n.to_string())));
             }
-            FilterValue::HtmlEscape => {
-                attributes.push(("type".to_string(), JValue::String("htmlEscape".to_string())));
-            }
-            FilterValue::HtmlUnescape => {
-                attributes.push((
-                    "type".to_string(),
-                    JValue::String("htmlUnescape".to_string()),
-                ));
-            }
             FilterValue::Regex { value, .. } => {
-                attributes.push(("type".to_string(), JValue::String("regex".to_string())));
                 attributes.push(("expr".to_string(), value.to_json()));
             }
             FilterValue::Replace {
@@ -753,37 +667,37 @@ impl ToJson for FilterValue {
                 new_value,
                 ..
             } => {
-                attributes.push(("type".to_string(), JValue::String("replace".to_string())));
-                attributes.push(("old_value".to_string(), old_value.to_json()));
+                attributes.push((
+                    "old_value".to_string(),
+                    JValue::String(old_value.to_string()),
+                ));
                 attributes.push((
                     "new_value".to_string(),
                     JValue::String(new_value.to_string()),
                 ));
             }
-            FilterValue::UrlEncode => {
-                attributes.push(("type".to_string(), JValue::String("urlEncode".to_string())));
-            }
-            FilterValue::UrlDecode => {
-                attributes.push(("type".to_string(), JValue::String("urlDecode".to_string())));
+            FilterValue::ReplaceRegex {
+                pattern, new_value, ..
+            } => {
+                attributes.push(("pattern".to_string(), pattern.to_json()));
+                attributes.push((
+                    "new_value".to_string(),
+                    JValue::String(new_value.to_string()),
+                ));
             }
             FilterValue::Split { sep, .. } => {
-                attributes.push(("type".to_string(), JValue::String("split".to_string())));
                 attributes.push(("sep".to_string(), JValue::String(sep.to_string())));
             }
             FilterValue::ToDate { fmt, .. } => {
-                attributes.push(("type".to_string(), JValue::String("toDate".to_string())));
                 attributes.push(("fmt".to_string(), JValue::String(fmt.to_string())));
             }
-            FilterValue::ToFloat => {
-                attributes.push(("type".to_string(), JValue::String("toFloat".to_string())));
-            }
-            FilterValue::ToInt => {
-                attributes.push(("type".to_string(), JValue::String("toInt".to_string())));
+            FilterValue::UrlQueryParam { param, .. } => {
+                attributes.push(("param".to_string(), JValue::String(param.to_string())));
             }
             FilterValue::XPath { expr, .. } => {
-                attributes.push(("type".to_string(), JValue::String("xpath".to_string())));
                 attributes.push(("expr".to_string(), JValue::String(expr.to_string())));
             }
+            _ => {}
         }
         JValue::Object(attributes)
     }
@@ -791,7 +705,7 @@ impl ToJson for FilterValue {
 
 impl ToJson for Placeholder {
     fn to_json(&self) -> JValue {
-        JValue::String(format!("{{{{{}}}}}", self))
+        JValue::String(format!("{{{{{self}}}}}"))
     }
 }
 
@@ -813,9 +727,10 @@ impl ToJson for NaturalOption {
 pub mod tests {
     use hurl_core::ast::{
         LineTerminator, Method, Number, PredicateFunc, SourceInfo, Status, Template,
-        TemplateElement, Version, Whitespace,
+        TemplateElement, Version, Whitespace, I64,
     };
     use hurl_core::reader::Pos;
+    use hurl_core::typing::ToSource;
 
     use super::*;
 
@@ -840,38 +755,38 @@ pub mod tests {
             Request {
                 line_terminators: vec![],
                 space0: whitespace(),
-                method: Method("GET".to_string()),
+                method: Method::new("GET"),
                 space1: whitespace(),
-                url: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
+                url: Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "http://example.com".to_string(),
-                        encoded: "not_used".to_string(),
+                        source: "not_used".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                },
+                    SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                ),
                 line_terminator0: line_terminator(),
                 headers: vec![KeyValue {
                     line_terminators: vec![],
                     space0: whitespace(),
-                    key: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
+                    key: Template::new(
+                        None,
+                        vec![TemplateElement::String {
                             value: "Foo".to_string(),
-                            encoded: "unused".to_string(),
+                            source: "unused".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                    },
+                        SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0))
+                    ),
                     space1: whitespace(),
                     space2: whitespace(),
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
+                    value: Template::new(
+                        None,
+                        vec![TemplateElement::String {
                             value: "Bar".to_string(),
-                            encoded: "unused".to_string(),
+                            source: "unused".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                    },
+                        SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0))
+                    ),
                     line_terminator0: line_terminator(),
                 }],
                 sections: vec![],
@@ -955,14 +870,14 @@ pub mod tests {
             source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             value: QueryValue::Header {
                 space0: whitespace(),
-                name: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
+                name: Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "Content-Length".to_string(),
-                        encoded: "10".to_string(),
+                        source: "Content-Length".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                },
+                    SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                ),
             },
         }
     }
@@ -971,18 +886,20 @@ pub mod tests {
         Capture {
             line_terminators: vec![],
             space0: whitespace(),
-            name: Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            name: Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "size".to_string(),
-                    encoded: "unused".to_string(),
+                    source: "unused".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-            },
+                SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+            ),
             space1: whitespace(),
             space2: whitespace(),
             query: header_query(),
             filters: vec![],
+            space3: whitespace(),
+            redacted: false,
             line_terminator0: line_terminator(),
         }
     }
@@ -1007,8 +924,10 @@ pub mod tests {
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
                 value: PredicateFuncValue::Equal {
                     space0: whitespace(),
-                    value: PredicateValue::Number(Number::Integer(value)),
-                    operator: false,
+                    value: PredicateValue::Number(Number::Integer(I64::new(
+                        value,
+                        value.to_string().to_source(),
+                    ))),
                 },
             },
         }
@@ -1053,7 +972,7 @@ pub mod tests {
         assert_eq!(
             equal_int_predicate(10).to_json(),
             JValue::Object(vec![
-                ("type".to_string(), JValue::String("equal".to_string())),
+                ("type".to_string(), JValue::String("==".to_string())),
                 ("value".to_string(), JValue::Number("10".to_string()))
             ]),
         );
@@ -1077,7 +996,7 @@ pub mod tests {
                 (
                     "predicate".to_string(),
                     JValue::Object(vec![
-                        ("type".to_string(), JValue::String("equal".to_string())),
+                        ("type".to_string(), JValue::String("==".to_string())),
                         ("value".to_string(), JValue::Number("10".to_string()))
                     ])
                 )
