@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,9 +17,7 @@
  */
 pub use self::json::format as format_json;
 pub use self::text::format as format_text;
-pub use self::token::{Token, Tokenizable};
 
 mod json;
 mod serialize_json;
 mod text;
-mod token;
